// Scale detection

/*
  HaniMandl Version W.0.3
  ------------------------
  Copyright (C) 2018-2024 by Marc Vasterling, Marc Wetzel, Clemens Gruber, Marc Junker, Andreas Holzhammer, Johannes Kuder, Jeremias Bruker, Roland Rust, Achim Pfaff, Jérémie Lehmann
            
  2018-05 Marc Vasterling    | initial version, 
                               publicizing in the Facebook group "Imkerei und Technik. Eigenbau",
                               Marc Vasterling: "my code can be used, changed and uploaded, as long by anyone as long as they dont stick their name on it."
  2018-06 Marc Vasterling    | version upgrade, 
                               publicized in the Facebook group
  2019-01 Marc Wetzel        | rebuilding and documentation, 
                               publicized in the Facebook group
  2019-02 Clemens Gruber     | code beautifying with small renaming of variables and functions
                               adaptation for Heltec WiFi Kit 32 (ESP32 onboard OLED) 
                               - pins changed at OLED intialization
                               - pins changed to avoid conflicts with hard wired OLED pins
  2019-02 Clemens Gruber     | activation of all internal pull-downs for all digital entries
  2019-02 Clemens Gruber     | "normal" pins to Vcc / GND changed to make cabling somewhat simpler and more agreable
  2020-05 Andreas Holzhammer | adaptations to the changed ;-( pin layout of Heltec version 2 
                               now sells as "New Wifi Kit 32" or "Wifi Kit 32 V2"
  2020-05 Marc Junker        | - extension from potentiometer to rotary encoder 
                               - all Serial.prints in #ifdef locked
                               - "jar" now as array with individual tare
                               - correction value and choice of fillquantity now settable by clicking and rolling rotary encoder
  2020-05 Andreas Holzhammer | - tare for jar to be filled automatically adapted (variable tare_jar)
                               - code also works with no scale
  2020-06 Andreas Holzhammer | - code option to use Heltec V1 or V2
                               - code option to use potentiometer or rotatry encoder
                               - tare for jar configurable
                               - opening angle for maximum opening and fine metering can be configured in the setup
                               - correction and jar size can be selected in automatic mode using rotary encoder
                               - preferences rest within setup menu
                               - weight flashes on full auto if jar not completely filled
                               - scale missing and scale not calibrated warnings added
                               - tare is only set >20g to prevent autostart when scale is empty
                               - scale zeroing at every startup to +-20g otherwise warning displayed
  2020-07 Andreas Holzhammer | Version 0.2.4
                               - SCALE_READS on set on 2? about 100ms faster than 3, but fluctuates around +-1g
                               - order of boot messages optimized so that only relevant warnings are issued
                               - implemented autocorrection
                               - LOGO! and credits (suggested by Johannes Kuder)
                               - stop button leaves setup undermenus (suggested by Johannes Kuder)
                               - preferences only saved when modified
  2020-07 Andreas Holzhammer | Version 0.2.5
                               - display of previous value in setup
                               - Overfill value in autocorrection settable
                               - setup cleaned up, minimum servo angle adjustable
  2020-07 Andreas Holzhammer | Version 0.2.6
                               - scale calibration upgrade; rounding up of measured values; scale "warm-up" before bootscreen
                               - piezo buzzer activated (suggested by Johannes Kuder)
  2020-07 Johannes Kuder     | 0.2.7
                               - counter by filled jars and weight (only in auto mode)
  2020-07 Jeremias Bruker    | 0.2.8
                               - "JarType" integrated in every menus and automatic mode
                               - 5 different jars can be configured for weight and jar type by user in "Fill quantity" menu and are save in a nonvolatile manner. This way every use can set their own particular jars types
                               - stabilization of scale values at will (define ERRORCORRECTION_SCALE)
                               - calibration weight can be changed by user during calibration process
                                 (not everyone has 500g as a calibration weight) and is stored in a nonvolatile manner 
                               - rotating main menu
                               - reversible servo for left-hand opening honeygate :-)
  2020-10 Andreas Holzhammer | 0.2.8.1
                               - bugfix: servo could be moved below minimum in manual mode
                               - jar tolerance can be controlled via variable and adjusted to +-20g 
  2020-12 Andreas Holzhammer | 0.2.9
                               - built in progressbar
                               - adapted to ESP32Servo from the library manager
  2021-02 Andreas Holzhammer | 0.2.10
                               - correction can be set between -90 and +20
                               - autocorrection also without autostart
                               - preferences implemented in a more flash-friendly way
  2021-07 Andreas Holzhammer | 0.2.11
                               - credits page
                               - fix for rotary with step > 1
  2021-11 Andreas Holzhammer | 0.2.12
                               - configurable jar tolerance
                               - confort adjustments for fill quantity (1g/5g/25g steps)
  2023-01 Clemens Gruber     | 0.2.13
                               - pin adaptation for hardware version V3 of Heltec "WiFi Kit 32 V3" with pin layout changed again
                               - adaptation for ESP32 Arduino core version ≥ 2.x
                               - display, U8g2: HW instead of SW in constructor (possibly problems with olderHeltec versions)
                               - rotary: de-bouncing code in isr2 commented out, was leading to crashes
  2023-04 Roland Rust        | W.0.1
                               - code adaptation for ESP32 Wroom DevKit and 2.4 inch display (128x64)
                               - pin layout changed
                               - everything in relation to Heltec module thrown out (Hardware Level removed)
                               - code for potentiometer removed. Rotary encoder only
                               - code for scale simulation removed
                               - list setup removed. Scroll setup only
                               - all the bling removed
                               - scale detection changed. (I don't always have the HX711 connected because the scale and MC unit are separate. I just checked whether an HX711 was connected)
                               - introduced choice for OLED over I2C or SPI
                               - added INA219 menu (shows as soon as an INA219 is installed)
                               - added measurement of servo current (function is still experimental)
                               - correction setup moved out to the automatic setup. Makes more sense to me and with the bigger display there's room for it
                               - implemented TFT display (320x240)
  2023-12 Achim Pfaff        | W.0.2
                               - automatic filling with volume flow to prevent filling up too quickly.
                               - integrated automatic in menu
  2024-01 Roland Rust          - rotary turn directions changed in some menus
                               - rotary HW-040 Bug fixed (the menu should work correctly now)
                               - volume flow implemented in OLED display
                               - English language added
                               - OTA Update added
  2024-01 Achim Pfaff          - jar detection improved because filling is started by simply putting up full jar

  2024-02 Jérémie Lehmann    | W.0.3
                               -all commentaries and code expressions translated from the original German version to English. The translator was'nt a native speaker of either languages, errors and inaccuracies are to be expected  :)
                               -French language added
                              
  
                               

  This code is in the public domain.
  
  Notes on hardware
  ---------------------
  - internal pull-downs are activated for all digital entries, no external resistances needed!
*/

#include <Arduino.h>
#include <Wire.h>
#include <U8g2lib.h>                /* from the library manager */
#include <HX711.h>                  /* from the library manager */
#include <ESP32Servo.h>             /* from the library manager */
#include <Adafruit_INA219.h>        /* from the library manager */
#include <Preferences.h>            /* from the expressif BSP, available when the right board is chosen */
#include <nvs_flash.h>              /* from the expressif BSP, available when the right board is chosen */
#include <Arduino_GFX_Library.h>    /* from the library manager */

// Version
String version = "W.0.3";

//
// User setup
//
// You can leave it on 1. User 2 and User 3 have other jar settings
//
#define USER 1                    // 1 = Hanimandl Standard (this is the default settting as you are used to)
                                  // 2 = Gerold (will not work for you, as the logos are missing in repos)
                                  // 3 = Roli

//
// Setup the code for the used hardware here
//
#define HARDWARE_LEVEL 1          // 1 = ESP32-WROOM 38Pin Connector
#define SCALE_TYP 1               // 1 = 2kg load cell
                                  // 2 = 5kg load cell
#define SERVO_EXPANDED           // define in case the hardware uses an older potentiometer or the servo moves to little
                                  // if not, the servo will stay open a few degrees in stop position! Check after updating!
#define ROTARY_SCALE 2            // in what steps the rotary encoder moves. 
                                  // Examples: KY-040 = 2, HW-040 = 1, for potentiometer use, set 1
#define DISPLAY_TYPE 3            // 1 = 128x64 pixel OLED display linked over I2C
                                  // 2 = 128x64 pixel OLED display linked over SPI
                                  // 3 = 320x240 pixel TFT display ST7789 linked over SPI
                                  // 99 = OLED over I2C and  TFT over SPI for development
#define LANGUAGE 3                // 1 = Deutsch
                                  // 2 = English
                                  // 3 = Français
#define OTA 1                     // 0 = OTA Uptade deactivated
                                  // 1 = OTA Update activated
//#define ERRORCORRECTION_SCALE   // If weight jumps occur, they can be absorbed here
                                  // Warning: this can slow down the weighing process. Check hardware beforehand.
//#define HONEYGATE_LEFT       // Inverses the servo, in case of a left-hand opening honeygate
//
// End of user setup!
// 

//
// From here on, only change if you know exactly what you're doing!
//
//#define isDebug 1             // Activate serial debug output serielle. Values > 3 will output every measurement
                                // set to 4 for additional lead times
                                // set to 4 for additional debug infos
                                // WARNING: to many serial outputs can trigger an ISR watchdog reset!

#if SCALE_TYP == 1
  #define MAXIMUMWEIGHT 1500     // Maximum weight for a  2kg loadcell
#elif SCALE_TYP == 2
  #define MAXIMUMWEIGHT 4500     // Maximum weight for a  5kg loadcell
#else
  #error No scale defined
#endif

// Driving of scale
#define SCALE_READS 2           // Parameter for hx711 library. Measured value displayed on screen
#define SCALE_GETUNITS(n)       round(scale.get_units(n))

// Driveig of servo
#ifdef HONEYGATE_LEFT
  #define SERVO_WRITE(n)     servo.write(180-n)
#else
  #define SERVO_WRITE(n)     servo.write(n)
#endif

// Rotary encoder button pulls level to low
#define SELECT_SW outputSW
#define SELECT_LEVEL LOW

// Operation mode 
#define MODE_SETUP       0
#define MODE_AUTOMATIC   1
#define MODE_MANUAL 2

// Buzzer sounds
#define BUZZER_SHORT   1
#define BUZZER_LONG    2
#define BUZZER_SUCCESS 3
#define BUZZER_ERROR   4

// INA 219
Adafruit_INA219 ina219;

// Display
#if DISPLAY_TYPE == 1
  U8G2_SSD1306_128X64_NONAME_F_HW_I2C u8g2(U8G2_R0, /* clock=*/ 22, /* data=*/ 21, /* reset=*/ U8X8_PIN_NONE);
#elif DISPLAY_TYPE == 2
  U8G2_SSD1309_128X64_NONAME0_F_4W_SW_SPI u8g2(U8G2_R0, /* clock=*/ 18, /* data=*/ 23, /* cs=*/ 5, /* dc=*/ 13, /* reset=*/ 14);
#elif DISPLAY_TYPE == 3
  Arduino_DataBus *bus = new Arduino_HWSPI(13 /* DC */, 5 /* CS */);
  Arduino_GFX *gfx = new Arduino_ST7789(bus, 14 /* RST */, 3 /* rotation */);
#elif DISPLAY_TYPE == 99
  U8G2_SSD1306_128X64_NONAME_F_HW_I2C u8g2(U8G2_R0, /* clock=*/ 22, /* data=*/ 21, /* reset=*/ U8X8_PIN_NONE);
  Arduino_DataBus *bus = new Arduino_HWSPI(13 /* DC */, 5 /* CS */);
  Arduino_GFX *gfx = new Arduino_ST7789(bus, 14 /* RST */, 3 /* rotation */);
  //Arduino_GFX *gfx = new Arduino_ILI9341(bus, 14 /* RST */, 1 /* rotation */);
#else
  #error No display defined
#endif

//Language
#if LANGUAGE == 1
  #include "./Resources/resources_de.h"
#elif LANGUAGE == 2
  #include "./Resources/resources_en.h"
#elif LANGUAGE == 3
  #include "./Resources/resources_fr.h"
#endif

//OTA
#if OTA == 1
  #include "./Resources/wifi.h"
  #include <WiFi.h>
  #include <WiFiClient.h>
  #include <WebServer.h>
  #include <ElegantOTA.h>                 /* from the library manager */
  WebServer server(80);
  unsigned long ota_progress_millis = 0;
  void OTASetup(void);
#endif

// Fonts
#if DISPLAY_TYPE == 3 or DISPLAY_TYPE == 99
  #include "./Fonts/Punk_Mono_Bold_120_075.h"           //10 x 7
  #include "./Fonts/Punk_Mono_Bold_160_100.h"           //13 x 9
  #include "./Fonts/Punk_Mono_Bold_200_125.h"           //16 x 12
  #include "./Fonts/Punk_Mono_Bold_240_150.h"           //19 x 14
  //#include "./Fonts/Punk_Mono_Bold_280_175.h"          //22 x 16
  #include "./Fonts/Punk_Mono_Bold_320_200.h"           //25 x 18
  #include "./Fonts/Punk_Mono_Bold_600_375.h"           //48 x 36
  #include "./Fonts/Punk_Mono_Thin_120_075.h"           //10 x 7
  #include "./Fonts/Punk_Mono_Thin_160_100.h"           //13 x 9
  #include "./Fonts/Punk_Mono_Thin_240_150.h"           //19 x 14
  #include "./Fonts/Icons_Start_Stop.h"                 //A=Start, B=Stop, M=Frames
  #include "./Fonts/Checkbox.h"                         //A=OK, B=nOK
#endif

//Logos
#if DISPLAY_TYPE == 1 or DISPLAY_TYPE == 2 or DISPLAY_TYPE == 99
  #if USER == 2
    #include "./Logos/LogoGeroldOLED.h"
  #elif USER == 3
    #include "./Logos/LogoRoliOLED.h"
  #else
    #include "./Logos/LogoBieneOLED.h"
  #endif
#endif


#if DISPLAY_TYPE == 3 or DISPLAY_TYPE == 99
  #if USER == 2
    #include "./Logos/LogoGeroldTFT.h"
  #elif USER == 3
    #include "./Logos/LogoRoliTFT.h"
  #else
    #include "./Logos/LogoBieneTFT.h"
  #endif
#endif


// ** Pin defintion 
// ----------------------

// ESP32-WROOM 38Pin Connector
//
#if HARDWARE_LEVEL == 1
  // Rotary Encoder
  const int outputA  = 33;
  const int outputB  = 26;
  const int outputSW = 32;
  // Servo
  const int servo_pin = 2;
  // 3-way selector ON 1 - OFF - ON 2, VCC pins are connected directly to VCC
  const int switch_mode_pin = 15;
  const int switch_setup_pin   = 4;
  // Buttons, VCC pins are connected directly to VCC
  const int button_start_pin     = 12;
  const int button_stop_pin      = 27;
  // Loadcell IC 
  const int hx711_sck_pin = 17;
  const int hx711_dt_pin  = 35;
  // Buzzer and LED
  static int buzzer_pin = 25;
  int led_pin = 0;
#else
  #error Hardware Level not defined! Set correct #define !
#endif

Servo servo;
HX711 scale;
Preferences preferences;

// Structure for Rotary Encoder
struct rotary {                        
  int Value;
  int Minimum;
  int Maximum;
  int Step;
};
#define SW_ANGLE    0
#define SW_CORRECTION 1
#define SW_FLOW     2
#define SW_MENU      3
struct rotary rotaries[4];         // Initialized in setup()
int rotary_select = SW_ANGLE;
static boolean rotating = false;   // debounce management for rotary encoder

// Fill quantities for 5 different jars
struct jars { 
  int Weight;
  int JarType;
  int Tare;
  int TripCount;
  int Count;
};
#if USER == 2
const char *JarTypeArray[2] = {"TOF", "SPZ"};
struct jars jars[5] = { 
                            {50, 0, -9999, 0, 0},
                            {125, 0, -9999, 0, 0},
                            {250, 0, -9999, 0, 0},
                            {500, 0, -9999, 0, 0},
                            {1000, 0, -9999, 0, 0} 
                          };
#elif USER == 3
const char *JarTypeArray[1] = {"TOF"};
struct jars jars[5] = { 
                            {250, 0, -9999, 0, 0},
                            {500, 0, -9999, 0, 0},
                            {0, 0, -9999, 0, 0},
                            {0, 0, -9999, 0, 0},
                            {0, 0, -9999, 0, 0} 
                          };
#else
const char *JarTypeArray[3] = {"DIB", "TOF", "DEE"};
struct jars jars[5] = { 
                            {125, 0, -9999, 0, 0},
                            {250, 1, -9999, 0, 0},
                            {250, 2, -9999, 0, 0},
                            {500, 1, -9999, 0, 0},
                            {500, 0, -9999, 0, 0} 
                          };
#endif

// General variables
int i;                              // general count variable
int pos;                            // current position of potentiometer or rotary 
int weight;                        // current weight
int tare;                           // tare for chosen jar, for automatic mode
int tare_jar;                      // tare forcurrent jar, in case jar weight differs
long weight_empty;                  // weight of empty scale
float factor;                       // scaling factor for scale values
int fquantity;                         // chosen fill quantity
int fquantity_index;                   // index in jars[]
int correction;                      // correction value for fill quantity
int autostart;                      // full automatic on/off
int autocorrection;                  // autocorrection on/off
int intweight;                     // A.P. Flow rate per run that should flow (g / time unit) Filling speed, this should give a constant speed when filling, regardless of the filling level.
int intweightD = 1;                // A.P. allows deviation from intweight
const int RunningAverageCount = 3;  // A.P.
int NextRunningAverage;             // A.P.   
float RunningAverageBuffer[RunningAverageCount];     //A.P.
int Anglemarker;  // A.P. Angle marker for optimum next filling
int overfill_gr;                      // desired overfilling for autcorrection mode in grams
int angle;                         // current servo angle
int angle_hard_min = 0;            // hard limit for servo
int angle_hard_max = 180;          // hard limit for servo
int angle_min = 0;                 // configurable in setup
int angle_max = 85;                // configurable in setup
int angle_fine = 35;               // configurable in setup
float fine_dosing_weight = 60;     // float due to calculation of the closing angle
int servo_active = 0;                // activate servo yes/no
#if USER == 2
int cali_weight = 2000;            // free choosable weight for calibration
#elif USER == 3
int cali_weight = 500;             // free choosable weight for calibration
#else
int cali_weight = 500;             // free choosable weight for calibration
#endif
char output[30];                   // Fontsize 12 = 13 maximum characters in one line
int mode = -1;                     // wether to drive the servo to minimum on mode change
int auto_active = 0;                 // for automatic mode system on/off?
int scale_available = 0;            // not talking to HX711 when no scale is connected, otherwise crash
long preferences_chksum;            // checksum to not write uncoherent prefs
int buzzermode = 0;                 // 0 = off, 1 = on.
int ledmode = 0;                    // 0 = off, 1 = on.
bool counted = true;               // Kud count flag
int jartolerance = 20;              // weight for autostart may vary by +-20g, total 40g!
int MenupointsCount;              // number of menu items in the setup menu
int lastpos = 0;                    // last position in the setup menu
int progressbar = 0;                // Variable for the progressbar
int showlogo = 1;                   // 0 = off, 1 = on
int showcredits = 1;                // 0 = off, 1 = on
int ina219_installed = 0;           // 0 = no INA219 installed, 1 = INA219 is installed
int current_servo = 0;              // 0 = INA219 will be ignored, 1-1000 = maximum current allowed for servo in mA
int current_mA;                     // current Amps consumed by servo
int updatetime_ina219 = 500;        // INA219 readtime in ms (500 -> on current measurement taken every 0,5 secound)
int last_ina219_measurement = 0;    // last timepoint at which current was measured
int overcurrenttime = 1500;         // time during which the servo is allowed to pull more current than setup in ms
int last_overcurrenttime = 0;       // last timepoint at which no overcurrent was measured
int alarm_overcurrent = 0;          // alarmflag when the servo pulls too much current
int show_current = 0;               // 0 = off, 1 = on / Shows the current event though the INA is ignored
int inawatchdog = 1;                // 0 = off, 1 = on / requires to suspend INA measurements
int offset_angle = 0;              // Offset in degrees  from closed angle when servo was overcurrent  (max +3 degrees from setup angle min)
int color_scheme = 0;               // 0 = dark, 1 = light / change color scheme for the TFT display
int color_marker = 2;               // marker color for the TFT display
int ota_done = 0;                   // variable for OTA update

//Variables for TFT update
bool no_ina;
int weight_alt;
int angle_min_old;
int pos_old;
int angle_is_old;
int tare_old;
int current_mA_old;
int servo_active_old;
int auto_active_old;
int jar_old;
int corr_old;
int autocorr_gr_old;
int intweight_old;

//Color scheme for the TFT display
unsigned long  BACKGROUND;
unsigned long  TEXT;
unsigned long  MENU_POS1;
unsigned long  MENU_POS2;
unsigned long  MENU_POS3;
unsigned long  MENU_POS4;
unsigned long  MARKER;
//#define MARKER 0x1AF6

#if DISPLAY_TYPE == 3 or DISPLAY_TYPE == 99
  void tft_colors() {
    if (color_scheme == 0) {          //dark
      BACKGROUND  = 0x0000;
      TEXT        = 0xFFFF;
      MENU_POS1   = 0x73AF;
      MENU_POS2   = 0x5AEC;
      MENU_POS3   = 0x39C8;
      MENU_POS4   = 0x20E6;
    }
    else {                            //light
      BACKGROUND  = 0xFFFF;
      TEXT        = 0x0000;
      MENU_POS1   = 0x1082;
      MENU_POS2   = 0x2104;
      MENU_POS3   = 0x3166;
      MENU_POS4   = 0x4208;
    }
  }

  void tft_marker() {                
    if (color_marker == 0)                              {MARKER = 0x05ff;}
    else if (color_marker == 1)                         {MARKER = 0x021f;}
    else if (color_marker == 2)                         {MARKER = 0x001f;}
    else if (color_marker == 3)                         {MARKER = 0x4851;}
    else if (color_marker == 4)                         {MARKER = 0xc050;}
    else if (color_marker == 5)                         {MARKER = 0xf800;}
    else if (color_marker == 6)                         {MARKER = 0xfbc0;}
    else if (color_marker == 7)                         {MARKER = 0xfde0;}
    else if (color_marker == 8)                         {MARKER = 0xd6e1;}
    else if (color_marker == 9)                         {MARKER = 0xffe0;}
    else if (color_marker == 10)                        {MARKER = 0x2724;}
    else if (color_marker == 11)                        {MARKER = 0x0da1;}
  }
#endif

// Rotary button. The interrupt only takes effect in automatic mode and only when servo is inactive.
// The button switches to one of three modes in which different values ​​are counted.
void IRAM_ATTR isr1() {
  static unsigned long last_interrupt_time = 0; 
  unsigned long interrupt_time = millis();
  if (interrupt_time - last_interrupt_time > 300) {      // If interrupts come faster than 300ms, assume it's a bounce and ignore
    if ( mode == MODE_AUTOMATIC && servo_active == 0 ) { // the click is only relevant in automatic mode
      rotary_select = (rotary_select + 1) % 4;
      #if DISPLAY_TYPE == 3 or DISPLAY_TYPE == 99
        if      (rotary_select == SW_CORRECTION) {corr_old = -99999;}
        else if (rotary_select == SW_FLOW)     {intweight_old = -1; corr_old = -99999;}
        else if (rotary_select == SW_MENU)      {jar_old = -1; intweight_old = -1;}
        else {corr_old = -99999; intweight_old = -1; jar_old = -1;}
      #endif
      #ifdef isDebug
        Serial.print("Rotary Button changed to ");
        Serial.println(rotary_select);
      #endif 
    }
    last_interrupt_time = interrupt_time;
  }
}

// Rotary encoder. Counts into one of three data structures: 
// SW_ANGLE    = Setup of the servo angle
// SW_CORRECTION = Correction factor for fill weight
// SW_FLOW     = Factor for flow speed
// SW_MENU      = Counter for menu selection  
void IRAM_ATTR isr2() {
  static int aState;
  static int aLastState = 2;  // real values are 0 and 1
  aState = digitalRead(outputA); // Reads the "current" state of the outputA
  if (aState != aLastState) {     
    // If the outputB state is different to the outputA state, that means the encoder is rotating clockwise
    if (digitalRead(outputB) != aState) {
      rotaries[rotary_select].Value += rotaries[rotary_select].Step;
    }
    else {    // counter-clockwise
      rotaries[rotary_select].Value -= rotaries[rotary_select].Step;
    }
    rotaries[rotary_select].Value = constrain( rotaries[rotary_select].Value, rotaries[rotary_select].Minimum, rotaries[rotary_select].Maximum);
    rotating = false;
    #ifdef isDebug
      #if isDebug >= 4
        Serial.print(" Rotary Value changed to ");
        Serial.println(getRotariesValue(rotary_select));
      #endif
    #endif
  }
  aLastState = aState; // Updates the previous state of the outputA with the current state
}

//
// Scaling of rotoray for different rotary encoders
int getRotariesValue( int rotary_mode ) {
  return ((rotaries[rotary_mode].Value - (rotaries[rotary_mode].Value % (rotaries[rotary_mode].Step * ROTARY_SCALE))) / ROTARY_SCALE );
}

void setRotariesValue( int rotary_mode, int rotary_value ) {
  rotaries[rotary_mode].Value = rotary_value * ROTARY_SCALE;
}

void initRotaries( int rotary_mode, int rotary_value, int rotary_min, int rotary_max, int rotary_step ) {
  rotaries[rotary_mode].Value     = rotary_value * ROTARY_SCALE;
  rotaries[rotary_mode].Minimum   = rotary_min   * ROTARY_SCALE;
  rotaries[rotary_mode].Maximum   = rotary_max   * ROTARY_SCALE;
  rotaries[rotary_mode].Step      = rotary_step;
  #ifdef isDebug
    Serial.print("initRotaries..."); 
    Serial.print(" Rotary Mode: ");  Serial.print(rotary_mode);
    Serial.print(" rotary_value: "); Serial.print(rotary_value);
    Serial.print(" Value: ");        Serial.print(rotaries[rotary_mode].Value);
    Serial.print(" Min: ");          Serial.print(rotaries[rotary_mode].Minimum);
    Serial.print(" Max: ");          Serial.print(rotaries[rotary_mode].Maximum);
    Serial.print(" Step: ");         Serial.print(rotaries[rotary_mode].Step);
    Serial.print(" Scale: ");        Serial.println(ROTARY_SCALE);
  #endif
}
// End function for rotary encoder
//

void getPreferences(void) {
  preferences.begin("EEPROM", false);                     // Read parameter from EEPROM
  factor          = preferences.getFloat("factor", 0.0);  // in case this is not set -> scale is not calibrated
  pos             = preferences.getUInt("pos", 0);
  weight_empty    = preferences.getUInt("weight_empty", 0); 
  correction       = preferences.getUInt("correction", 0);
  autostart       = preferences.getUInt("autostart", 0);
  autocorrection   = preferences.getUInt("autocorrection", 0);
  intweight      = preferences.getUInt("intweight", intweight);  // on 0 .A.P.
  overfill_gr       = preferences.getUInt("overfill_gr", 5);
  fquantity_index    = preferences.getUInt("fquantity_index", 3);
  angle_min      = preferences.getUInt("angle_min", angle_min);
  angle_max      = preferences.getUInt("angle_max", angle_max);
  angle_fine     = preferences.getUInt("angle_fine", angle_fine);
  buzzermode      = preferences.getUInt("buzzermode", buzzermode);
  ledmode         = preferences.getUInt("ledmode", ledmode);
  showlogo        = preferences.getUInt("showlogo", showlogo);
  showcredits     = preferences.getUInt("showcredits", showcredits);
  cali_weight    = preferences.getUInt("cali_weight", cali_weight);
  jartolerance    = preferences.getUInt("jartolerance", jartolerance);
  current_servo   = preferences.getUInt("current_servo", current_servo);
  show_current    = preferences.getUInt("show_current", show_current);
  color_scheme    = preferences.getUInt("color_scheme", color_scheme);
  color_marker    = preferences.getUInt("color_marker", color_marker);
  Anglemarker    = angle_min; //  A.P. Take data from EEProm.
  preferences_chksum = factor + pos + weight_empty + correction + autostart + autocorrection + intweight + overfill_gr + fquantity_index +
                       angle_min + angle_max + angle_fine + buzzermode + ledmode + showlogo + showcredits + 
                       cali_weight + current_servo + jartolerance + show_current + color_scheme + color_marker; //A.P.
  i = 0;
  #if USER == 2
    int Resetweights[] = {50,125,250,500,1000,};
    int ResetJarType[] = {0,0,0,0,0,};
  #elif USER == 3
    int Resetweights[] = {250,500,0,0,0,};
    int ResetJarType[] = {0,0,0,0,0,};
  #else
    int Resetweights[] = {125,250,250,500,500,};
    int ResetJarType[] = {0,1,2,1,0,};
  #endif
  while( i < 5) {
    sprintf(output, "Weight%d", i);
    jars[i].Weight = preferences.getInt(output, Resetweights[i]);
    preferences_chksum += jars[i].Weight;
    sprintf(output, "JarType%d", i);
    jars[i].JarType = preferences.getInt(output, ResetJarType[i]);
    preferences_chksum += jars[i].JarType;
    sprintf(output, "Tare%d", i);
    jars[i].Tare= preferences.getInt(output, -9999);
    preferences_chksum += jars[i].Tare;
    sprintf(output, "TripCount%d", i);
    jars[i].TripCount = preferences.getInt(output, 0);
    preferences_chksum += jars[i].TripCount;
    sprintf(output, "Count%d", i);
    jars[i].Count = preferences.getInt(output, 0);
    preferences_chksum += jars[i].Count;
    i++;
  }
  preferences.end();
  #ifdef isDebug
    Serial.println("get Preferences:");
    Serial.print("pos = ");             Serial.println(pos);
    Serial.print("factor = ");          Serial.println(factor);
    Serial.print("weight_empty = ");    Serial.println(weight_empty);
    Serial.print("correction = ");       Serial.println(correction);
    Serial.print("autostart = ");       Serial.println(autostart);
    Serial.print("autocorrection = ");   Serial.println(autocorrection);
    Serial.print("intweight = ");      Serial.println(intweight);  //A.P.
    Serial.print("overfill_gr = ");       Serial.println(overfill_gr);
    Serial.print("fquantity_index = ");    Serial.println(fquantity_index);
    Serial.print("angle_min = ");      Serial.println(angle_min);
    Serial.print("angle_max = ");      Serial.println(angle_max);
    Serial.print("angle_fine = ");     Serial.println(angle_fine);
    Serial.print("buzzermode = ");      Serial.println(buzzermode);
    Serial.print("ledmode = ");         Serial.println(ledmode);
    Serial.print("showlogo = ");        Serial.println(showlogo);
    Serial.print("showcredits = ");     Serial.println(showcredits);
    Serial.print("current_servo = ");   Serial.println(current_servo);
    Serial.print("color_scheme = ");    Serial.println(color_scheme);
    Serial.print("color_marker = ");    Serial.println(color_marker);
    Serial.print("cali_weight = ");    Serial.println(cali_weight);
    Serial.print("jartolerance = ");    Serial.println(jartolerance);
    Serial.print("show_current = ");    Serial.println(show_current);
    i = 0;
    while( i < 5 ) {
      sprintf(output, "Weight%d = ", i);
      Serial.print(output);         
      Serial.println(jars[i].Weight);
      sprintf(output, "JarType%d = ", i);
      Serial.print(output);         
      Serial.println(JarTypeArray[jars[i].JarType]);
      sprintf(output, "Tare%d = ", i);
      Serial.print(output);         
      Serial.println(jars[i].Tare);
      i++;
    }
    Serial.print("Checksum:");    
    Serial.println(preferences_chksum);    
  #endif
}

void setPreferences(void) {
  long preferences_newchksum;
  int angle = getRotariesValue(SW_ANGLE);
  int i;
  preferences.begin("EEPROM", false);
  // Angle setup dealt with separately, changes often
  if ( angle != preferences.getUInt("pos", 0) ) {
    preferences.putUInt("pos", angle);
    #ifdef isDebug
      Serial.print("angle recorded: ");
      Serial.println(angle);
    #endif
  }
  // Counter dealt with separately, changes often
  for ( i=0 ; i < 5; i++ ) {
    sprintf(output, "TripCount%d", i);
    if (jars[i].TripCount != preferences.getInt(output, 0)) {
      preferences.putInt(output, jars[i].TripCount);
    }
    sprintf(output, "Count%d", i);
    if (jars[i].Count != preferences.getInt(output, 0)) {
      preferences.putInt(output, jars[i].Count);
    }
    #ifdef isDebug
      Serial.print("Counter recorded: Index ");
      Serial.print(i);
      Serial.print(" Trip ");
      Serial.print(jars[fquantity_index].TripCount);
      Serial.print(" Total ");
      Serial.println(jars[fquantity_index].Count);      
    #endif
  }
  // We're doing everything else together, it is rather static
  preferences_newchksum = factor + weight_empty + correction + autostart + autocorrection + intweight +
                          fquantity_index + angle_min + angle_max + angle_fine + overfill_gr +
                          buzzermode + ledmode + showlogo + showcredits + current_servo + cali_weight + 
                          jartolerance + show_current + color_scheme + color_marker; //A.P.
  i = 0;
  while( i < 5 ) {
    preferences_newchksum += jars[i].Weight;
    preferences_newchksum += jars[i].JarType;
    preferences_newchksum += jars[i].Tare;
    i++;
  }
  if( preferences_newchksum == preferences_chksum ) {
    #ifdef isDebug
      Serial.println("Preferences unchanged");
    #endif
    return;
  }
  preferences_chksum = preferences_newchksum;
  preferences.putFloat("factor", factor);
  preferences.putUInt("weight_empty", weight_empty);
  preferences.putUInt("correction", correction);
  preferences.putUInt("autostart", autostart);
  preferences.putUInt("autocorrection", autocorrection);
  preferences.putUInt("intweight", intweight); //A.P.
  preferences.putUInt("overfill_gr", overfill_gr);
  preferences.putUInt("angle_min", angle_min);
  preferences.putUInt("angle_max", angle_max);
  preferences.putUInt("angle_fine", angle_fine);
  preferences.putUInt("fquantity_index", fquantity_index);
  preferences.putUInt("buzzermode", buzzermode);
  preferences.putUInt("ledmode", ledmode);
  preferences.putUInt("showlogo", showlogo);
  preferences.putUInt("showcredits", showcredits);
  preferences.putUInt("cali_weight", cali_weight);
  preferences.putUInt("jartolerance", jartolerance);
  preferences.putUInt("current_servo", current_servo);
  preferences.putUInt("show_current", show_current);
  preferences.putUInt("color_scheme", color_scheme);
  preferences.putUInt("color_marker", color_marker);
  i = 0;
  while( i < 5 ) {
    sprintf(output, "Weight%d", i);
    preferences.putInt(output, jars[i].Weight);
    sprintf(output, "JarType%d", i);
    preferences.putInt(output, jars[i].JarType);  
    sprintf(output, "Tare%d", i);
    preferences.putInt(output, jars[i].Tare);
    i++;
  }
  preferences.end();
  #ifdef isDebug
    Serial.println("Set Preferences:");
    Serial.print("pos = ");             Serial.println(angle);
    Serial.print("factor = ");          Serial.println(factor);
    Serial.print("weight_empty = ");    Serial.println(weight_empty);
    Serial.print("correction = ");       Serial.println(correction);
    Serial.print("autostart = ");       Serial.println(autostart);
    Serial.print("intweight = ");      Serial.println(intweight); //A.P.
    Serial.print("autocorrection = ");   Serial.println(autocorrection);
    Serial.print("overfill_gr = ");       Serial.println(overfill_gr);
    Serial.print("fquantity_index = ");    Serial.println(fquantity_index);
    Serial.print("angle_min = ");      Serial.println(angle_min);
    Serial.print("angle_max = ");      Serial.println(angle_max);
    Serial.print("angle_fine = ");     Serial.println(angle_fine);
    Serial.print("buzzermode = ");      Serial.println(buzzermode);
    Serial.print("ledmode = ");         Serial.println(ledmode);
    Serial.print("showlogo = ");        Serial.println(showlogo);
    Serial.print("showcredits = ");     Serial.println(showcredits);
    Serial.print("current_servo = ");   Serial.println(current_servo);
    Serial.print("cali_weight = ");    Serial.println(cali_weight);
    Serial.print("jartolerance = ");    Serial.println(jartolerance);
    Serial.print("show_current = ");    Serial.println(show_current);
    Serial.print("color_scheme = ");    Serial.println(color_scheme);
    Serial.print("color_marker = ");    Serial.println(color_marker);
    i = 0;
    while( i < 5 ) {
      sprintf(output, "Weight%d = ", i);
      Serial.print(output);         Serial.println(jars[i].Weight);
      sprintf(output, "JarType%d = ", i);
      Serial.print(output);         Serial.println(JarTypeArray[jars[i].JarType]);
      sprintf(output, "Tare%d = ", i);
      Serial.print(output);         Serial.println(jars[i].Tare);
      i++;
    }
  #endif
}

void setupTripCounter(void) {
  int j;
  i = 1;
  float TripFillweight = 0;
  #if DISPLAY_TYPE == 3 or DISPLAY_TYPE == 99
    int y_offset_tft = 28;
    gfx->fillScreen(BACKGROUND);
    gfx->setTextColor(TEXT);
    gfx->setFont(Punk_Mono_Bold_240_150);
    int x_pos = CenterPosX(SZ_TO02, 14, 320);
    gfx->setCursor(x_pos, 25);
    gfx->println(SZ_TO02);
    gfx->drawLine(0, 30, 320, 30, TEXT);
  #endif
  while (i > 0) { //First screen: count per jar size
    if (digitalRead(button_stop_pin) == HIGH or digitalRead(switch_setup_pin) == LOW) {
      while (digitalRead(button_stop_pin) == HIGH) {
        delay(1);
      }
      mode = -1;
      return;
    }
    #if DISPLAY_TYPE == 1 or DISPLAY_TYPE == 2 or DISPLAY_TYPE == 99
      j = 0;
      u8g2.setFont(u8g2_font_courB08_tf);
      u8g2.clearBuffer();
      while (j < 5) {
        u8g2.setCursor(1, 10 + (j * 13));
        sprintf(output, "%4dg %3s", jars[j].Weight, JarTypeArray[jars[j].JarType]);
        u8g2.print(output);
        u8g2.setCursor(50, 10 + (j * 13));
        sprintf(output, "%9d St.", jars[j].TripCount);
        u8g2.print(output);
        j++;
      }
      u8g2.sendBuffer();
    #endif
    #if DISPLAY_TYPE == 3 or DISPLAY_TYPE == 99
      j = 0;
      while ( j < 5 ) {
        gfx->setCursor(5, 30+((j+1) * y_offset_tft));
        sprintf(output, "%4dg %3s", jars[j].Weight,JarTypeArray[jars[j].JarType]);
        gfx->print(output);
        gfx->setCursor(110, 30+((j+1) * y_offset_tft));
        sprintf(output, "%11d St.", jars[j].TripCount);
        gfx->print(output);
        j++;
      }
    #endif
    if (digitalRead(SELECT_SW) == SELECT_LEVEL) {
      //Leave screen
      while(digitalRead(SELECT_SW) == SELECT_LEVEL) {
        delay(1);
      }
      i = 0;
      #if DISPLAY_TYPE == 3 or DISPLAY_TYPE == 99
        gfx->fillRect(0, 31, 320, 209, BACKGROUND);
      #endif
    }
  }
  i = 1;
  while (i > 0) { //Second screen: weight per jar size
    if (digitalRead(button_stop_pin) == HIGH or digitalRead(switch_setup_pin) == LOW) {
      while (digitalRead(button_stop_pin) == HIGH) {
        delay(1);
      }
      mode = -1;
      return;
    }
    #if DISPLAY_TYPE == 1 or DISPLAY_TYPE == 2 or DISPLAY_TYPE == 99
      j = 0;
      u8g2.setFont(u8g2_font_courB08_tf);
      u8g2.clearBuffer();
      while (j < 5) {
        u8g2.setCursor(1, 10 + (j * 13));
        sprintf(output, "%4dg %3s", jars[j].Weight,JarTypeArray[jars[j].JarType]);
        u8g2.print(output);
        u8g2.setCursor(65, 10 + (j * 13));
        sprintf(output, "%8.3fkg", jars[j].Weight * jars[j].TripCount / 1000.0);
        u8g2.print(output);
        j++;
      }
      u8g2.sendBuffer();
    #endif
    #if DISPLAY_TYPE == 3 or DISPLAY_TYPE == 99
      j = 0;
      while ( j < 5 ) {
        gfx->setCursor(5, 30+((j+1) * y_offset_tft));
        sprintf(output, "%4dg %3s", jars[j].Weight,JarTypeArray[jars[j].JarType]);
        gfx->print(output);
        gfx->setCursor(150, 30+((j+1) * y_offset_tft));
        sprintf(output, "%10.3fkg", jars[j].Weight * jars[j].TripCount / 1000.0);
        gfx->print(output);
        j++;
      }
    #endif
    if (digitalRead(SELECT_SW) == SELECT_LEVEL) {
      //Leave screen
      while(digitalRead(SELECT_SW) == SELECT_LEVEL) {
        delay(1);
      }
      i = 0;
      #if DISPLAY_TYPE == 3 or DISPLAY_TYPE == 99
        gfx->fillRect(0, 31, 320, 209, BACKGROUND);
      #endif
    }
  }
  i = 1;
  while (i > 0) { //Third screen: total weight
    TripFillweight = 0;
    j = 0;
    if (digitalRead(button_stop_pin) == HIGH or digitalRead(switch_setup_pin) == LOW) {
      while (digitalRead(button_stop_pin) == HIGH) {
        delay(1);
      }
      mode = -1;
      return;
    }
    while (j < 5) {
      TripFillweight += jars[j].Weight * jars[j].TripCount / 1000.0;
      j++;
    }
    #if DISPLAY_TYPE == 1 or DISPLAY_TYPE == 2 or DISPLAY_TYPE == 99
      u8g2.clearBuffer();
      u8g2.setFont(u8g2_font_courB14_tf);
      u8g2.setCursor(5, 15);
      u8g2.print(TOTAL);
      u8g2.setFont(u8g2_font_courB18_tf);
      u8g2.setCursor(10, 50);
      sprintf(output, "%5.1fkg", TripFillweight);
      u8g2.print(output);
      u8g2.sendBuffer();
    #endif
    #if DISPLAY_TYPE == 3 or DISPLAY_TYPE == 99
      gfx->setFont(Punk_Mono_Bold_600_375);
      sprintf(output, TOTAL);
      x_pos = CenterPosX(output, 36, 320);
      gfx->setCursor(x_pos, 124);
      gfx->print(output);
      sprintf(output, "%.1fkg", TripFillweight);
      x_pos = CenterPosX(output, 36, 320);
      gfx->setCursor(x_pos, 184);
      gfx->print(output);
      gfx->setFont(Punk_Mono_Bold_240_150);
    #endif
    if (digitalRead(SELECT_SW) == SELECT_LEVEL) {
      //Leave screen
      while(digitalRead(SELECT_SW) == SELECT_LEVEL) {
        delay(1);
      }
      i = 0;
      #if DISPLAY_TYPE == 3 or DISPLAY_TYPE == 99
        gfx->fillRect(0, 31, 320, 209, BACKGROUND);
      #endif
    }
  }
  i = 1;
  while (i > 0) { //Fourth screen: reset
    initRotaries(SW_MENU, 1, 0, 1, 1);
    rotaries[SW_MENU].Value = 1;
    i = 1;
    while (i > 0) {
      if (digitalRead(button_stop_pin) == HIGH or digitalRead(switch_setup_pin) == LOW) {
        while (digitalRead(button_stop_pin) == HIGH) {
          delay(1);
        }
        mode = -1;
        return;
      }
      pos = getRotariesValue(SW_MENU);
      #if DISPLAY_TYPE == 1 or DISPLAY_TYPE == 2 or DISPLAY_TYPE == 99
        u8g2.setFont(u8g2_font_courB10_tf);
        u8g2.clearBuffer();
        u8g2.setCursor(10, 12);    u8g2.print(RESET);
        u8g2.setCursor(10, 28);    u8g2.print(ABORT);
        u8g2.setCursor(0, 12 + ((pos) * 16));
        u8g2.print("*");
        u8g2.sendBuffer();
      #endif
      #if DISPLAY_TYPE == 3 or DISPLAY_TYPE == 99
        gfx->setCursor(5, 30+(1 * y_offset_tft));
        if (pos == 0) {gfx->setTextColor(MARKER);}
        else {gfx->setTextColor(TEXT);}
        gfx->print(RESET);
        gfx->setCursor(5, 30+(2 * y_offset_tft));
        if (pos == 1) {gfx->setTextColor(MARKER);}
        else {gfx->setTextColor(TEXT);}
        gfx->print(ABORT);
      #endif
      if (digitalRead(SELECT_SW) == SELECT_LEVEL) {
        #if DISPLAY_TYPE == 1 or DISPLAY_TYPE == 2 or DISPLAY_TYPE == 99
          u8g2.setCursor(105, 12 + ((pos) * 16));
          u8g2.print("OK");
          u8g2.sendBuffer();
        #endif
        #if DISPLAY_TYPE == 3 or DISPLAY_TYPE == 99
          gfx->setTextColor(MARKER);
          gfx->setCursor(283, 30+((pos+1) * y_offset_tft));
          gfx->print("OK");
        #endif
        if ( pos == 0) {
          j = 0;
          while ( j < 5  ) {
            jars[j].TripCount = 0;
            j++;
          }
          setPreferences();
        }
        delay(1000);
        mode = -1;
        i = 0;
      }
    }
  }
}

void setupCounter(void) {
  int j;
  i = 1;
  float Fillweight = 0;
  #if DISPLAY_TYPE == 3 or DISPLAY_TYPE == 99
    int y_offset_tft = 28;
    gfx->fillScreen(BACKGROUND);
    gfx->setTextColor(TEXT);
    gfx->setFont(Punk_Mono_Bold_240_150);
    int x_pos = CenterPosX(SZ_TO01, 14, 320);
    gfx->setCursor(x_pos, 25);
    gfx->println(SZ_TO01);
    gfx->drawLine(0, 30, 320, 30, TEXT);
  #endif
  while (i > 0) { //First screen: count per jar size
    if (digitalRead(button_stop_pin) == HIGH or digitalRead(switch_setup_pin) == LOW) {
      while (digitalRead(button_stop_pin) == HIGH) {
        delay(1);
      }
      mode = -1;
      return;
    }
    #if DISPLAY_TYPE == 1 or DISPLAY_TYPE == 2 or DISPLAY_TYPE == 99
      j = 0;
      u8g2.setFont(u8g2_font_courB08_tf);
      u8g2.clearBuffer();
      while ( j < 5 ) {
        u8g2.setCursor(1, 10 + (j * 13));
        sprintf(output, "%4dg %3s", jars[j].Weight,JarTypeArray[jars[j].JarType]);
        u8g2.print(output);
        u8g2.setCursor(50, 10 + (j * 13));
        sprintf(output, "%9d St.", jars[j].Count);
        u8g2.print(output);
        j++;
      }
      u8g2.sendBuffer();
    #endif
    #if DISPLAY_TYPE == 3 or DISPLAY_TYPE == 99
      j = 0;
      while ( j < 5 ) {
        gfx->setCursor(5, 30+((j+1) * y_offset_tft));
        sprintf(output, "%4dg %3s", jars[j].Weight,JarTypeArray[jars[j].JarType]);
        gfx->print(output);
        gfx->setCursor(110, 30+((j+1) * y_offset_tft));
        sprintf(output, "%11d St.", jars[j].Count);
        gfx->print(output);
        j++;
      }
    #endif
    if (digitalRead(SELECT_SW) == SELECT_LEVEL) {
      //Leave screen
      while(digitalRead(SELECT_SW) == SELECT_LEVEL) {
        delay(1);
      }
      i = 0;
      #if DISPLAY_TYPE == 3 or DISPLAY_TYPE == 99
        gfx->fillRect(0, 31, 320, 209, BACKGROUND);
      #endif
    }
  }
  i = 1;
  while (i > 0) { //Second screen: weight per jar size
    if (digitalRead(button_stop_pin) == HIGH or digitalRead(switch_setup_pin) == LOW) {
      while (digitalRead(button_stop_pin) == HIGH) {
        delay(1);
      }
      mode = -1;
      return;
    }
    #if DISPLAY_TYPE == 1 or DISPLAY_TYPE == 2 or DISPLAY_TYPE == 99
      j = 0;
      u8g2.setFont(u8g2_font_courB08_tf);
      u8g2.clearBuffer();
      while ( j < 5) {
        u8g2.setCursor(1, 10 + (j * 13));
        sprintf(output, "%4dg %3s", jars[j].Weight,JarTypeArray[jars[j].JarType]);
        u8g2.print(output);
        u8g2.setCursor(65, 10 + (j * 13));
        sprintf(output, "%8.3fkg", jars[j].Weight * jars[j].Count / 1000.0);
        u8g2.print(output);
        j++;
      }
      u8g2.sendBuffer();
    #endif
    #if DISPLAY_TYPE == 3 or DISPLAY_TYPE == 99
      j = 0;
      while ( j < 5 ) {
        gfx->setCursor(5, 30+((j+1) * y_offset_tft));
        sprintf(output, "%4dg %3s", jars[j].Weight,JarTypeArray[jars[j].JarType]);
        gfx->print(output);
        gfx->setCursor(150, 30+((j+1) * y_offset_tft));
        sprintf(output, "%10.3fkg", jars[j].Weight * jars[j].Count / 1000.0);
        gfx->print(output);
        j++;
      }
    #endif
    if (digitalRead(SELECT_SW) == SELECT_LEVEL) {
      //Leave screen
      while(digitalRead(SELECT_SW) == SELECT_LEVEL) {
        delay(1);
      }
      i = 0;
      #if DISPLAY_TYPE == 3 or DISPLAY_TYPE == 99
        gfx->fillRect(0, 31, 320, 209, BACKGROUND);
      #endif
    }
  }
  i = 1;
  while (i > 0) { //Third screen: total weight
    Fillweight = 0;
    j = 0;
    if (digitalRead(button_stop_pin) == HIGH  or digitalRead(switch_setup_pin) == LOW) {
      while (digitalRead(button_stop_pin) == HIGH) {
        delay(1);
      }
      mode = -1;
      return;
    }
    while ( j < 5  ) {
      Fillweight += jars[j].Weight * jars[j].Count / 1000.0;
      j++;
    }
    #if DISPLAY_TYPE == 1 or DISPLAY_TYPE == 2 or DISPLAY_TYPE == 99
      u8g2.clearBuffer();
      u8g2.setFont(u8g2_font_courB14_tf);
      u8g2.setCursor(1, 15);
      u8g2.print(TOTAL);
      u8g2.setFont(u8g2_font_courB18_tf);
      u8g2.setCursor(10, 50);
      sprintf(output, "%5.1fkg", Fillweight);
      u8g2.print(output);
      u8g2.sendBuffer();
    #endif
    #if DISPLAY_TYPE == 3 or DISPLAY_TYPE == 99
      gfx->setFont(Punk_Mono_Bold_600_375);
      sprintf(output, TOTAL);
      x_pos = CenterPosX(output, 36, 320);
      gfx->setCursor(x_pos, 124);
      gfx->print(output);
      sprintf(output, "%.1fkg", Fillweight);
      x_pos = CenterPosX(output, 36, 320);
      gfx->setCursor(x_pos, 184);
      gfx->print(output);
      gfx->setFont(Punk_Mono_Bold_240_150);
    #endif
    if ((digitalRead(SELECT_SW)) == SELECT_LEVEL) {
      //Leave screen
      while(digitalRead(SELECT_SW) == SELECT_LEVEL) {
        delay(1);
      }
      i = 0;
      #if DISPLAY_TYPE == 3 or DISPLAY_TYPE == 99
        gfx->fillRect(0, 31, 320, 209, BACKGROUND);
      #endif
    }
  }
  i = 1;
  while (i > 0) { //Fourth screen: reset
    initRotaries(SW_MENU, 1, 0, 1, 1);
    rotaries[SW_MENU].Value = 1;
    i = 1;
    while (i > 0) {
      if (digitalRead(button_stop_pin) == HIGH or digitalRead(switch_setup_pin) == LOW) {
        while (digitalRead(button_stop_pin) == HIGH) {
          delay(1);
        }
        mode = -1;
        return;
      }
      pos = getRotariesValue(SW_MENU);
      #if DISPLAY_TYPE == 1 or DISPLAY_TYPE == 2 or DISPLAY_TYPE == 99
        u8g2.setFont(u8g2_font_courB10_tf);
        u8g2.clearBuffer();
        u8g2.setCursor(10, 12);    u8g2.print(RESET);
        u8g2.setCursor(10, 28);    u8g2.print(ABORT);
        u8g2.setCursor(0, 12 + ((pos) * 16));
        u8g2.print("*");
        u8g2.sendBuffer();
      #endif
      #if DISPLAY_TYPE == 3 or DISPLAY_TYPE == 99
        gfx->setCursor(5, 30+(1 * y_offset_tft));
        if (pos == 0) {gfx->setTextColor(MARKER);}
        else {gfx->setTextColor(TEXT);}
        gfx->print(RESET);
        gfx->setCursor(5, 30+(2 * y_offset_tft));
        if (pos == 1) {gfx->setTextColor(MARKER);}
        else {gfx->setTextColor(TEXT);}
        gfx->print(ABORT);
      #endif
      if (digitalRead(SELECT_SW) == SELECT_LEVEL) {
        while(digitalRead(SELECT_SW) == SELECT_LEVEL) {
          delay(1);
        }
        #if DISPLAY_TYPE == 1 or DISPLAY_TYPE == 2 or DISPLAY_TYPE == 99
          u8g2.setCursor(105, 12 + ((pos) * 16));
          u8g2.print("OK");
          u8g2.sendBuffer();
        #endif
        #if DISPLAY_TYPE == 3 or DISPLAY_TYPE == 99
          gfx->setTextColor(MARKER);
          gfx->setCursor(283, 30+((pos+1) * y_offset_tft));
          gfx->print("OK");
        #endif
        if ( pos == 0) {
          j = 0;
          while ( j < 5  ) {
            jars[j].Count = 0;
            jars[j].TripCount = 0;
            j++;
          }
          setPreferences();
        }
        delay(1000);
        mode = -1;
        i = 0;
      }
    }
  }
}

void setupTare(void) {
  int j;
  int x_pos;
  tare = 0;
  initRotaries(SW_MENU, 0, 0, 4, 1);
  i = 0;
  #if DISPLAY_TYPE == 3 or DISPLAY_TYPE == 99
    gfx->fillScreen(BACKGROUND);
    bool change = false;
  #endif
  while (i == 0)  {
    if (digitalRead(button_stop_pin) == HIGH or digitalRead(switch_setup_pin) == LOW) {
      while (digitalRead(button_stop_pin) == HIGH) {
        delay(1);
      }
      mode = -1;
      return;
    }
    else if (digitalRead(SELECT_SW) == SELECT_LEVEL) {
      tare = int(SCALE_GETUNITS(10));
      if (tare > 20) {                  // Jars must be 20g minimum
         jars[getRotariesValue(SW_MENU)].Tare = tare;
      }
      #if DISPLAY_TYPE == 3 or DISPLAY_TYPE == 99
        change = true;
      #endif
      i++;
    }
    #if DISPLAY_TYPE == 1 or DISPLAY_TYPE == 2 or DISPLAY_TYPE == 99
      u8g2.setFont(u8g2_font_courB08_tf);
      u8g2.clearBuffer();
      j = 0;
      while(j < 5) {
        u8g2.setCursor(8, 10+(j*13));
        if (jars[j].Weight < 1000) {
          sprintf(output, "%4dg - %3s", jars[j].Weight, JarTypeArray[jars[j].JarType]); 
        } 
        else {
          sprintf(output, "%.1fkg - %3s", float(jars[j].Weight) / 1000, JarTypeArray[jars[j].JarType]); 
        }
        u8g2.print(output);
        u8g2.setCursor(80, 10+(j*13));
        if (jars[j].Tare > 0) { 
          sprintf(output, "%7dg", jars[j].Tare); 
          u8g2.print(output);
        }
        else {
          sprintf(output, "%8s", ST_TI02); 
          u8g2.print(output);
        }
        j++;
      }
      u8g2.setCursor(0, 10+(getRotariesValue(SW_MENU)*13) );    
      u8g2.print("*");
      u8g2.sendBuffer();
    #endif
    #if DISPLAY_TYPE == 3 or DISPLAY_TYPE == 99
      if (change) {
        gfx->fillScreen(BACKGROUND);
      }
      gfx->setTextColor(TEXT);
      gfx->setFont(Punk_Mono_Bold_240_150);
      x_pos = CenterPosX(ST_TI01, 14, 320);
      gfx->setCursor(x_pos, 25);
      gfx->println(ST_TI01);
      gfx->drawLine(0, 30, 320, 30, TEXT);
      j = 0;
      while(j < 5) {
        if (j == getRotariesValue(SW_MENU)) {
          gfx->setTextColor(MARKER);
        }
        else {
          gfx->setTextColor(TEXT);
        }
        gfx->setCursor(5, 60+(j*30));
        if (jars[j].Weight < 1000) {
          sprintf(output, "%4dg - %3s", jars[j].Weight, JarTypeArray[jars[j].JarType]); 
        } 
        else {
          sprintf(output, "%.1fkg - %3s", float(jars[j].Weight) / 1000, JarTypeArray[jars[j].JarType]); 
        }
        gfx->println(output);
        gfx->setCursor(165, 60+(j*30));
        if (jars[j].Tare > 0) { 
          sprintf(output, "%10dg", jars[j].Tare); 
          gfx->print(output);
        }
        else {
          sprintf(output, "%11s", ST_TI02); 
          gfx->print(output);
        }
        j++;
      }
    #endif
  }
  mode = -1;
  delay(2000);
}

void setupCalibration(void) {
  float weight_raw;
  #if DISPLAY_TYPE == 1 or DISPLAY_TYPE == 2 or DISPLAY_TYPE == 99
    u8g2.clearBuffer();
    u8g2.setFont(u8g2_font_courB08_tf);
    u8g2.setCursor(0, 12);    u8g2.print(SC_LI01);        //Please empty the
    u8g2.setCursor(0, 24);    u8g2.print(SC_LI02);        //scale and confirm
    u8g2.setCursor(0, 36);    u8g2.print(SC_LI03);        //with OK
    u8g2.sendBuffer();
  #endif
  #if DISPLAY_TYPE == 3 or DISPLAY_TYPE == 99
    gfx->fillScreen(BACKGROUND);
    int kali_weight_old = -1;
    gfx->setTextColor(TEXT);
    gfx->setFont(Punk_Mono_Bold_240_150);
    int x_pos = CenterPosX(SC_TO01, 14, 320);             //Calibration
    gfx->setCursor(x_pos, 25);
    gfx->println(SC_TO01);
    gfx->drawLine(0, 30, 320, 30, TEXT);
    sprintf(output, SC_LI01);                            //Please empty the
    x_pos = CenterPosX(output, 14, 320);
    gfx->setCursor(x_pos, 100); gfx->print(output);
    sprintf(output, SC_LI02);                            //scale and confirm
    x_pos = CenterPosX(output, 14, 320);
    gfx->setCursor(x_pos, 128); gfx->print(output);
    sprintf(output, SC_LI03);                            //with OK
    x_pos = CenterPosX(output, 14, 320);
    gfx->setCursor(x_pos, 156); gfx->print(output);
  #endif
  i = 1;
  while (i > 0) {
    if (digitalRead(button_stop_pin) == HIGH or digitalRead(switch_setup_pin) == LOW) {
      while (digitalRead(button_stop_pin) == HIGH) {
        delay(1);
      }
      mode = -1;
      return;
    }
    else if (digitalRead(SELECT_SW) == SELECT_LEVEL) {
      scale.set_scale();
      scale.tare(10);
      delay(500);
      i = 0;
    }
  }
  #if DISPLAY_TYPE == 3 or DISPLAY_TYPE == 99
    gfx->fillScreen(BACKGROUND);
    gfx->setTextColor(TEXT);
    gfx->setFont(Punk_Mono_Bold_240_150);
    x_pos = CenterPosX(SC_TO01, 14, 320);
    gfx->setCursor(x_pos, 25);
    gfx->println(SC_TO01);
    gfx->drawLine(0, 30, 320, 30, TEXT);
  #endif
  initRotaries(SW_MENU, cali_weight, 100, 9999, 1); 
  i = 1;
  while (i > 0) {
    if (digitalRead(button_stop_pin) == HIGH or digitalRead(switch_setup_pin) == LOW) {
      while (digitalRead(button_stop_pin) == HIGH) {
        delay(1);
      }
      mode = -1;
      return;
    }
    cali_weight = getRotariesValue(SW_MENU);
    #if DISPLAY_TYPE == 1 or DISPLAY_TYPE == 2 or DISPLAY_TYPE == 99
      u8g2.clearBuffer();
      u8g2.setFont(u8g2_font_courB08_tf);
      u8g2.setCursor(0, 12);
      u8g2.print(SC_LI04);                                        //Please set up
      sprintf(output, " %dg", cali_weight);                     //calibration weight
      u8g2.print(output);
      u8g2.setCursor(0, 24);    u8g2.print(SC_LI05);              //and confirm
      u8g2.setCursor(0, 36);    u8g2.print(SC_LI06);              //with ok
      u8g2.sendBuffer();
    #endif
    #if DISPLAY_TYPE == 3 or DISPLAY_TYPE == 99
      if (cali_weight != kali_weight_old) {
        gfx->setFont(Punk_Mono_Bold_240_150);
        gfx->setTextColor(TEXT);
        sprintf(output, "%s", SC_LI04);                            //Please set up
        x_pos = CenterPosX(output, 14, 320);
        gfx->setCursor(x_pos, 100); gfx->print(output);
        gfx->setTextColor(BACKGROUND);
        sprintf(output, "%dg", kali_weight_old);
        x_pos = CenterPosX(output, 14, 320);
        gfx->setCursor(x_pos, 128); gfx->print(output);
        gfx->setTextColor(MARKER);
        sprintf(output, "%dg", cali_weight);
        x_pos = CenterPosX(output, 14, 320);
        gfx->setCursor(x_pos, 128); gfx->print(output);
        gfx->setTextColor(TEXT);
        sprintf(output, "%s", SC_LI05);                            //and confirm
        x_pos = CenterPosX(output, 14, 320);
        gfx->setCursor(x_pos, 156); gfx->print(output);
        sprintf(output, "%s", SC_LI06);                            //with ok
        x_pos = CenterPosX(output, 14, 320);
        gfx->setCursor(x_pos, 184); gfx->print(output);
        kali_weight_old = cali_weight;
      }
    #endif
    if (digitalRead(SELECT_SW) == SELECT_LEVEL) {
      weight_raw  = scale.get_units(10);
      factor       = weight_raw / cali_weight;
      scale.set_scale(factor);
      weight_empty = scale.get_offset();    // Record scale empty weight
      #ifdef isDebug
        Serial.print("calibration_weight = ");
        Serial.println(cali_weight);
        Serial.print("weight_empty = ");
        Serial.println(weight_empty);
        Serial.print("weight_raw = ");
        Serial.println(weight_raw);
        Serial.print(" factor = ");
        Serial.println(factor);
      #endif        
      delay(1000);
      mode = -1;
      i = 0;        
    }
  }
}

void setupServoAngle(void) {
  int menuitem;
  int menuitem_used = 4;
  int lastmin  = angle_min;
  int lastfein = angle_fine;
  int lastmax  = angle_max;
  int value_alt;
  bool value_change = false;
  bool servo_live = false;
  initRotaries(SW_MENU, 0, 0, menuitem_used, 1);
  #if DISPLAY_TYPE == 1 or DISPLAY_TYPE == 2 or DISPLAY_TYPE == 99
    int y_offset = 8;
  #endif
  #if DISPLAY_TYPE == 3 or DISPLAY_TYPE == 99
    int y_offset_tft = 28;
    bool change = false;
    int value_old = -1;
    int MenupointsCount = 5;
    const char *menupoints[MenupointsCount] = {SS_MI01, SS_MI02, SS_MI03, SS_MI04, SS_MI05};
    gfx->fillScreen(BACKGROUND);
    gfx->setTextColor(TEXT);
    gfx->setFont(Punk_Mono_Bold_240_150);
    int x_pos = CenterPosX(SS_TO01, 14, 320);
    gfx->setCursor(x_pos, 25);
    gfx->println(SS_TO01);
    gfx->drawLine(0, 30, 320, 30, TEXT);
  #endif
  i = 1;
  while (i > 0) {
    if (digitalRead(button_stop_pin) == HIGH or digitalRead(switch_setup_pin) == LOW) {
      while (digitalRead(button_stop_pin) == HIGH) {
        delay(1);
      }
      angle_min  = lastmin;
      angle_fine = lastfein;
      angle_max  = lastmax;
      if (servo_live == true) {
        SERVO_WRITE(angle_min);
      }
      mode = -1;
      return;
    }
    if (value_change == false) {
      menuitem = getRotariesValue(SW_MENU);
      #if DISPLAY_TYPE == 3 or DISPLAY_TYPE == 99
        pos = menuitem;
      #endif
      if (menuitem == menuitem_used) {
        menuitem = 6;
      }
    }
    else {
      switch (menuitem) {
        case 0: servo_live  = getRotariesValue(SW_MENU);
                break;
        case 1: angle_min  = getRotariesValue(SW_MENU);
                if (servo_live == true) SERVO_WRITE(angle_min);
                break;
        case 2: angle_fine = getRotariesValue(SW_MENU);
                if (servo_live == true) SERVO_WRITE(angle_fine);
                break;
        case 3: angle_max  = getRotariesValue(SW_MENU);
                if (servo_live == true) SERVO_WRITE(angle_max);
                break;
      }
    }
    #if DISPLAY_TYPE == 1 or DISPLAY_TYPE == 2 or DISPLAY_TYPE == 99
      u8g2.setFont(u8g2_font_courB08_tf);
      u8g2.clearBuffer();
      u8g2.setCursor(10, 1 * y_offset);
      u8g2.print(SS_MI01);
      u8g2.setCursor(95, 1 * y_offset);
      sprintf(output,"%5s", (servo_live==false?OFF:ON));
      u8g2.print(output);
      u8g2.setCursor(10, 2 * y_offset);
      u8g2.print(SS_MI02);
      u8g2.setCursor(95, 2 * y_offset);
      sprintf(output,"%4d°", angle_min);
      u8g2.print(output);
      u8g2.setCursor(10, 3 * y_offset);
      u8g2.print(SS_MI03);
      u8g2.setCursor(95, 3 * y_offset);
      sprintf(output,"%4d°", angle_fine);
      u8g2.print(output);
      u8g2.setCursor(10, 4 * y_offset);
      u8g2.print(SS_MI04);
      u8g2.setCursor(95, 4 * y_offset);
      sprintf(output,"%4d°", angle_max);
      u8g2.print(output);
      u8g2.setCursor(10, (7 * y_offset) + 5);
      u8g2.print(SS_MI05);
      if (value_change == false && menuitem < menuitem_used) {
        u8g2.setCursor(1, 10+((menuitem)*y_offset)); u8g2.print("*");
      }
      else if (value_change == true && menuitem < menuitem_used) {
        u8g2.setCursor(1, 8+((menuitem)*y_offset)); u8g2.print("-");
      }
      else if (value_change == false && menuitem == 6) {
        u8g2.setCursor(1, 10+((menuitem)*y_offset) + 5); u8g2.print("*");
      }
      u8g2.sendBuffer();
    #endif
    #if DISPLAY_TYPE == 3 or DISPLAY_TYPE == 99
      int j = 0;
      while(j < MenupointsCount) {
        gfx->setTextColor(TEXT);
        if (j == pos and value_change == false) {
          gfx->setTextColor(MARKER);
        }
        if (j < MenupointsCount - 1) {
          gfx->setCursor(5, 30+((j+1) * y_offset_tft));
          gfx->print(menupoints[j]);
          if (j == pos and value_change == true) {
            //sprintf(output,"(%d°)", value_alt);
            //if (j > 0) {
            //  gfx->setCursor(170, 30+((j+1) * y_offset_tft));
            //  gfx->print(output);
            //}
            gfx->setTextColor(MARKER);
          }
          switch (j) {
            case 0: sprintf(output,"%s", servo_live==false?OFF:ON);
                    if (value_old != servo_live and value_change == true and j == pos) {
                      change = true;
                      value_old = servo_live;
                    }
                    break;
            case 1: sprintf(output,"%d°", angle_min);
                    if (value_old != angle_min and value_change == true and j == pos) {
                      change = true;
                      value_old = angle_min;
                    }
                    break;
            case 2: sprintf(output,"%d°", angle_fine);
                    if (value_old != angle_fine and value_change == true and j == pos) {
                      change = true;
                      value_old = angle_fine;
                    }
                    break;
            case 3: sprintf(output,"%d°", angle_max);
                    if (value_old != angle_max and value_change == true and j == pos) {
                      change = true;
                      value_old = angle_max;
                    }
                    break;
          }
          if (change) {
            gfx->fillRect(251, 27+((j+1) * y_offset_tft)-19, 64, 25, BACKGROUND);
            change = false;
          }
          int y = get_length(output);
          gfx->setCursor(320 - y * 14 - 5, 30+((j+1) * y_offset_tft));
          gfx->print(output);
        }
        else {
          gfx->setCursor(5, 30+(7 * y_offset_tft));
          gfx->print(menupoints[j]);
        }
        j++;
      }
    #endif
    if (digitalRead(SELECT_SW) == SELECT_LEVEL && menuitem < menuitem_used  && value_change == false) {
      while(digitalRead(SELECT_SW) == SELECT_LEVEL) {
        delay(1);
      }
      switch (menuitem) { 
        case 0: initRotaries(SW_MENU, servo_live, 0, 1, 1);
              break;
        case 1: initRotaries(SW_MENU, angle_min, angle_hard_min, angle_fine, 1);
              value_alt = lastmin;
              break;
        case 2: initRotaries(SW_MENU, angle_fine, angle_min, angle_max, 1);
              value_alt = lastfein;
              break;
        case 3: initRotaries(SW_MENU, angle_max, angle_fine, angle_hard_max, 1);
              value_alt = lastmax;
              break;
      }
      #if DISPLAY_TYPE == 3 or DISPLAY_TYPE == 99
        value_old = -1;
      #endif
      value_change = true;
    }
    if (digitalRead(SELECT_SW) == SELECT_LEVEL && menuitem < menuitem_used  && value_change == true) {
      while(digitalRead(SELECT_SW) == SELECT_LEVEL) {
        delay(1);
      }
      if (servo_live == true) {
        SERVO_WRITE(angle_min);
      }
      initRotaries(SW_MENU, menuitem, 0, menuitem_used, 1);
      value_change = false;
      #if DISPLAY_TYPE == 3 or DISPLAY_TYPE == 99
        gfx->fillRect(170, 30+y_offset_tft+3, 96, 3*y_offset_tft + 2, BACKGROUND);
      #endif
    }
    if (digitalRead(SELECT_SW) == SELECT_LEVEL && menuitem == 6) {
      while(digitalRead(SELECT_SW) == SELECT_LEVEL) {
        delay(1);
      }
      #if DISPLAY_TYPE == 1 or DISPLAY_TYPE == 2 or DISPLAY_TYPE == 99
        u8g2.setCursor(111, ((menuitem + 1) * y_offset) + 5);
        u8g2.print("OK");
        u8g2.sendBuffer();
      #endif
      #if DISPLAY_TYPE == 3 or DISPLAY_TYPE == 99
        gfx->setCursor(283, 30+(7 * y_offset_tft));
        gfx->print("OK");
      #endif
      mode = -1;
      delay(1000);
      i = 0;
    }
  }
}

void setupAutomatik(void) {
  int menuitem;
  int menuitem_used = 6;  //A.P.
  int lastautostart     = autostart;
  int lastjartolerance  = jartolerance;
  int lastautocorrection = autocorrection;
  int lastoverfill        = overfill_gr;
  int correction_old = correction;
  int lastintweight = intweight; //A.P.
  bool value_change = false;
  #if DISPLAY_TYPE == 1 or DISPLAY_TYPE == 2 or DISPLAY_TYPE == 99
    int y_offset = 8;
  #endif
  #if DISPLAY_TYPE == 3 or DISPLAY_TYPE == 99
    int y_offset_tft = 28;
    bool change = false;
    int value_old = -1;
    int MenupointsCount = 7;
    const char *menupoints[MenupointsCount] = {SA_MI01, SA_MI02, SA_MI03, SA_MI04, SA_MI05, SA_MI06, SA_MI07};
    gfx->fillScreen(BACKGROUND);
    gfx->setTextColor(TEXT);
    gfx->setFont(Punk_Mono_Bold_240_150);
    int x_pos = CenterPosX(SA_TO01, 14, 320);
    gfx->setCursor(x_pos, 25);
    gfx->println(SA_TO01);
    gfx->drawLine(0, 30, 320, 30, TEXT);
  #endif
  initRotaries(SW_MENU, 0, 0, menuitem_used, 1);
  i = 1;
  while (i > 0) {
    if (digitalRead(button_stop_pin) == HIGH or digitalRead(switch_setup_pin) == LOW) {
      while (digitalRead(button_stop_pin) == HIGH) {
        delay(1);
      }
      autostart     = lastautostart;
      autocorrection = lastautocorrection;
      overfill_gr     = lastoverfill;
      jartolerance  = lastjartolerance;
      correction     = correction_old;
      intweight    = lastintweight; //A.P.
      setRotariesValue(SW_CORRECTION, correction_old);
      rotary_select = SW_MENU;
      mode = -1;
      return;
    }
    if (value_change == false) {
      menuitem = getRotariesValue(SW_MENU);
      #if DISPLAY_TYPE == 3 or DISPLAY_TYPE == 99
        pos = menuitem;
      #endif
      if (menuitem == menuitem_used) {
        menuitem = 7;  //A.P.
      }
    }
    else {
      switch (menuitem) {
        case 0: autostart     = getRotariesValue(SW_MENU);
                break;
        case 1: jartolerance  = getRotariesValue(SW_MENU);
                break;
        case 2: correction     = getRotariesValue(SW_CORRECTION);
                break;
        case 3: autocorrection = getRotariesValue(SW_MENU);
                break;
        case 4: overfill_gr     = getRotariesValue(SW_MENU);
                break;
        case 5: intweight    = getRotariesValue(SW_MENU);  //A.P.
                break; //A.P.
      }
    }
    #if DISPLAY_TYPE == 1 or DISPLAY_TYPE == 2 or DISPLAY_TYPE == 99
      u8g2.setFont(u8g2_font_courB08_tf);
      u8g2.clearBuffer();
      u8g2.setCursor(10, 1 * y_offset);
      u8g2.print(SA_MI01);
      u8g2.setCursor(95, 1 * y_offset);
      sprintf(output,"%5s", (autostart==0?OFF:ON));
      u8g2.print(output);
      u8g2.setCursor(10, 2 * y_offset);
      u8g2.print(SA_MI02);
      u8g2.setCursor(95, 2 * y_offset);
      if (jartolerance > 0) {
        if (jartolerance > 9) {
          sprintf(output," %c%2dg", 177, jartolerance);
        }
        else {
          sprintf(output,"  %c%1dg", 177, jartolerance);
        }
      }
      else {
        sprintf(output,"%4dg", jartolerance);
      }
      u8g2.print(output);
      u8g2.setCursor(10, 3 * y_offset);
      u8g2.print(SA_MI03);
      u8g2.setCursor(95, 3 * y_offset);
      sprintf(output,"%4dg", correction);
      u8g2.print(output);
      u8g2.setCursor(10, 4 * y_offset);
      u8g2.print(SA_MI04);
      u8g2.setCursor(95, 4 * y_offset);
      sprintf(output,"%5s", (autocorrection==0?OFF:ON));
      u8g2.print(output);
      u8g2.setCursor(10, 5 * y_offset);
      u8g2.print(SA_MI05);
      u8g2.setCursor(95, 5 * y_offset);
      sprintf(output,"%4dg", overfill_gr);
      u8g2.print(output);
      u8g2.setCursor(10, 6 * y_offset);
      u8g2.print(SA_MI06);
      u8g2.setCursor(95, 6 * y_offset);
      if (intweight > 0) {
        sprintf(output,"%4dg", intweight);
      }
      else {
        sprintf(output,"%5s", OFF);
      }
      u8g2.print(output);
      u8g2.setCursor(10, (7 * y_offset) + 5);
      u8g2.print(SA_MI07);
      // Show position in menu. "*" when not selected, arrow when selected
      if (value_change == false && menuitem < menuitem_used) {
        u8g2.setCursor(1, 10+((menuitem)*y_offset)); u8g2.print("*");
      }
      else if (value_change == true && menuitem < menuitem_used) {
        u8g2.setCursor(1, 8+((menuitem)*y_offset)); u8g2.print("-");
      }
      else if (value_change == false && menuitem == 7) {
        u8g2.setCursor(1, 10+((menuitem - 1)*y_offset) + 5); u8g2.print("*");
      }
      u8g2.sendBuffer();
    #endif
    #if DISPLAY_TYPE == 3 or DISPLAY_TYPE == 99
      int j = 0;
      while(j < MenupointsCount) {
        gfx->setTextColor(TEXT);
        if (j == pos and value_change == false) {
          gfx->setTextColor(MARKER);
        }
        if (j < MenupointsCount - 1) {
          gfx->setCursor(5, 30+((j+1) * y_offset_tft));
          gfx->print(menupoints[j]);
          if (j == pos and value_change == true) {
            gfx->setTextColor(MARKER);
          }
          switch (j) {
            case 0: sprintf(output,"%s", autostart==false?OFF:ON);
                    if (value_old != autostart and value_change == true and j == pos) {
                      change = true;
                      value_old = autostart;
                    }
                    break;
            case 1: if (jartolerance > 0) {
                      sprintf(output,"±%dg", jartolerance);
                    }
                    else {
                      sprintf(output,"%dg", jartolerance);
                    }
                    if (value_old != jartolerance and value_change == true and j == pos) {
                      change = true;
                      value_old = jartolerance;
                    }
                    break;
            case 2: sprintf(output,"%dg", correction);
                    if (value_old != correction and value_change == true and j == pos) {
                      change = true;
                      value_old = correction;
                    }
                    break;
            case 3: sprintf(output,"%s", autocorrection==false?OFF:ON);
                    if (value_old != autocorrection and value_change == true and j == pos) {
                      change = true;
                      value_old = autocorrection;
                    }
                    break;
            case 4: sprintf(output,"%dg", overfill_gr);
                    if (value_old != overfill_gr and value_change == true and j == pos) {
                      change = true;
                      value_old = overfill_gr;
                    }
                    break;
            case 5: if (intweight > 0) {
                      sprintf(output,"%dg", intweight);
                    }
                    else {
                      sprintf(output,"%s", OFF);
                    }
                    if (value_old != intweight and value_change == true and j == pos) {
                      change = true;
                      value_old = intweight;
                    }
                    break;
          }
          if (change) {
            gfx->fillRect(251, 27+((j+1) * y_offset_tft)-19, 64, 27, BACKGROUND);
            change = false;
          }
          int y = get_length(output);
          gfx->setCursor(320 - y * 14 - 5, 30+((j+1) * y_offset_tft));
          gfx->print(output);
        }
        else {
          gfx->setCursor(5, 30+(7 * y_offset_tft));
          gfx->print(menupoints[j]);
        }
        j++;
      }
    #endif
    // Selected menupoint to change
    if (digitalRead(SELECT_SW) == SELECT_LEVEL && menuitem < menuitem_used  && value_change == false) { 
      while(digitalRead(SELECT_SW) == SELECT_LEVEL) {
        delay(1);
      }
      switch (menuitem) { 
        case 0: rotary_select = SW_MENU;
                initRotaries(SW_MENU, autostart, 0, 1, 1);
                break;
        case 1: rotary_select = SW_MENU;
                initRotaries(SW_MENU, jartolerance, 0, 99, 1);
                break;
        case 2: rotary_select = SW_CORRECTION;
                break;
        case 3: rotary_select = SW_MENU;
                initRotaries(SW_MENU, autocorrection, 0, 1, 1);
                break;
        case 4: rotary_select = SW_MENU;
                initRotaries(SW_MENU, overfill_gr, 0, 99, 1);
                break;
        case 5: rotary_select = SW_MENU; //A.P.
                initRotaries(SW_MENU, intweight, 0, 99, 1); //A.P.
                break; //a.P.
      }
      #if DISPLAY_TYPE == 3 or DISPLAY_TYPE == 99
        value_old = -1;
      #endif
      value_change = true;
    }
    // Taking changes in menupoint into acount
    if (digitalRead(SELECT_SW) == SELECT_LEVEL && menuitem < menuitem_used  && value_change == true) {
      while(digitalRead(SELECT_SW) == SELECT_LEVEL) {
        delay(1);
      }
      rotary_select = SW_MENU;
      initRotaries(SW_MENU, menuitem, 0, menuitem_used, 1);
      value_change = false;
    }
    // Leave menu 
    if (digitalRead(SELECT_SW) == SELECT_LEVEL && menuitem == 7) {  //A.P.
      while(digitalRead(SELECT_SW) == SELECT_LEVEL) {
        delay(1);
      }
      #if DISPLAY_TYPE == 1 or DISPLAY_TYPE == 2 or DISPLAY_TYPE == 99
        u8g2.setCursor(111, (menuitem * y_offset) + 5);
        u8g2.print("OK");
        u8g2.sendBuffer();
      #endif
      #if DISPLAY_TYPE == 3 or DISPLAY_TYPE == 99
        gfx->setCursor(283, 30+(7 * y_offset_tft));
        gfx->print("OK");
      #endif
      delay(1000);
      mode = -1;
      i = 0;
    }
  }
  rotary_select = SW_MENU;
}

void setupFuellmenge(void) {
   #if DISPLAY_TYPE == 3 or DISPLAY_TYPE == 99
    int y_offset_tft = 28;
    String value_old = "";
    gfx->fillScreen(BACKGROUND);
    gfx->setTextColor(TEXT);
    gfx->setFont(Punk_Mono_Bold_240_150);
    int x_pos = CenterPosX(SF_TO01, 14, 320);
    gfx->setCursor(x_pos, 25);
    gfx->println(SF_TO01);
    gfx->drawLine(0, 30, 320, 30, TEXT);
  #endif
  int j;
  initRotaries(SW_MENU, fquantity_index, 0, 4, 1);
  i = 1;
  while (i > 0) {
    if (digitalRead(button_stop_pin) == HIGH or digitalRead(switch_setup_pin) == LOW) {
      while (digitalRead(button_stop_pin) == HIGH) {
        delay(1);
      }
      mode = -1;
      return;
    }
    pos = getRotariesValue(SW_MENU);
    #if DISPLAY_TYPE == 1 or DISPLAY_TYPE == 2 or DISPLAY_TYPE == 99
      u8g2.clearBuffer();
      u8g2.setFont(u8g2_font_courB08_tf);
      j = 0;
      while(j < 5) {
        u8g2.setCursor(20, 10+(j*13)); 
        sprintf(output, "%4dg      %3s", jars[j].Weight, JarTypeArray[jars[j].JarType]);
        u8g2.print(output);
        j++;
      }
      u8g2.setCursor(10, 10+(getRotariesValue(SW_MENU)*13));    
      u8g2.print("*");
      u8g2.sendBuffer();
    #endif
    #if DISPLAY_TYPE == 3 or DISPLAY_TYPE == 99
        j = 0;
        while(j < 5) {
          if (j == pos) {
            gfx->setTextColor(MARKER);
          }
          else {
            gfx->setTextColor(TEXT);
          }
          gfx->setCursor(10, 30+((j+1) * y_offset_tft));
          sprintf(output, "%4dg", jars[j].Weight);
          gfx->print(output);
          gfx->setCursor(110, 30+((j+1) * y_offset_tft));
          if (JarTypeArray[jars[j].JarType] == "DIB") {
            gfx->print("DE Imker Bund");
          }
          else if (JarTypeArray[jars[j].JarType] == "TOF") {
            gfx->print("TwistOff");
          }
          else if (JarTypeArray[jars[j].JarType] == "DEE") {
            gfx->print("DeepTwist");
          }
          else if (JarTypeArray[jars[j].JarType] == "SPZ") {
            gfx->print("Spezial");
          }
          j++;
       }
    #endif
    if (digitalRead(SELECT_SW) == SELECT_LEVEL) { // Fill quantity chosen
      while((digitalRead(SELECT_SW) == SELECT_LEVEL)) {
        delay(1);
      }
      i = 0;
    }
  }
  i = 1;
  initRotaries(SW_MENU, weight2step(jars[pos].Weight) , 25, weight2step(MAXIMUMWEIGHT), 1);
  while (i > 0){
    if ((digitalRead(button_stop_pin)) == HIGH  or digitalRead(switch_setup_pin) == LOW) {
      mode = -1;
      return;
    }
    jars[pos].Weight = step2weight(getRotariesValue(SW_MENU));
    #if DISPLAY_TYPE == 1 or DISPLAY_TYPE == 2 or DISPLAY_TYPE == 99
      u8g2.clearBuffer();    
      j = 0;
      while(j < 5) {
        u8g2.setCursor(20, 10+(j*13));
        if (j == pos){ 
          u8g2.setFont(u8g2_font_open_iconic_arrow_1x_t);
          u8g2.drawGlyph(5, 11+(j*13), 0x42);
          u8g2.drawGlyph(55, 11+(j*13), 0x41);
          u8g2.setFont(u8g2_font_courB08_tf);   
          sprintf(output, "%4dg      %3s", jars[j].Weight, JarTypeArray[jars[j].JarType]);  
        }
        else {
          sprintf(output, "%4dg      %3s", jars[j].Weight, JarTypeArray[jars[j].JarType]);
        }
        u8g2.print(output);
        j++;
      }                     
      u8g2.sendBuffer();
    #endif
    #if DISPLAY_TYPE == 3 or DISPLAY_TYPE == 99
      j = 0;
      while(j < 5) {
        gfx->setCursor(10, 30+((j+1) * y_offset_tft));
        if (j == pos) {
          if (value_old != String(jars[j].Weight)) {
            gfx->fillRect(0, 27+((j+1) * y_offset_tft)-19, 100, 27, BACKGROUND);
            value_old = String(jars[j].Weight);
          }
          gfx->setTextColor(MARKER);
        }
        sprintf(output, "%4dg", jars[j].Weight);
        gfx->print(output);
        gfx->setTextColor(TEXT);
        gfx->setCursor(110, 30+((j+1) * y_offset_tft));
        if (JarTypeArray[jars[j].JarType] == "DIB") {
          gfx->print("DE Imker Bund");
        }
        else if (JarTypeArray[jars[j].JarType] == "TOF") {
          gfx->print("TwistOff");
        }
        else if (JarTypeArray[jars[j].JarType] == "DEE") {
          gfx->print("DeepTwist");
        }
        else if (JarTypeArray[jars[j].JarType] == "SPZ") {
          gfx->print("Spezial");
        }
        j++;
      }
    #endif
    if (digitalRead(SELECT_SW) == SELECT_LEVEL) { // Weight confirmed
      while((digitalRead(SELECT_SW) == SELECT_LEVEL)) {
        delay(1);
      }
      i = 0;
    }
  }
  i = 1;
  initRotaries(SW_MENU, jars[pos].JarType, 0, sizeof(JarTypeArray)/sizeof(JarTypeArray[0]) - 1, 1);
  while (i > 0){ 
    if (digitalRead(button_stop_pin) == HIGH or digitalRead(switch_setup_pin) == LOW) {
      while (digitalRead(button_stop_pin) == HIGH) {
        delay(1);
      }
      mode = -1;
      return;
    }
    jars[pos].JarType = getRotariesValue(SW_MENU);
    #if DISPLAY_TYPE == 1 or DISPLAY_TYPE == 2 or DISPLAY_TYPE == 99
      u8g2.clearBuffer();
      j = 0;
      while(j < 5) {
        u8g2.setCursor(20, 10+(j*13));
        if (j == pos){
          u8g2.setFont(u8g2_font_open_iconic_arrow_1x_t);
          u8g2.drawGlyph(72, 11+(j*13), 0x42);
          u8g2.drawGlyph(108, 11+(j*13), 0x41);
          u8g2.setFont(u8g2_font_courB08_tf);
          sprintf(output, "%4dg      %3s", jars[j].Weight, JarTypeArray[jars[j].JarType]);
        }
        else {
          sprintf(output, "%4dg      %3s", jars[j].Weight, JarTypeArray[jars[j].JarType]);
        }
        u8g2.print(output);
        j++;
      }
      u8g2.sendBuffer();
    #endif
    #if DISPLAY_TYPE == 3 or DISPLAY_TYPE == 99
      j = 0;
      while(j < 5) {
        gfx->setCursor(10, 30+((j+1) * y_offset_tft));
        sprintf(output, "%4dg", jars[j].Weight);
        gfx->print(output);
        if (j == pos) {
          if (value_old != String(JarTypeArray[jars[j].JarType])) {
            gfx->fillRect(100, 27+((j+1) * y_offset_tft)-19, 220, 27, BACKGROUND);
            value_old = String(JarTypeArray[jars[j].JarType]);
          }
          gfx->setTextColor(MARKER);
        }
        gfx->setCursor(110, 30+((j+1) * y_offset_tft));
        if (JarTypeArray[jars[j].JarType] == "DIB") {
          gfx->print("DE Imker Bund");
        }
        else if (JarTypeArray[jars[j].JarType] == "TOF") {
          gfx->print("TwistOff");
        }
        else if (JarTypeArray[jars[j].JarType] == "DEE") {
          gfx->print("DeepTwist");
        }
        else if (JarTypeArray[jars[j].JarType] == "SPZ") {
          gfx->print("Spezial");
        }
        gfx->setTextColor(TEXT);
        j++;
      }
    #endif
    if (digitalRead(SELECT_SW) == SELECT_LEVEL) { //Jar type confirmed
      while((digitalRead(SELECT_SW) == SELECT_LEVEL)) {
        delay(1);
      }
      i = 0;
    }
  }
  fquantity = jars[pos].Weight;
  tare   = jars[pos].Tare;
  fquantity_index = pos; 
  mode = -1;
  i = 0;
}

void setupParameter(void) {
  int menuitem;
  int menuitem_used = 4;
  int lastbuzzer    = buzzermode;
  int lastled       = ledmode;
  int lastlogo      = showlogo;
  int lastcredits   = showcredits;
  int lastcolor_scheme = color_scheme;
  int lastcolor_marker = color_marker;
  bool value_change = false;
  #if DISPLAY_TYPE == 1 or DISPLAY_TYPE == 2 or DISPLAY_TYPE == 99
    int y_offset = 8;
  #endif
  #if DISPLAY_TYPE == 3 or DISPLAY_TYPE == 99
    int y_offset_tft = 28;
    int value_old = -1;
    bool change = false;
    bool change_scheme = true;
    bool change_marker = true;
    menuitem_used = 6;
    int MenupointsCount = 7;
    const char *menupoints[MenupointsCount] = {SP_MI01, SP_MI02, SP_MI03, SP_MI04, SP_MI05, SP_MI06, SP_MI07};
  #endif
  initRotaries(SW_MENU, 0, 0, menuitem_used, 1);
  i = 1;
  while (i > 0) {
    if (digitalRead(button_stop_pin) == HIGH or digitalRead(switch_setup_pin) == LOW) {
      while (digitalRead(button_stop_pin) == HIGH) {
        delay(1);
      }
      buzzermode = lastbuzzer;
      ledmode = lastled;
      showlogo = lastlogo;
      showcredits = lastcredits;
      color_scheme = lastcolor_scheme;
      color_marker = lastcolor_marker;
      mode = -1;
      return;
    }
    if (value_change == false) {
      menuitem = getRotariesValue(SW_MENU);
      #if DISPLAY_TYPE == 3 or DISPLAY_TYPE == 99
        pos = menuitem;
      #endif
      if (menuitem == menuitem_used) {
        menuitem = 6;
      }
    }
    else {
      switch (menuitem) {
        case 0: buzzermode    = getRotariesValue(SW_MENU);
                break;
        case 1: ledmode       = getRotariesValue(SW_MENU);
                break;
        case 2: showlogo      = getRotariesValue(SW_MENU);
                break;
        case 3: showcredits   = getRotariesValue(SW_MENU);
                break;
        case 4: color_scheme  = getRotariesValue(SW_MENU);
                break;
        case 5: color_marker  = getRotariesValue(SW_MENU);
                break;
      }
    }
    // Menu
    #if DISPLAY_TYPE == 1 or DISPLAY_TYPE == 2 or DISPLAY_TYPE == 99
      u8g2.setFont(u8g2_font_courB08_tf);
      u8g2.clearBuffer();
      u8g2.setCursor(10, 1 * y_offset);
      u8g2.print(SP_MI01);
      u8g2.setCursor(95, 1 * y_offset);
      sprintf(output,"%5s", (buzzermode==0?OFF:ON));
      u8g2.print(output);
      u8g2.setCursor(10, 2 * y_offset);
      u8g2.print(SP_MI02);
      u8g2.setCursor(95, 2 * y_offset);
      sprintf(output,"%5s", (ledmode==0?OFF:ON));
      u8g2.print(output);
      u8g2.setCursor(10, 3 * y_offset);
      u8g2.print(SP_MI03);
      u8g2.setCursor(95, 3 * y_offset);
      sprintf(output,"%5s", (showlogo==0?OFF:ON));
      u8g2.print(output);
      u8g2.setCursor(10, 4 * y_offset);
      u8g2.print(SP_MI04);
      u8g2.setCursor(95, 4 * y_offset);
      sprintf(output,"%5s", (showcredits==0?OFF:ON));
      u8g2.print(output);
      u8g2.setCursor(10, (7 * y_offset) + 5);
      u8g2.print(SP_MI07);
      // Show position in menu. "*" when not selected, arrow when selected
      if (value_change == false && menuitem < menuitem_used) {
        u8g2.setCursor(1, 10+((menuitem)*y_offset)); u8g2.print("*");
      }
      else if (value_change == true && menuitem < menuitem_used) {
        u8g2.setCursor(1, 8+((menuitem)*y_offset)); u8g2.print("-");
      }
      else if (value_change == false && menuitem == 6) {
        u8g2.setCursor(1, 10+((menuitem)*y_offset) + 5); u8g2.print("*");
      }
      u8g2.sendBuffer();
    #endif
    #if DISPLAY_TYPE == 3 or DISPLAY_TYPE == 99
      if(change_scheme) {
        gfx->fillScreen(BACKGROUND);
        gfx->setTextColor(TEXT);
        gfx->setFont(Punk_Mono_Bold_240_150);
        int x_pos = CenterPosX(SP_TO01, 14, 320);
        gfx->setCursor(x_pos, 25);
        gfx->println(SP_TO01);
        gfx->drawLine(0, 30, 320, 30, TEXT);
        change_scheme = false;
        change_marker = true;
      }
      int j = 0;
      while(j < MenupointsCount) {
        gfx->setTextColor(TEXT);
        if (j == pos and value_change == false) {
          gfx->setTextColor(MARKER);
        }
        if (j < MenupointsCount - 1) {
          gfx->setCursor(5, 30+((j+1) * y_offset_tft));
          gfx->print(menupoints[j]);
          if (j == pos and value_change == true) {
            gfx->setTextColor(MARKER);
          }
          switch (j) {
            case 0: sprintf(output,"%s", buzzermode==false?OFF:ON);
                    if (value_old != buzzermode and value_change == true and j == pos) {
                      change = true;
                      value_old = buzzermode;
                    }
                    break;
            case 1: sprintf(output,"%s", ledmode==false?OFF:ON);
                    if (value_old != ledmode and value_change == true and j == pos) {
                      change = true;
                      value_old = ledmode;
                    }
                    break;
            case 2: sprintf(output,"%s", showlogo==false?OFF:ON);
                    if (value_old != showlogo and value_change == true and j == pos) {
                      change = true;
                      value_old = showlogo;
                    }
                    break;
            case 3: sprintf(output,"%s", showcredits==false?OFF:ON);
                    if (value_old != showcredits and value_change == true and j == pos) {
                      change = true;
                      value_old = showcredits;
                    }
                    break;
            case 4: sprintf(output,"%s", color_scheme==false?DARK:LIGHT);
                    if (value_old != color_scheme and value_change == true and j == pos) {
                      change = true;
                      change_scheme = true;
                      value_old = color_scheme;
                      tft_colors();
                    }
                    break;
            case 5: sprintf(output,"");
                    if (value_old != color_marker and value_change == true and j == pos) {
                      change = true;
                      change_marker = true;
                      value_old = color_marker;
                      tft_marker();
                    }
                    break;
          }
          if (change) {
            gfx->fillRect(211, 27+((j+1) * y_offset_tft)-19, 104, 25, BACKGROUND);
            change = false;
          }
          if (change_marker) {
            gfx->fillRect(265, 27+(6 * y_offset_tft)-17, 45, 21, MARKER);
            change_marker = false;
          }
          int y = get_length(output);
          gfx->setCursor(320 - y * 14 - 5, 30+((j+1) * y_offset_tft));
          gfx->print(output);
        }
        else {
          gfx->setCursor(5, 30+(7 * y_offset_tft));
          gfx->print(menupoints[j]);
        }
        j++;
      }
    #endif
    // Selected menupoint to change
    if (digitalRead(SELECT_SW) == SELECT_LEVEL && menuitem < menuitem_used  && value_change == false) {
      while(digitalRead(SELECT_SW) == SELECT_LEVEL) {
        delay(1);
      } 
      switch (menuitem) { 
        case 0: initRotaries(SW_MENU, buzzermode, 0, 1, 1);
                break;
        case 1: initRotaries(SW_MENU, ledmode, 0, 1, 1);
                break;
        case 2: initRotaries(SW_MENU, showlogo, 0, 1, 1);
                break;
        case 3: initRotaries(SW_MENU, showcredits, 0, 1, 1);
                break;
        case 4: initRotaries(SW_MENU, color_scheme, 0, 1, 1);
                break;
        case 5: initRotaries(SW_MENU, color_marker, 0, 11, 1);
                break;
      }
      #if DISPLAY_TYPE == 3 or DISPLAY_TYPE == 99
        value_old = -1;
      #endif
      value_change = true;
    }
    // Taking changes in menupoint into account
    if (digitalRead(SELECT_SW) == SELECT_LEVEL && menuitem < menuitem_used  && value_change == true) {
      while(digitalRead(SELECT_SW) == SELECT_LEVEL) {
        delay(1);
      }
      initRotaries(SW_MENU, menuitem, 0, menuitem_used, 1);
      value_change = false;
    }
    // Leave menu
    if (digitalRead(SELECT_SW) == SELECT_LEVEL && menuitem == 6) {
      while(digitalRead(SELECT_SW) == SELECT_LEVEL) {
        delay(1);
      }
      #if DISPLAY_TYPE == 1 or DISPLAY_TYPE == 2 or DISPLAY_TYPE == 99
        u8g2.setCursor(111, ((menuitem + 1) * y_offset) + 5);
        u8g2.print("OK");
        u8g2.sendBuffer();
      #endif
      #if DISPLAY_TYPE == 3 or DISPLAY_TYPE == 99
        gfx->setCursor(283, 30+(7 * y_offset_tft));
        gfx->print("OK");
      #endif
      delay(1000);
      mode = -1;
      i = 0;
    }
  }
}

void setupClearPrefs(void) {
  initRotaries(SW_MENU, 2, 0, 2, 1);
  int x_pos;
  #if DISPLAY_TYPE == 1 or DISPLAY_TYPE == 2 or DISPLAY_TYPE == 99
    int y_offset = 8;
  #endif
  #if DISPLAY_TYPE == 3 or DISPLAY_TYPE == 99
    int y_offset_tft = 28;
    gfx->fillScreen(BACKGROUND);
    int MenupointsCount = 3;
    const char *menupoints[MenupointsCount - 1] = {CP_MI01, CP_MI02, RETURN};
    gfx->setTextColor(TEXT);
    gfx->setFont(Punk_Mono_Bold_240_150);
    x_pos = CenterPosX(CP_TO01, 14, 320);
    gfx->setCursor(x_pos, 25);
    gfx->println(CP_TO01);
    gfx->drawLine(0, 30, 320, 30, TEXT);
  #endif
  i = 1;
  while (i > 0) {
    if (digitalRead(button_stop_pin) == HIGH  or digitalRead(switch_setup_pin) == LOW) {
      while (digitalRead(button_stop_pin) == HIGH) {
        delay(1);
      }
      mode = -1;
      return;
    }
    pos = getRotariesValue(SW_MENU);
    #if DISPLAY_TYPE == 1 or DISPLAY_TYPE == 2 or DISPLAY_TYPE == 99
      u8g2.setFont(u8g2_font_courB08_tf);
      u8g2.clearBuffer();
      u8g2.setCursor(10, 1 * y_offset);
      u8g2.print(CP_MI01);
      u8g2.setCursor(10, 2 * y_offset);
      u8g2.print(CP_MI02);
      u8g2.setCursor(10, (7 * y_offset) + 5);
      u8g2.print(RETURN);
      if (pos == 2) {
        u8g2.setCursor(0, (7 * y_offset) + 7);
      }
      else {
        u8g2.setCursor(0, (pos + 1) * y_offset + 2);
      }
      u8g2.print("*");
      u8g2.sendBuffer();
    #endif
    #if DISPLAY_TYPE == 3 or DISPLAY_TYPE == 99
      int j = 0;
      while(j < MenupointsCount) {
        if (j == pos) {
          gfx->setTextColor(MARKER);
        }
        else {
          gfx->setTextColor(TEXT);
        }
        if (j < MenupointsCount - 1) {
          gfx->setCursor(10, 30+((j+1) * y_offset_tft));
          gfx->print(menupoints[j]);
        }
        else {
          gfx->setCursor(10, 30+(7 * y_offset_tft));
          gfx->print(menupoints[j]);
        }
        j++;
      }
    #endif
    if ((digitalRead(SELECT_SW)) == SELECT_LEVEL) {  
      while(digitalRead(SELECT_SW) == SELECT_LEVEL) {
        delay(1);
      }
      if (pos == 0) {
        preferences.begin("EEPROM", false);
        preferences.clear();
        preferences.end();
        //Go for restart right away
        ESP.restart();
      }
      else if (pos == 1) {
        nvs_flash_erase();    // erase the NVS partition and...
        nvs_flash_init();     // initialize the NVS partition.
        //Go for restart right away
        ESP.restart();
      }
      #if DISPLAY_TYPE == 1 or DISPLAY_TYPE == 2 or DISPLAY_TYPE == 99
        u8g2.setCursor(115, (7 * y_offset) + 5);   
        u8g2.print("OK");
        u8g2.sendBuffer();
      #endif
      #if DISPLAY_TYPE == 3 or DISPLAY_TYPE == 99
        gfx->setCursor(280, 30+(7 * y_offset_tft));
        gfx->print("OK");
      #endif
      delay(1000);
      mode = -1;
      i = 0;
    }
  }
}

void setupINA219(void) {                            //Only works when both menus have the same count. Feel free to change it :-)
  int menuitem;
  int lastcurrent             = current_servo;
  int lastangle_min          = angle_min;
  int lastshow_current        = show_current;
  bool value_change = false;
  int menuitem_used           = 2;
  String calibration_status   = START;
  String honeygate           = CLOSE;
  bool cal_done = false;
  int cal_angle = 0;
  int j = 0;
  int k = 0;
  #if DISPLAY_TYPE == 1 or DISPLAY_TYPE == 2 or DISPLAY_TYPE == 99
    int y_offset = 8;
  #endif
  #if DISPLAY_TYPE == 3 or DISPLAY_TYPE == 99
    int y_offset_tft = 28;
    bool change = false;
    int value_old = -1;
    int MenupointsCount = 3;
    const char *menupoints_1[MenupointsCount] = {SI_MI01, SI_MI02, SI_MI04};
    const char *menupoints_2[MenupointsCount] = {SI_MI01, SI_MI03, SI_MI04};
    gfx->fillScreen(BACKGROUND);
    gfx->setTextColor(TEXT);
    gfx->setFont(Punk_Mono_Bold_240_150);
    int x_pos = CenterPosX(SI_TO01, 14, 320);
    gfx->setCursor(x_pos, 25);
    gfx->println(SI_TO01);
    gfx->drawLine(0, 30, 320, 30, TEXT);
  #endif
  initRotaries(SW_MENU, 0, 0, menuitem_used, 1);
  i = 1;
  while (i > 0) {
    if (digitalRead(button_stop_pin) == HIGH  or digitalRead(switch_setup_pin) == LOW) {
      while (digitalRead(button_stop_pin) == HIGH) {
        delay(1);
      }
      current_servo = lastcurrent;
      angle_min = lastangle_min;
      show_current = lastshow_current;
      mode = -1;
      return;
    }
    if (value_change == false) {
      menuitem = getRotariesValue(SW_MENU);
      #if DISPLAY_TYPE == 3 or DISPLAY_TYPE == 99
        pos = menuitem;
      #endif
      if (menuitem == menuitem_used) {
        menuitem = 6;
      }
    }
    else {
      switch (menuitem) {
        case 0: current_servo         = getRotariesValue(SW_MENU);
                break;
        case 1: if (current_servo == 0) {
                  show_current = getRotariesValue(SW_MENU);
                }
                else {
                  j                     = 1;
                  calibration_status    = START;
                  value_change          = false;
                  menuitem_used         = 1;
                  setRotariesValue(SW_MENU, 0);
                  initRotaries(SW_MENU, 0, 0, menuitem_used, 1);
                }
                break;
      }
    }
    // Menu
    #if DISPLAY_TYPE == 1 or DISPLAY_TYPE == 2 or DISPLAY_TYPE == 99
      u8g2.setFont(u8g2_font_courB08_tf);
      u8g2.clearBuffer();
      u8g2.setCursor(10, 1 * y_offset);
      u8g2.print(SI_MI01);
      u8g2.setCursor(90, 1 * y_offset);
      if (current_servo > 0) {
        sprintf(output,"%4dmA", (current_servo));
      }
      else {
        sprintf(output,"%6s", (OFF));
      }
      u8g2.print(output);
      if (current_servo > 0) {
        u8g2.setCursor(10, 2 * y_offset);
        u8g2.print(SI_MI02);
        menuitem_used = 2;
      }
      else {
        u8g2.setCursor(10, 2 * y_offset);
        u8g2.print(SI_MI03);
        u8g2.setCursor(90, 2 * y_offset);
        sprintf(output,"%6s", (show_current==0?OFF:ON));
        u8g2.print(output);
        menuitem_used = 2;
      }
      u8g2.setCursor(10, (7 * y_offset) + 5);
      u8g2.print(SI_MI04);
      // Show position in menu. "*" when not selected, arrow when selected
      if (value_change == false && menuitem < menuitem_used) {
        u8g2.setCursor(1, 10+((menuitem)*y_offset)); u8g2.print("*");
      }
      else if (value_change == true && menuitem < menuitem_used) {
        u8g2.setCursor(1, 8+((menuitem)*y_offset)); u8g2.print("-");
      }
      else if (value_change == false && menuitem == 6) {
        u8g2.setCursor(1, 10+((menuitem)*y_offset) + 5); u8g2.print("*");
      }
      u8g2.sendBuffer();
    #endif
    #if DISPLAY_TYPE == 3 or DISPLAY_TYPE == 99
      int a = 0;
      while(a < MenupointsCount) {
        gfx->setTextColor(TEXT);
        if (a == pos and value_change == false) {
          gfx->setTextColor(MARKER);
        }
        if (a < MenupointsCount - 1) {
          if (current_servo == 0) {
            gfx->setCursor(5, 30+((a+1) * y_offset_tft));
            gfx->print(menupoints_2[a]);
          }
          else {
            gfx->setCursor(5, 30+((a+1) * y_offset_tft));
            gfx->print(menupoints_1[a]);
          }
          if (a == pos and value_change == true) {
            gfx->setTextColor(MARKER);
          }
          switch (a) {
            case 0: 
                    if (current_servo > 0) {
                      sprintf(output,"%dmA", current_servo);
                    }
                    else {
                      sprintf(output,"%s", OFF);
                    }
                    if (value_old != current_servo and value_change == true and a == pos) {
                      change = true;
                      value_old = current_servo;
                    }
                    break;
            case 1: if (current_servo == 0) {
                      sprintf(output,"%s", show_current==false?OFF:ON);
                      if (value_old != show_current and value_change == true and a == pos) {
                        change = true;
                        value_old = show_current;
                      }
                    }
                    else {
                      sprintf(output,"%s", "");
                      if (value_old != show_current and value_change == true and a == pos) {
                        change = true;
                        value_old = show_current;
                      }
                    }
                    break;
          }
          if (change) {
            if (a == 0) {
              gfx->fillRect(215, 27+((a+1) * y_offset_tft)-19, 100, 25, BACKGROUND);
              if (value_old == 0 or value_old == 50) {                                                //not entirely clean with the threshold values 
                gfx->fillRect(0, 27+((a+2) * y_offset_tft)-19, 320, 27, BACKGROUND);
              }
            }
            else {
              gfx->fillRect(251, 27+((a+1) * y_offset_tft)-19, 64, 25, BACKGROUND);
            }
            change = false;
          }
          int y = get_length(output);
          gfx->setCursor(320 - y * 14 - 5, 30+((a+1) * y_offset_tft));
          gfx->print(output);
        }
        else {
          gfx->setCursor(5, 30+(7 * y_offset_tft));
          gfx->print(menupoints_2[a]);
        }
        a++;
      }
    #endif
    if (digitalRead(SELECT_SW) == SELECT_LEVEL && menuitem < menuitem_used  && value_change == false) {
      while(digitalRead(SELECT_SW) == SELECT_LEVEL) {
        delay(1);
      } 
      switch (menuitem) { 
        case 0: initRotaries(SW_MENU, current_servo, 0, 1500, 50);
                break;
        case 1: if (current_servo == 0) {initRotaries(SW_MENU, show_current, 0, 1, 1);}
                break;
      }
      value_change = true;
    }
    // Taking changes in menupoint into account 
    if (digitalRead(SELECT_SW) == SELECT_LEVEL && menuitem < menuitem_used  && value_change == true) {
      while(digitalRead(SELECT_SW) == SELECT_LEVEL) {
        delay(1);
      }
      initRotaries(SW_MENU, menuitem, 0, menuitem_used, 1);
      value_change = false;
    }
    // Leave menu
    if (digitalRead(SELECT_SW) == SELECT_LEVEL && menuitem == 6) {
      while(digitalRead(SELECT_SW) == SELECT_LEVEL) {
        delay(1);
      }
      #if DISPLAY_TYPE == 1 or DISPLAY_TYPE == 2 or DISPLAY_TYPE == 99
        u8g2.setCursor(111, ((menuitem + 1) * y_offset) + 5);
        u8g2.print("OK");
        u8g2.sendBuffer();
      #endif
      #if DISPLAY_TYPE == 3 or DISPLAY_TYPE == 99
        gfx->setCursor(283, 30+(7 * y_offset_tft));
        gfx->print("OK");
      #endif
      delay(1000);
      mode = -1;
      i = 0;
    }
    while (j > 0) {
      if (digitalRead(switch_setup_pin) == LOW) {
        current_servo = lastcurrent;
        show_current = lastshow_current;
        mode = -1;
        return;
      }
      if (value_change == false) {
        menuitem = getRotariesValue(SW_MENU);
        if (menuitem == menuitem_used) {
          menuitem = 6;
        }
      }
      menuitem_used = 1;
      #if DISPLAY_TYPE == 1 or DISPLAY_TYPE == 2 or DISPLAY_TYPE == 99
        u8g2.setFont(u8g2_font_courB08_tf);
        u8g2.clearBuffer();
        u8g2.setCursor(10, 1 * y_offset);
        u8g2.print(calibration_status);
        u8g2.setCursor(10, 3 * y_offset);
        u8g2.print(SI_LI01);
        u8g2.setCursor(90, 3 * y_offset);
        sprintf(output,"%4imA", current_servo - 30);
        u8g2.print(output);
        u8g2.setCursor(10, 4 * y_offset);
        u8g2.print(SI_LI02);
        u8g2.setCursor(90, 4 * y_offset);
        sprintf(output,"%5i°", angle_min);
        u8g2.print(output);
        u8g2.setCursor(10, (7 * y_offset) + 5);
        u8g2.print(RETURN);
        // Show position in menu. "*" when not selected, arrow when selected
        if (value_change == false && menuitem < menuitem_used && calibration_status == START) {
          u8g2.setCursor(1, 10+((menuitem)*y_offset)); u8g2.print("*");
        }
        else if (value_change == false && menuitem == 6) {
          u8g2.setCursor(1, 10+((menuitem)*y_offset) + 5); u8g2.print("*");
        }
        u8g2.sendBuffer();
      #endif
      #if DISPLAY_TYPE == 3 or DISPLAY_TYPE == 99
        if (j == 1) {
          gfx->fillScreen(BACKGROUND);
          gfx->setTextColor(TEXT);
          gfx->setFont(Punk_Mono_Bold_240_150);
          x_pos = CenterPosX(SI_TO01, 14, 320);
          gfx->setCursor(x_pos, 25);
          gfx->println(SI_TO01);
          gfx->drawLine(0, 30, 320, 30, TEXT);
          gfx->setCursor(5, 30+(3 * y_offset_tft));
          gfx->print(SI_LI01);
          sprintf(output,"%4imA", current_servo - 30);
          int y = get_length(output);
          gfx->setCursor(320 - y * 14 - 5, 30+(3 * y_offset_tft));
          gfx->print(output);
          gfx->setCursor(5, 30+(4 * y_offset_tft));
          gfx->print(SI_LI02);
          sprintf(output,"%3i°", angle_min);
          y = get_length(output);
          gfx->setCursor(320 - y * 14 - 5, 30+(4 * y_offset_tft));
          gfx->print(output);
          j++;
        }
        if (value_change == false && menuitem < menuitem_used && calibration_status == START) {
          gfx->setTextColor(MARKER);
          gfx->setCursor(5, 30+(1 * y_offset_tft));
          gfx->print(calibration_status);
          gfx->setTextColor(TEXT);
          gfx->setCursor(5, 30+(7 * y_offset_tft));
          gfx->print(RETURN);
        }
        else {
          gfx->setTextColor(TEXT);
          gfx->setCursor(5, 30+(1 * y_offset_tft));
          gfx->print(calibration_status);
          gfx->setTextColor(MARKER);
          gfx->setCursor(5, 30+(7 * y_offset_tft));
          gfx->print(RETURN);
        }
      #endif
      if (value_change == true && menuitem < menuitem_used) {
        lastcurrent = current_servo;
        calibration_status = SI_LI03;
        honeygate = CLOSE;
        cal_done = false;
        cal_angle = 0;
        k = 1;
      }
      if (digitalRead(SELECT_SW) == SELECT_LEVEL && menuitem < menuitem_used  && value_change == false) {
        while(digitalRead(SELECT_SW) == SELECT_LEVEL) {
          delay(1);
        } 
        value_change = true;
      }
      // Taking changes in menupoint into account
      if (digitalRead(SELECT_SW) == SELECT_LEVEL && menuitem < menuitem_used  && value_change == true) {
        while(digitalRead(SELECT_SW) == SELECT_LEVEL) {
          delay(1);
        }
        initRotaries(SW_MENU, 0, 0, menuitem_used, 1);
        value_change = false;
      }
      //Leave
      if ((digitalRead(SELECT_SW) == SELECT_LEVEL && menuitem == 6) or digitalRead(button_stop_pin) == HIGH) {
        while(digitalRead(SELECT_SW) == SELECT_LEVEL or digitalRead(button_stop_pin) == HIGH) {
          delay(1);
        }
        #if DISPLAY_TYPE == 3 or DISPLAY_TYPE == 99
          gfx->fillScreen(BACKGROUND);
          x_pos = CenterPosX(SI_TO01, 14, 320);
          gfx->setCursor(x_pos, 25);
          gfx->println(SI_TO01);
          gfx->drawLine(0, 30, 320, 30, TEXT);
        #endif
        j = 0;
        value_change = false;
        menuitem_used = 2;
        initRotaries(SW_MENU, 0, 0, menuitem_used, 1);
      }
      while (k > 0) {
        SERVO_WRITE(90);
        honeygate = OPEN;
        int scaletime = millis();
        bool measurement_run = false;
        while (!cal_done) {
          #if DISPLAY_TYPE == 1 or DISPLAY_TYPE == 2 or DISPLAY_TYPE == 99
            u8g2.setFont(u8g2_font_courB08_tf);
            u8g2.clearBuffer();
            u8g2.setCursor(10, 1 * y_offset);
            u8g2.print(calibration_status);
            u8g2.setCursor(10, 3 * y_offset);
            u8g2.print(ABORT);
            u8g2.setCursor(10, 4 * y_offset);
            u8g2.print(SI_LI04);
            u8g2.setCursor(90, 4 * y_offset);
            sprintf(output,"%4imA", current_mA);
            u8g2.print(output);
            u8g2.setCursor(10, 4 * y_offset);
            u8g2.print(SI_LI05);
            u8g2.setCursor(90, 4 * y_offset);
            sprintf(output,"%5i°", cal_angle);
            u8g2.print(output);
            u8g2.setCursor(10, 5 * y_offset);
            u8g2.print(SI_LI06);
            sprintf(output,"%6s", honeygate);
            u8g2.print(output);
            u8g2.setCursor(1, (7 * y_offset) + 7); 
            u8g2.print("*");
            u8g2.setCursor(10, (7 * y_offset) + 5);
            u8g2.print(ABORT);
            u8g2.sendBuffer();
          #endif
          #if DISPLAY_TYPE == 3 or DISPLAY_TYPE == 99
            if (k == 1) {
              gfx->fillScreen(BACKGROUND);
              gfx->setTextColor(TEXT);
              gfx->setFont(Punk_Mono_Bold_240_150);
              x_pos = CenterPosX(SI_TO01, 14, 320);
              gfx->setCursor(x_pos, 25);
              gfx->println(SI_TO01);
              gfx->drawLine(0, 30, 320, 30, TEXT);
              gfx->setCursor(5, 30+(3 * y_offset_tft));
              change = true;
              k++;
            }
            if (change) {
              gfx->setTextColor(TEXT);
              gfx->setCursor(5, 30+(1 * y_offset_tft));
              gfx->print(calibration_status);
              gfx->fillRect(205, 27+(3 * y_offset_tft)-19, 110, 3 * y_offset_tft, BACKGROUND);
              gfx->setCursor(5, 30+(3 * y_offset_tft));
              gfx->print(SI_LI04);
              sprintf(output,"%4imA", current_mA);
              int y = get_length(output);
              gfx->setCursor(320 - y * 14 - 5, 30+(3 * y_offset_tft));
              gfx->print(output);
              gfx->setCursor(5, 30+(4 * y_offset_tft));
              gfx->print(SI_LI05);
              sprintf(output,"%3i°", cal_angle);
              y = get_length(output);
              gfx->setCursor(320 - y * 14 - 5, 30+(4 * y_offset_tft));
              gfx->print(output);
              gfx->setCursor(5, 30+(5 * y_offset_tft));
              gfx->print(SI_LI06);
              sprintf(output,"%5s", honeygate);
              y = get_length(output);
              gfx->setCursor(320 - y * 14 - 5, 30+(5 * y_offset_tft));
              gfx->print(output);
              gfx->setTextColor(MARKER);
              gfx->setCursor(5, 30+(7 * y_offset_tft));
              gfx->print(ABORT);
              change = false;
            }
          #endif
          if (millis() - scaletime >= 800 and !measurement_run) {
            SERVO_WRITE(cal_angle);
            honeygate = CLOSE;
            measurement_run = true;
            scaletime = millis();
            #if DISPLAY_TYPE == 3 or DISPLAY_TYPE == 99
              change = true;
            #endif
          }
          else if (millis() - scaletime >= 800 and measurement_run) {
            current_mA = GetCurrent(50);
            if (current_mA > current_servo - 30) {   //calibrate 30mA under the max. value
              SERVO_WRITE(90);
              honeygate = OPEN;
              cal_angle++;
              #if DISPLAY_TYPE == 3 or DISPLAY_TYPE == 99
                change = true;
              #endif
            }
            else {
              cal_done = true;
              calibration_status = SI_LI07;
              angle_min = cal_angle;
              k++;
            }
            measurement_run = false;
            scaletime = millis();
          }
          //Leave
          if ((digitalRead(SELECT_SW) == SELECT_LEVEL) or digitalRead(button_stop_pin) == HIGH) {
            while(digitalRead(SELECT_SW) == SELECT_LEVEL or digitalRead(button_stop_pin) == HIGH) {
              delay(1);
            }
            #if DISPLAY_TYPE == 3 or DISPLAY_TYPE == 99
              gfx->fillScreen(BACKGROUND);
              gfx->setTextColor(TEXT);
              x_pos = CenterPosX(SI_TO01, 14, 320);
              gfx->setCursor(x_pos, 25);
              gfx->println(SI_TO01);
              gfx->drawLine(0, 30, 320, 30, TEXT);
            #endif
            j = 0;
            k = 0;
            angle_min = lastangle_min;
            cal_done = true;
            value_change = false;
            menuitem_used = 2;
            initRotaries(SW_MENU, 0, 0, menuitem_used, 1);
          }
        }
        while (cal_done and k > 1) {
          current_mA = GetCurrent(50);
          #if DISPLAY_TYPE == 3 or DISPLAY_TYPE == 99
            if (value_old != current_mA) {
              change = true;
              value_old = current_mA;
            }
          #endif
          #if DISPLAY_TYPE == 1 or DISPLAY_TYPE == 2 or DISPLAY_TYPE == 99
            u8g2.setFont(u8g2_font_courB08_tf);
            u8g2.clearBuffer();
            u8g2.setCursor(10, 1 * y_offset);
            u8g2.print(calibration_status);
            u8g2.setCursor(10, 3 * y_offset);
            u8g2.print(SI_LI04);
            u8g2.setCursor(90, 3 * y_offset);
            sprintf(output,"%4imA", current_mA);
            u8g2.print(output);
            u8g2.setCursor(10, 4 * y_offset);
            u8g2.print(SI_LI05);
            u8g2.setCursor(90, 4 * y_offset);
            sprintf(output,"%5i°", angle_min);
            u8g2.print(output);
            u8g2.setCursor(1, (7 * y_offset) + 7); 
            u8g2.print("*");
            u8g2.setCursor(10, (7 * y_offset) + 5);
            u8g2.print(SI_MI04);
            u8g2.sendBuffer();
          #endif
          #if DISPLAY_TYPE == 3 or DISPLAY_TYPE == 99
            if (k==2) {
              gfx->setTextColor(TEXT);
              gfx->fillRect(0, 27+(1 * y_offset_tft)-19, 320, 25, BACKGROUND);
              gfx->fillRect(0, 27+(5 * y_offset_tft)-19, 320, 25, BACKGROUND);
              gfx->fillRect(0, 27+(7 * y_offset_tft)-19, 320, 25, BACKGROUND);
              gfx->setCursor(5, 30+(1 * y_offset_tft));
              gfx->print(calibration_status);
              gfx->setTextColor(MARKER);
              gfx->setCursor(5, 30+(7 * y_offset_tft));
              gfx->print(SI_MI04);
              gfx->setTextColor(TEXT);
              k++;
            }
            if (change) {
              gfx->fillRect(200, 27+(3 * y_offset_tft)-19, 120, 25, BACKGROUND);
              gfx->setCursor(5, 30+(3 * y_offset_tft));
              gfx->print(SI_LI04);
              sprintf(output,"%4imA", current_mA);
              int y = get_length(output);
              gfx->setCursor(320 - y * 14 - 5, 30+(3 * y_offset_tft));
              gfx->print(output);
              change = false;
            }
          #endif
          //Leave
          if (digitalRead(button_stop_pin) == HIGH) {
            while(digitalRead(button_stop_pin) == HIGH) {
              delay(1);
            }
            j = 0;
            k = 0;
            value_change = false;
            menuitem_used = 2;
            angle_min = lastangle_min;
            initRotaries(SW_MENU, 0, 0, menuitem_used, 1);
            #if DISPLAY_TYPE == 3 or DISPLAY_TYPE == 99
              gfx->fillScreen(BACKGROUND);
              x_pos = CenterPosX(SI_TO01, 14, 320);
              gfx->setCursor(x_pos, 25);
              gfx->println(SI_TO01);
              gfx->drawLine(0, 30, 320, 30, TEXT);
            #endif
          }
          if (digitalRead(SELECT_SW) == SELECT_LEVEL) {
            while(digitalRead(SELECT_SW) == SELECT_LEVEL) {
              delay(1);
            }
            j = 0;
            k = 0;
            mode = -1;
            value_change = false;
            menuitem_used = 2;
            initRotaries(SW_MENU, 0, 0, menuitem_used, 1);
            setPreferences();
            return;
          }
        }
      }
    }
  }
}

void processSetup(void) {
  int x_pos;
  int MenupointsCount = 9;
  int menuitem_old = -1;
  if (ina219_installed) {MenupointsCount++;}
  int posmenu[MenupointsCount];
  const char *menupoints[MenupointsCount] = {PS_MI01,PS_MI02,PS_MI03,PS_MI04,PS_MI05,PS_MI06,PS_MI07,PS_MI08,PS_MI09};
  //MenupointsCount = sizeof(menupoints)/sizeof(menupoints[0]);
  if (ina219_installed) {
      menupoints[MenupointsCount - 1] = menupoints[MenupointsCount -2];    //Clear Pref pushed once to the back
      menupoints[MenupointsCount - 2] = PS_MI10;         
  }
  mode = MODE_SETUP;
  angle = angle_min;          // Close honeygate
  servo_active = 0;              // Servo deactivated
  SERVO_WRITE(angle);
  rotary_select = SW_MENU;
  initRotaries(SW_MENU, lastpos, -1, MenupointsCount, -1);
  while (mode == MODE_SETUP and (digitalRead(switch_setup_pin)) == HIGH) {
    if (rotaries[SW_MENU].Value < 0) {
      rotaries[SW_MENU].Value = (MenupointsCount * ROTARY_SCALE) - 1;
    }
    else if (rotaries[SW_MENU].Value > (MenupointsCount * ROTARY_SCALE) - 1) {
      rotaries[SW_MENU].Value = 0;
    }
    int menuitem = getRotariesValue(SW_MENU) % MenupointsCount;
    for (i = 0; i < MenupointsCount; i++) {
      posmenu[i] = (menuitem + i) % MenupointsCount;
    }
    #if DISPLAY_TYPE == 1 or DISPLAY_TYPE == 2 or DISPLAY_TYPE == 99
      u8g2.clearBuffer();
      u8g2.setFont(u8g2_font_courB08_tf);
      x_pos = CenterPosX(menupoints[posmenu[MenupointsCount-1]], 6, 128); //one character is about 6 pixels wide
      u8g2.setCursor(x_pos,12);  
      u8g2.print(menupoints[posmenu[MenupointsCount-1]]);
      u8g2.drawLine(0, 18, 128, 18);
      u8g2.setFont(u8g2_font_courB12_tf);
      x_pos = CenterPosX(menupoints[posmenu[0]], 10, 128); //one character is about 10 pixels wide
      u8g2.setCursor(x_pos, 38);   
      u8g2.print(menupoints[posmenu[0]]);
      u8g2.drawLine(0, 47, 128, 47);
      u8g2.setFont(u8g2_font_courB08_tf);
      x_pos = CenterPosX(menupoints[posmenu[1]], 6, 128); //one character is about 6 pixels wide
      u8g2.setCursor(x_pos,62);   
      u8g2.print(menupoints[posmenu[1]]);
      u8g2.sendBuffer();
    #endif
    #if DISPLAY_TYPE == 3 or DISPLAY_TYPE == 99
      if (menuitem != menuitem_old) {
        gfx->fillScreen(BACKGROUND);
        gfx->drawLine(0, 100, 320, 100, TEXT);
        gfx->drawLine(0, 139, 320, 139, TEXT);
        gfx->setTextColor(TEXT);
        gfx->setFont(Punk_Mono_Bold_320_200);
        x_pos = CenterPosX(menupoints[posmenu[0]], 18, 320);
        gfx->setCursor(x_pos, 130);
        gfx->println(menupoints[posmenu[0]]);
        gfx->setTextColor(MENU_POS1);
        gfx->setFont(Punk_Mono_Bold_240_150);
        x_pos = CenterPosX(menupoints[posmenu[MenupointsCount-1]], 14, 320);
        gfx->setCursor(x_pos, 86);
        gfx->println(menupoints[posmenu[MenupointsCount-1]]);
        x_pos = CenterPosX(menupoints[posmenu[1]], 14, 320);
        gfx->setCursor(x_pos, 169);
        gfx->println(menupoints[posmenu[1]]);
        gfx->setTextColor(MENU_POS2);
        gfx->setFont(Punk_Mono_Bold_200_125);                   //9 x 11 pixel
        x_pos = CenterPosX(menupoints[posmenu[MenupointsCount-2]], 12, 320);
        gfx->setCursor(x_pos, 60);
        gfx->println(menupoints[posmenu[MenupointsCount-2]]);
        x_pos = CenterPosX(menupoints[posmenu[2]], 12, 320);
        gfx->setCursor(x_pos, 192);
        gfx->println(menupoints[posmenu[2]]);
        gfx->setTextColor(MENU_POS3);
        gfx->setFont(Punk_Mono_Bold_160_100);                   //7 x 9 pixel
        x_pos = CenterPosX(menupoints[posmenu[MenupointsCount-3]], 9, 320);
        gfx->setCursor(x_pos, 37);
        gfx->println(menupoints[posmenu[MenupointsCount-3]]);    
        x_pos = CenterPosX(menupoints[posmenu[3]], 9, 320);
        gfx->setCursor(x_pos, 213);
        gfx->println(menupoints[posmenu[3]]);
        gfx->setTextColor(MENU_POS4);
        gfx->setFont(Punk_Mono_Bold_120_075);                   //6 x 7 pixel
        x_pos = CenterPosX(menupoints[posmenu[MenupointsCount-4]], 7, 320);
        gfx->setCursor(x_pos, 16);
        gfx->println(menupoints[posmenu[MenupointsCount-4]]);
        x_pos = CenterPosX(menupoints[posmenu[4]], 7, 320);
        gfx->setCursor(x_pos, 232);
        gfx->println(menupoints[posmenu[4]]);
        menuitem_old = menuitem;
      }
    #endif
    lastpos = menuitem;
    if (digitalRead(SELECT_SW) == SELECT_LEVEL) {
      while(digitalRead(SELECT_SW) == SELECT_LEVEL) {
        delay(1);
      }
    #ifdef isDebug
      Serial.print("Setup Position: ");
      Serial.println(menuitem);
    #endif
      lastpos = menuitem;
      if (menupoints[menuitem] == PS_MI01)     setupTare();              // Tare values
      if (menupoints[menuitem] == PS_MI02)     setupCalibration();       // Calibration
      if (menupoints[menuitem] == PS_MI03)     setupFuellmenge();        // Fill quantity
      if (menupoints[menuitem] == PS_MI04)     setupAutomatik();         // Autostart/Autocorrection configuration 
      if (menupoints[menuitem] == PS_MI05)     setupServoAngle();       // Servosettings Minimum, Maximum und finedosage
      if (menupoints[menuitem] == PS_MI06)     setupParameter();         // Other parameters
      if (menupoints[menuitem] == PS_MI07)     setupCounter();           // Counter
      if (menupoints[menuitem] == PS_MI08)     setupTripCounter();       // Counter Trip
      if (menupoints[menuitem] == PS_MI10)     setupINA219();            // INA219 Setup
      setPreferences();
      if (menupoints[menuitem] == PS_MI09)     setupClearPrefs();        // Erase EEPROM
      initRotaries(SW_MENU,lastpos, 0,255, 1);                            // Menu parameters can be adjusted
    }
    #if OTA == 1
      if (digitalRead(button_start_pin) == HIGH) {
        while (digitalRead(button_start_pin) == HIGH) {delay(100);}
        menuitem_old = -1;
        OTASetup();
      }
    #endif
  }
}
void processAutomatic(void) {
  int goalweight;                 // Jar + correction
  static int autocorrection_gr = 0; 
  int force_servo_active = 0;
  boolean full = false;
  static float weight_old2;       // Weight of the previous run A.P.
  static int weight_previous;       // Weight of the previously filled jar
  static int collector_num = 5;      // Number of identical measurements for dripping
  int n;
  int y_offset = 0;
  if (mode != MODE_AUTOMATIC) {
     mode = MODE_AUTOMATIC;
     angle = angle_min;          // Closing honeygate
     servo_active = 0;              // deactivate servo
     SERVO_WRITE(angle);
     auto_active = 0;               // start automatic filling
     tare_jar = 0;
     rotary_select = SW_ANGLE;    // Angle setting over rotatry
     offset_angle = 0;            // Angle offset set to 0
     initRotaries(SW_MENU, fquantity_index, 0, 4, 1);
     weight_previous = jars[fquantity_index].Weight + correction;
     #if DISPLAY_TYPE == 3 or DISPLAY_TYPE == 99
      int x_pos;
      if (current_servo > 0) {
        no_ina = true;
      }
      else {
        no_ina = false;
      }
      jar_old = -1;
      current_mA_old = -1;
      corr_old = -99999;
      autocorr_gr_old = -99999;
      angle_min_old = -1;
      pos_old = -1;
      angle_is_old = -1;
      servo_active_old = -1;
      auto_active_old = -1;
      weight_alt = -9999999;
      intweight_old = -1;
      gfx->fillScreen(BACKGROUND);
      gfx->setTextColor(TEXT);
      gfx->drawLine(0, 167, 320, 167, TEXT);
      gfx->drawLine(160, 167, 160, 240, TEXT);
      gfx->setCursor(2, 185);
      gfx->setFont(Punk_Mono_Thin_160_100);
      gfx->print(AM_LI01);                          //INA219
      gfx->setFont(Checkbox);
      gfx->setCursor(140, 182);
      if (ina219_installed == 0){
        gfx->setTextColor(RED);
        gfx->print("B");
      }
      else {
        gfx->setTextColor(GREEN);
        gfx->print("A");
      }
      gfx->setTextColor(TEXT);
      gfx->setCursor(2, 202);
      gfx->setFont(Punk_Mono_Thin_160_100);
      gfx->print(AM_LI02);                           //Autostart
      gfx->setFont(Checkbox);
      gfx->setCursor(140, 199);
      if (autostart == 0){
        gfx->setTextColor(RED);
        gfx->print("B");
      }
      else {
        gfx->setTextColor(GREEN);
        gfx->print("A");
      }
      gfx->setTextColor(TEXT);
      gfx->setFont(Punk_Mono_Thin_160_100);
      gfx->setCursor(2, 219);
      gfx->print(AM_LI03);                          //Autocorrection
      gfx->setFont(Checkbox);
      gfx->setCursor(140, 216);
      if (autocorrection == 0){
        gfx->setTextColor(RED);
        gfx->print("B");
      }
      else {
        gfx->setTextColor(GREEN);
        gfx->print("A");
      }
      gfx->setTextColor(TEXT);
      gfx->setFont(Punk_Mono_Thin_160_100);
      gfx->setCursor(2, 236);
      gfx->print(AM_LI14);                          //Flow g/time
      gfx->setFont(Checkbox);
      gfx->setCursor(140, 233);
      if (intweight <= 0){
        gfx->setTextColor(RED);
        gfx->print("B");
      }
      else {
        gfx->setTextColor(GREEN);
        gfx->print("A");
      }
      gfx->setTextColor(TEXT);
      gfx->setFont(Punk_Mono_Thin_160_100);
      gfx->setCursor(170, 185);
      gfx->print(AM_LI04);                        //Current:
      gfx->setCursor(170, 202);
      gfx->print(AM_LI05);                        //Correction:
      gfx->setCursor(170, 219);
      gfx->print(AM_LI06);                        //Autocorr.:
      gfx->setCursor(170, 236);
      gfx->print(AM_LI15);                        //Flow:
      //Display Kopf
      gfx->setCursor(2, 17);
      gfx->print(AM_LI07);                        //Min:
      gfx->setCursor(119, 17);
      gfx->print(AM_LI08);                        //Max:
      gfx->setCursor(236, 17);
      gfx->print(AM_LI09);                        //Act:
      gfx->drawLine(0, 20, 320, 20, TEXT);
    #endif
  }
  pos = getRotariesValue(SW_ANGLE);
  // only regulate up to angular_fine, or solve via initRotaries?
  if (pos < angle_fine * 100 / angle_max) {                      
    pos = angle_fine * 100 / angle_max;
    setRotariesValue(SW_ANGLE, pos);
  }
  correction    = getRotariesValue(SW_CORRECTION);
  intweight   = getRotariesValue(SW_FLOW);
  fquantity_index = getRotariesValue(SW_MENU);
  fquantity       = jars[fquantity_index].Weight;
  tare         = jars[fquantity_index].Tare;
  if (tare <= 0) { 
    auto_active = 0;
  }
  // we only start if the tare for the fill quantity is set!
  // antoher push on the start button forces the servo activation
  if (digitalRead(button_start_pin) == HIGH && tare > 0) {
    while(digitalRead(button_start_pin) == HIGH) {
       delay(1);
    }
    if (auto_active == 1) {
      force_servo_active = 1;
      #ifdef isDebug
        Serial.println("force servo active");      
      #endif
    }
    auto_active    = 1;                            // autofill activation
    rotary_select = SW_ANGLE;                    // in case the start button was pushed during parameter changes    
    setPreferences();                             // in case parameters were changed using the rotary
    #if DISPLAY_TYPE == 3 or DISPLAY_TYPE == 99
      jar_old = -1;                              // jar type color reset if marked
      corr_old = -99999;                          // correction color reset if marked
    #endif
  }
  if (digitalRead(button_stop_pin) == HIGH) {
    angle      = angle_min + offset_angle;
    servo_active = 0;
    auto_active  = 0;
    tare_jar   = 0;
  }
  // Scale error correction in case the weight varies too much 
  #ifdef ERRORCORRECTION_SCALE
    int compareweight = (int(SCALE_GETUNITS(SCALE_READS))) - tare;
    for (byte j = 0 ; j < 3; j++) { // Number of repetitions if deviation is too high
      weight = int(SCALE_GETUNITS(SCALE_READS)) - tare;
      if (abs(weight - compareweight) < 50)  // Deviation for error detection
        break; 
      delay(100);
    }
  #else
    weight = int(SCALE_GETUNITS(SCALE_READS)) -   tare;
  #endif 
  // Jar removed -> close servo
  if (weight < -20) {
    angle      = angle_min + offset_angle;
    servo_active = 0;
    tare_jar   = 0;
    if (autostart != 1) {  // Autostart not active
      auto_active  = 0;
    }
  }
  // Automatic ON, empty jar set on, servo OFF -> fill jar
  if (auto_active == 1 && abs(weight) <= jartolerance && servo_active == 0) {
    rotary_select = SW_ANGLE;     // in case a jar is set during paramater changes
    #if DISPLAY_TYPE == 1 or DISPLAY_TYPE == 2 or DISPLAY_TYPE == 99 
      u8g2.clearBuffer();
      u8g2.setFont(u8g2_font_courB24_tf);
      u8g2.setCursor(15, 43);
      u8g2.print(AM_LI12);            //START
      u8g2.sendBuffer();
    #endif
    #if DISPLAY_TYPE == 3 or DISPLAY_TYPE == 99
      // START: not elegant but it works (maybe make it better with subprograms)
      //set PLAY ICON
      gfx->fillRect(12, 44, 38, 38, BACKGROUND);
      gfx->setFont(Icons_Start_Stop);
      gfx->setCursor(5, 88);
      gfx->print("M");
      gfx->setCursor(5, 88);
      gfx->setTextColor(GREEN);
      gfx->print("A");
      //set color to text for correction value
      gfx->setTextColor(TEXT);
      gfx->setFont(Punk_Mono_Thin_160_100);
      gfx->setCursor(265, 202);
      sprintf(output, "%5ig", correction + autocorrection_gr);
      gfx->print(output);
      //set color to text for flow value
      gfx->setTextColor(TEXT);
      gfx->setFont(Punk_Mono_Thin_160_100);
      gfx->setCursor(265, 236);
      sprintf(output, "%5ig", intweight);
      gfx->print(output);
      //set color to text for jar weight
      gfx->setTextColor(TEXT);
      gfx->fillRect(0, 136, 320, 27, BACKGROUND);
      gfx->setFont(Punk_Mono_Thin_240_150);
      gfx->setCursor(2, 156);
      sprintf(output, "%dg ", (jars[fquantity_index].Weight));
      gfx->print(output);
      gfx->setFont(Punk_Mono_Thin_120_075);
      gfx->setCursor(2 + 14*StringLenght(output), 148);
      sprintf(output, "+%dg ", (overfill_gr));
      gfx->print(output);
      // END:   not elegant but it works
      gfx->fillRect(80, 24, 240, 80, BACKGROUND);
      gfx->setFont(Punk_Mono_Bold_600_375);
      gfx->setCursor(100, 85);
      gfx->print(AM_LI12);                      //START
    #endif
    // A.P. waiting a short while and check over 5 measurements if the weight fluctuation was only occasional 
    int weight_middle = 0;
    for (int i = 0; i < 5; i++){
      weight = int(SCALE_GETUNITS(SCALE_READS)) - tare;
      weight_middle += weight;
      delay(300);
    }
    weight = weight_middle / 5;                 //A.P.  Average weight over 1,5s
    if (intweight <= 0) {
      angle = n ;                       //A.P. set the optimal angle for next jar according to previous jar.
    }
    if (abs(weight) <= jartolerance) {
      if (intweight <= 0) {
        weight = int(SCALE_GETUNITS(SCALE_READS)) - tare;  // A.P.
      }
      tare_jar = weight;
      #if DISPLAY_TYPE == 3 or DISPLAY_TYPE == 99
        gfx->fillRect(109, 136, 211, 27, BACKGROUND);
        gfx->setFont(Punk_Mono_Thin_240_150);
        sprintf(output, "%s:%dg ", AM_LI13, tare_jar);               //Jar tare
        int i = StringLenght(output);
        gfx->setCursor(320 - 14 * i, 156);
        gfx->print(output);
      #endif
      #ifdef isDebug 
        Serial.print(" weight: ");           Serial.print(weight);
        Serial.print(" weight_previous: ");    Serial.print(weight_previous);
        Serial.print(" Goalweight: ");       Serial.print(fquantity + correction + tare_jar + autocorrection_gr);
        Serial.print(" Overfill: ");            Serial.print(overfill_gr);
        Serial.print(" Autocorrection: ");     Serial.println(autocorrection_gr);
      #endif      
      servo_active = 1;
      collector_num = 0;
      full = false;
      counted = false;
      weight_alt = -9999999;               //to activate the displaying of weight
      buzzer(BUZZER_SHORT);
    }
  }
  goalweight = fquantity + correction + tare_jar + autocorrection_gr;
  // Adaptation of the autocorrection value
  if (autocorrection == 1) {                                                       
    if ( auto_active == 1 && servo_active == 0 && angle == angle_min + offset_angle && weight >= goalweight && collector_num <= 5) {     
      full = true;                       
      if (weight == weight_previous && collector_num < 5) { // we want to see 5times the same weight  
        collector_num++;
      } 
      else if (weight != weight_previous) {               // otherwise track the weight changes
        weight_previous = weight;
        collector_num = 0;
      } 
      else if (collector_num == 5) {                        // weight is 5 times identical, confirm autocorrection
        autocorrection_gr = (fquantity + overfill_gr + tare_jar) - (weight - autocorrection_gr);
        if (correction + autocorrection_gr > overfill_gr) {   // Autocorrection not allowed to overcorrect, max fill quantity plus Overfill
          autocorrection_gr = overfill_gr - correction;
          #ifdef isDebug
            Serial.print("Autocorrection limited to ");
            Serial.println(autocorrection_gr);
          #endif
        }
        buzzer(BUZZER_SUCCESS);
        collector_num++;                                      // correction value for this run has been reached
      }
      if (full == true && counted == false) {
        jars[fquantity_index].TripCount++;
        jars[fquantity_index].Count++;
        counted = true;
      }
      #ifdef isDebug
        Serial.print("Dripping:");
        Serial.print(" weight: ");        Serial.print(weight);
        Serial.print(" weight_previous: "); Serial.print(weight_previous);
        Serial.print(" collector_num: ");    Serial.print(collector_num);
        Serial.print(" Correction: ");      Serial.println(autocorrection_gr);
        Serial.print(" Counter Trip: ");    Serial.print(jars[fquantity_index].TripCount); //Kud
        Serial.print(" Counter: ");         Serial.println(jars[fquantity_index].Count); //Kud
      #endif
    }
  }
  // Jar is partly filled up. Start will be forced through start button
  if (auto_active == 1 && weight > 5 && force_servo_active == 1) {
    servo_active = 1;
    full = false;
    counted = false;
    buzzer(BUZZER_SHORT);
  }
  if (servo_active == 1 && intweight == 0) { 
    angle = (angle_max * pos / 100);  // A.P. is deactivated when intweight is near 0 (automatic through flow speed is 0)
  }
  if (intweight > 0) {
    if (servo_active == 1 &&(goalweight - weight <= fine_dosing_weight) && (angle_max*pos / 100 * goalweight-weight / fine_dosing_weight <= angle)) {
      angle = ((angle_max*pos / 100) * ((goalweight-weight) / fine_dosing_weight));    // AP honeygate will only be closed if the angle calculated here is smaller than the current angle (otherwise it will open with smaller opening angles)
    }
  }
  else {
    if (servo_active == 1 && (goalweight - weight <= fine_dosing_weight)) {
      angle = ((angle_max*pos / 100) * ((goalweight-weight) / fine_dosing_weight));
    }
  }
  if (servo_active == 1 && angle <= angle_fine) { 
    angle = angle_fine;
  }
  if (intweight > 0) {
    // A.P.  A moving average is calculated over 3 measurements.
    int Rawweight = weight-weight_old2;
    RunningAverageBuffer[NextRunningAverage++] = Rawweight;
    if (NextRunningAverage >= RunningAverageCount)
    {
      NextRunningAverage = 0; 
    }
    float RunningAverageweight = 0;
    for(int i=0; i< RunningAverageCount; ++i) {
      RunningAverageweight += RunningAverageBuffer[i];
    }
    RunningAverageweight /= RunningAverageCount;
    //Optimization of the throughflow speed  A.P.
    if (servo_active == 1 && ((RunningAverageweight + intweightD) > intweight) && (intweight >= 0) && ((goalweight - weight) >= fine_dosing_weight)) {
      angle = angle -2;
      n  = angle; // A.P. Records the last optimum opening angle of servo.
    }    //A.P.  Automatic speed for filling process
    if (servo_active == 1 && ((RunningAverageweight - intweightD) < intweight) && (intweight >= 0) && ((goalweight - weight) >= fine_dosing_weight)) {
      angle = angle +2;
      if (angle >= (angle_max*pos/100)) {
        angle = (angle_max*pos/100);
      }    
      n  = angle; // A.P. Records the last optimum opening angle of servo.
    }  //A.P.  Automatic speed for filling process
    weight_old2 = weight;  //A.P.
  }
  // Jar is full
  if (servo_active == 1 && weight >goalweight) {
    angle      = angle_min + offset_angle;
    servo_active = 0;
    if (counted == false) {
      jars[fquantity_index].TripCount++;
      jars[fquantity_index].Count++;
      counted = true;
    }
    if (autostart != 1)       // autostart is not active, no further start
      auto_active = 0;
    if (autocorrection == 1)   // autocorrection, weight recording
      weight_previous = weight;
    buzzer(BUZZER_SHORT);
  }   
  // jar moved out -> Servo closing additional query
  if (angle > (angle_min + offset_angle)) {
    weight = int(SCALE_GETUNITS(SCALE_READS)) - tare; 
    if (weight < -20) { 
      delay(200);  // A.P.  To pevent an ESP reset when it gets closed shortyl after opening.
      angle      = angle_min + offset_angle;
    }
  }
  SERVO_WRITE(angle);
  #ifdef isDebug
    #if isDebug >= 4
      Serial.print("Automatic:");  
      Serial.print(" Weight: ");        Serial.print(weight);
      Serial.print(" Angle: ");         Serial.print(angle);
      Serial.print(" Duration ");           Serial.print(millis() - scaletime);
      Serial.print(" Fillquantity: ");      Serial.print(fquantity);
      Serial.print(" Correction: ");      Serial.print(correction);
      Serial.print(" Tare_jar:");       Serial.print(tare_jar);
      Serial.print(" Autocorrection: ");  Serial.print(autocorrection_gr);
      Serial.print(" Goalweight ");     Serial.pringoa);
      Serial.print(" Force Servo: "); Serial.print(force_servo_active);
      Serial.print(" servo_active ");     Serial.print(servo_active);
      Serial.print(" auto_active ");      Serial.println(auto_active);
    #endif 
  #endif
  if (ina219_installed and (current_servo > 0 or show_current == 1)) {
    y_offset = 4;
  }
  #if DISPLAY_TYPE == 1 or DISPLAY_TYPE == 2 or DISPLAY_TYPE == 99
    u8g2.clearBuffer();
    // Displays a warinig instead of a weight if no tare is defined for the jar
    if (tare > 0) {
      // no jar set up
      if (weight < -20) {
        u8g2.setFont(u8g2_font_courB10_tf);
        u8g2.setCursor(37, 31 + y_offset); u8g2.print(AM_LI10);       //Please put
        u8g2.setCursor(37, 43 + y_offset); u8g2.print(AM_LI11);       //up the jar
      } 
      else {
        if(autostart == 1 && auto_active == 1 && servo_active == 0 && weight >= -5 && weight - tare_jar < fquantity) {
          u8g2.setFont(u8g2_font_unifont_t_symbols);
          u8g2.drawGlyph(14, 38 + y_offset, 0x2612);
          u8g2.setFont(u8g2_font_courB24_tf);
        }
        u8g2.setCursor(10, 42 + y_offset);
        u8g2.setFont(u8g2_font_courB24_tf);
        sprintf(output,"%5dg", weight - tare_jar);
        u8g2.print(output);
      }
    } 
    else {
      u8g2.setFont(u8g2_font_courB14_tf);
      int y = get_length(AM_ER01);
      u8g2.setCursor(128 - y * 10 -5, 38 + y_offset);
      sprintf(output,"%6s", AM_ER01);                    //no tare
      u8g2.print(output);
    }
    // Play/Pause Icon, if automatic mode is active
    u8g2.setFont(u8g2_font_open_iconic_play_2x_t);
    u8g2.drawGlyph(0, 40 + y_offset, (auto_active==1)?0x45:0x44 );
    u8g2.setFont(u8g2_font_courB08_tf);
    // Top line, absolute opening angle and percent, autostart display
    u8g2.setCursor(1, 9);
    sprintf(output,"%s=%-3d", AM_LI16, angle);
    u8g2.print(output);
    n = StringLenght(output);
    u8g2.setCursor(n * 6 + 1, 9);
    u8g2.print("°");
    sprintf(output,"%3d%%", pos);
    u8g2.setCursor(104, 9);
    u8g2.print(output);
    sprintf(output,"%2s", (autostart==1)?AM_LI17:" ");
    u8g2.setCursor(58, 9);
    u8g2.print(output);
    //INA219 display
    if (ina219_installed and (current_servo > 0 or show_current == 1)) {
      u8g2.setCursor(0, 18);
      sprintf(output, AM_LI18);
      u8g2.print(output);
      sprintf(output, "%4imA", current_mA);
      u8g2.setCursor(128 - 36, 18);
      u8g2.print(output);
    }
    // Bottom line
    // Adjustment only if automatic is inactive, controlled via interrupt function 
    if (servo_active == 1) {
      progressbar = 128.0*((float)weight/(float)goalweight);
      progressbar = constrain(progressbar,0,128);
      u8g2.drawFrame(0, 55, 128, 9);
      u8g2.drawBox  (0, 55, progressbar, 9);
    } 
    else {
      u8g2.setFont(u8g2_font_courB08_tf);
      sprintf(output, "%s%s=%-3d" ,(autocorrection==1)?AM_LI19:"", AM_LI20 , correction + autocorrection_gr);
      n = StringLenght(output);
      if (rotary_select == SW_CORRECTION) {
        u8g2.setCursor(10, 62);
        u8g2.setFont(u8g2_font_open_iconic_arrow_1x_t);
        u8g2.drawGlyph(0, 63, 0x42);
        u8g2.drawGlyph(n * 6 + 1 + 10, 63, 0x41);
        u8g2.setFont(u8g2_font_courB08_tf);
      }
      else {
        u8g2.setCursor(0, 62);
      }
      u8g2.print(output);
      sprintf(output, "%s=%-3d" ,AM_LI21, intweight);
      n = StringLenght(output);
      if (rotary_select == SW_FLOW) {
        u8g2.setCursor(10, 54);
        u8g2.setFont(u8g2_font_open_iconic_arrow_1x_t);
        u8g2.drawGlyph(0, 55, 0x42);
        u8g2.drawGlyph(n * 6 + 1 + 10, 55, 0x41);
        u8g2.setFont(u8g2_font_courB08_tf);
      }
      else {
        u8g2.setCursor(0, 54);
      }
      u8g2.print(output);

      if (jars[fquantity_index].Weight > 999) {
        sprintf(output, "%2.1fkg-%3s", float(jars[fquantity_index].Weight) / 1000, JarTypeArray[jars[fquantity_index].JarType]);
      }
      else if (jars[fquantity_index].Weight >= 100) {
        sprintf(output, "%3dg-%3s", jars[fquantity_index].Weight, JarTypeArray[jars[fquantity_index].JarType]);
      }
      else {
        sprintf(output, "%2dg-%3s", jars[fquantity_index].Weight, JarTypeArray[jars[fquantity_index].JarType]);
      }
      n = StringLenght(output);
      if (rotary_select == SW_MENU) {
        u8g2.setCursor(128 - (n * 6 + 10), 62);
        u8g2.setFont(u8g2_font_open_iconic_arrow_1x_t);
        u8g2.drawGlyph(128 - 9, 63, 0x41);
        u8g2.drawGlyph(128 - (n * 6 + 20), 63, 0x42);
        u8g2.setFont(u8g2_font_courB08_tf);
      }
      else {
        u8g2.setCursor(128 - (n * 6), 62);
      }
      u8g2.print(output);
    }
    u8g2.sendBuffer();
  #endif
  #if DISPLAY_TYPE == 3 or DISPLAY_TYPE == 99
    //INA219 display                                                                                                    
    if (ina219_installed == 1 and current_mA != current_mA_old and (current_servo > 0 or show_current == 1)) {
      gfx->fillRect(260, 170, 70, 18, BACKGROUND);
      gfx->setTextColor(TEXT);
      gfx->setFont(Punk_Mono_Thin_160_100);
      gfx->setCursor(265, 185);
      if (current_mA > current_servo and current_servo > 0) {
        gfx->setTextColor(RED);
      }
      sprintf(output, "%4imA", current_mA);
      gfx->print(output);
      gfx->setTextColor(TEXT);
      current_mA_old = current_mA;
    }
    else if (ina219_installed == 1 and no_ina == 0 and show_current == 0) {
      no_ina = true;
      gfx->setTextColor(TEXT);
      gfx->setFont(Punk_Mono_Thin_160_100);
      gfx->setCursor(265, 185);
      sprintf(output, "%6s", OFF);
      gfx->print(output);
    }
    else if (ina219_installed == 0 and no_ina == 0) {
      no_ina = true;
      gfx->setTextColor(TEXT);
      gfx->setFont(Punk_Mono_Thin_160_100);
      gfx->setCursor(265, 185);
      sprintf(output, "%6s", "-");
      gfx->print(output);
    }
    //Correction display
    if (corr_old != correction + autocorrection_gr) {
      gfx->setFont(Punk_Mono_Thin_160_100);
      gfx->setTextColor(BACKGROUND);
      gfx->setCursor(265, 202);
      sprintf(output, "%5ig", corr_old);
      gfx->print(output);
      gfx->setTextColor(TEXT);
      if (rotary_select == SW_CORRECTION and servo_active == 0) {
        gfx->setTextColor(MARKER);
      }
      gfx->setCursor(265, 202);
      sprintf(output, "%5ig", correction + autocorrection_gr);
      gfx->print(output);
      gfx->setTextColor(TEXT);
      corr_old = correction + autocorrection_gr;
    }
    //Autocorrection display
    if (autocorr_gr_old != autocorrection_gr) {
      gfx->setFont(Punk_Mono_Thin_160_100);
      gfx->setTextColor(BACKGROUND);
      gfx->setCursor(265, 219);
      sprintf(output, "%5ig", autocorr_gr_old);
      gfx->print(output);
      gfx->setTextColor(TEXT);
      gfx->setCursor(265, 219);
      sprintf(output, "%5ig", autocorrection_gr);
      gfx->print(output);
      autocorr_gr_old = autocorrection_gr;
    }
    //Flow display
    if (intweight_old != intweight) {
      gfx->setFont(Punk_Mono_Thin_160_100);
      gfx->setTextColor(BACKGROUND);
      gfx->setCursor(265, 236);
      sprintf(output, "%5ig", intweight_old);
      gfx->print(output);
      gfx->setTextColor(TEXT);
      if (rotary_select == SW_FLOW and servo_active == 0) {
        gfx->setTextColor(MARKER);
      }
      gfx->setCursor(265, 236);
      sprintf(output, "%5ig", intweight);
      gfx->print(output);
      gfx->setTextColor(TEXT);
      if (intweight_old == 0 or intweight == 0) {
        gfx->setFont(Checkbox);
        if (intweight == 0){
          gfx->setCursor(140, 233);
          gfx->setTextColor(BACKGROUND);
          gfx->print("A");
          gfx->setCursor(140, 233);
          gfx->setTextColor(RED);
          gfx->print("B");
        }
        else {
          gfx->setCursor(140, 233);
          gfx->setTextColor(BACKGROUND);
          gfx->print("B");
          gfx->setCursor(140, 233);
          gfx->setTextColor(GREEN);
          gfx->print("A");
        }
      }
      gfx->setTextColor(TEXT);
      intweight_old = intweight;
    }
    //Jar choice
    if ((jar_old != fquantity_index and servo_active == 0 and weight <= jars[fquantity_index].Weight - tare)) {
      if (rotary_select == SW_MENU and servo_active == 0) {
        gfx->setTextColor(MARKER);
      }
      gfx->fillRect(0, 136, 320, 27, BACKGROUND);
      gfx->setFont(Punk_Mono_Thin_240_150);
      gfx->setCursor(2, 156);
      sprintf(output, "%dg ", (jars[fquantity_index].Weight));
      gfx->print(output);
      gfx->setFont(Punk_Mono_Thin_120_075);
      gfx->setCursor(2 + 14*StringLenght(output), 148);
      sprintf(output, "+%dg ", (overfill_gr));
      gfx->print(output);
      gfx->setFont(Punk_Mono_Thin_240_150);
      gfx->setCursor(110, 156);
      if (JarTypeArray[jars[fquantity_index].JarType] == "DIB") {
        gfx->print("DE Imker Bund");
      }
      else if (JarTypeArray[jars[fquantity_index].JarType] == "TOF") {
        gfx->print("TwistOff");
      }
      else if (JarTypeArray[jars[fquantity_index].JarType] == "DEE") {
        gfx->print("DeepTwist");
      }
      else if (JarTypeArray[jars[fquantity_index].JarType] == "SPZ") {
        gfx->print("Spezial");
      }
      gfx->setTextColor(TEXT);
      jar_old = fquantity_index;
    }
    //min. angle
    if (angle_min  + offset_angle != angle_min_old) {
      gfx->setTextColor(TEXT);
      gfx->fillRect(37, 3, 40, 16, BACKGROUND);
      gfx->setFont(Punk_Mono_Thin_160_100);
      gfx->setCursor(38, 17);
      sprintf(output, "%3i°", angle_min + offset_angle);
      gfx->print(output);
      angle_min_old = angle_min + offset_angle;
    }
    //Act. position
    if (pos != pos_old) {
      gfx->fillRect(154, 3, 40, 16, BACKGROUND);
      gfx->setFont(Punk_Mono_Thin_160_100);
      gfx->setCursor(155, 17);
      sprintf(output, "%3i°", angle_max*pos/100);
      gfx->print(output);
      pos_old = pos;
    }
    //max. angle
    if (angle != angle_is_old) {
      gfx->fillRect(271, 3, 40, 16, BACKGROUND);
      gfx->setFont(Punk_Mono_Thin_160_100);
      gfx->setCursor(272, 17);
      sprintf(output, "%3i°", angle);
      gfx->print(output);
      angle_is_old = angle;
    }
    //Start/Stop Icon
    if (auto_active != auto_active_old) {
      gfx->fillRect(12, 44, 38, 38, BACKGROUND);
      gfx->setFont(Icons_Start_Stop);
      gfx->setCursor(5, 88);
      gfx->print("M");
      gfx->setCursor(5, 88);
      if (auto_active == 1) {
        gfx->setTextColor(GREEN);
        gfx->print("A");
        jar_old = -1;
        corr_old = -99999;
        intweight_old = -1;
      }
      else {
        gfx->setTextColor(RED);
        gfx->print("B");
      }
      auto_active_old = auto_active;
        gfx->setTextColor(TEXT);
    }
    //put up jar
    if (tare > 0) {
      if (weight < -20 and weight != weight_alt) {
        gfx->fillRect(80, 24, 240, 80, BACKGROUND);
        gfx->setFont(Punk_Mono_Bold_320_200);
        gfx->setCursor(120, 58);
        gfx->print(AM_LI10);                         //Please put
        gfx->setCursor(120, 90);
        gfx->print(AM_LI11);                         //up the jar
        jar_old = -1;
      } 
      else if (weight != weight_alt) {
        gfx->fillRect(80, 24, 240, 80, BACKGROUND);
        gfx->setFont(Punk_Mono_Bold_600_375);
        gfx->setCursor(100, 85);
        sprintf(output, "%5ig", weight - tare_jar);
        gfx->print(output);
      }
      if (weight != weight_alt) {
        progressbar = 318.0*((float)weight/(float)(jars[fquantity_index].Weight));
        progressbar = constrain(progressbar,0,318);
        gfx->drawRect(0, 107, 320, 15, TEXT);
        if (jars[fquantity_index].Weight > weight) {
          gfx->fillRect  (1, 108, progressbar, 13, RED);
        }
        else if (weight >= jars[fquantity_index].Weight and weight <= jars[fquantity_index].Weight + overfill_gr){
          gfx->fillRect  (1, 108, progressbar, 13, GREEN);
        }
        else {
          gfx->fillRect  (1, 108, progressbar, 13, ORANGE);
        }
        gfx->fillRect  (1 + progressbar, 108, 318 - progressbar, 13, BACKGROUND);
        weight_alt = weight;
      }
    }
    //no tare present
    else if (weight != weight_alt) {        //no tare present
      gfx->fillRect(80, 24, 240, 80, BACKGROUND);
      gfx->fillRect(0, 107, 320, 15, BACKGROUND);
      gfx->setTextColor(RED);
      gfx->setFont(Punk_Mono_Bold_320_200);
      gfx->setCursor(120, 74);
      gfx->print(AM_ER01);                 //no tare!
      gfx->setTextColor(TEXT);
      weight_alt = weight;
    }
  #endif
  if (alarm_overcurrent) {i = 1;}
  while (i > 0) {
    inawatchdog = 0;                    //turn off the continuous INA measurement
    //Servo is closed
    if (servo.read() <= angle_min  + offset_angle and offset_angle < 3) {
      while(offset_angle < 3 and current_servo < current_mA) {
        offset_angle = offset_angle + 1;
        SERVO_WRITE(angle_min + offset_angle);
        current_mA = GetCurrent(10);
        delay(1000);
      }
      alarm_overcurrent = 0;
    }
    i = 0;
    inawatchdog = 1;
  }
  setPreferences();
}

void processManualmode(void) {
  i = 0;
  int y_offset = 0;
  if (mode != MODE_MANUAL) {
    mode = MODE_MANUAL;
    angle = angle_min;          // closing honeygate
    servo_active = 0;              // Servo off
    SERVO_WRITE(angle);
    rotary_select = SW_ANGLE;
    tare = 0;
    offset_angle = 0;            // Angle offset set to 0
    #if DISPLAY_TYPE == 3 or DISPLAY_TYPE == 99
      int x_pos;
      weight_alt = -9999999;
      pos_old = -1;
      tare_old = -1;
      current_mA_old = -1;
      servo_active_old = -1;
      angle_is_old = -1;
      gfx->fillScreen(BACKGROUND);
      gfx->setTextColor(TEXT);
      gfx->setFont(Punk_Mono_Bold_320_200);
      sprintf(output,MM_TO01);                       //Manual
      x_pos = CenterPosX(output, 18, 320);
      gfx->setCursor(x_pos, 27);
      gfx->print(output);
      gfx->drawLine(0, 30, 320, 30, TEXT);
      gfx->setFont(Punk_Mono_Bold_200_125);
      gfx->setCursor(5, 192);
      gfx->print(MM_LI01);                           //Servo
      gfx->drawLine(0, 170, 320, 170, TEXT);
      gfx->drawLine(160, 170, 160, 240, TEXT);
      gfx->setCursor(170, 192);
      gfx->print(MM_LI02);                           //Tare
      if (ina219_installed == 1) {
        gfx->setCursor(170, 215);
        sprintf(output, MM_LI03);                   //INA
        gfx->print(output);
        sprintf(output, "%s", (current_servo==0?OFF:ON));
        int y = get_length(output);
        gfx->setCursor(320 - y * 12 - 5, 215);
        gfx->print(output);
        if (ina219_installed == 1 and current_mA != current_mA_old and (current_servo > 0 or show_current == 1)) {
          gfx->setCursor(170, 238);
          gfx->print(MM_LI04);                       //Current
        }
      }
    #endif
  }
  pos = getRotariesValue(SW_ANGLE);
  weight = SCALE_GETUNITS(SCALE_READS) - tare;
  if ((digitalRead(button_start_pin)) == HIGH) {
    servo_active = 1;
  }
  if ((digitalRead(button_stop_pin)) == HIGH) {
    servo_active = 0;
  }
  if ((digitalRead(outputSW)) == LOW) {
      tare = SCALE_GETUNITS(SCALE_READS);
  }
  if (servo_active == 1) {
    angle = ((angle_max * pos) / 100);
  }
  else { 
    angle = angle_min + offset_angle;
  }
  angle = constrain(angle, angle_min + offset_angle, angle_max);

  
  SERVO_WRITE(angle);
  if (ina219_installed and (current_servo > 0 or show_current == 1)) {
    y_offset = 4;
  }
  #ifdef isDebug
    #if isDebug >= 4
      Serial.print("Manual Mode:");  
      Serial.print(" Weight ");     Serial.print(weight);
      Serial.print(" Angle ");      Serial.print(angle);
      Serial.print(" servo_active "); Serial.println(servo_active);
    #endif
  #endif
  #if DISPLAY_TYPE == 1 or DISPLAY_TYPE == 2 or DISPLAY_TYPE == 99
    u8g2.clearBuffer();
    u8g2.setFont(u8g2_font_courB24_tf);
    u8g2.setCursor(10, 42 + y_offset);
    sprintf(output,"%5dg", weight);
    u8g2.print(output);
    u8g2.setFont(u8g2_font_open_iconic_play_2x_t);
    u8g2.drawGlyph(0, 40 + y_offset, (servo_active==1)?0x45:0x44 );
    u8g2.setFont(u8g2_font_courB08_tf);
    u8g2.setCursor(0, 11);
    u8g2.setCursor(1, 9);
    sprintf(output,"%s=%d°", MM_LI07, angle);
    u8g2.print(output);
    sprintf(output,"%3d%%", pos);
    u8g2.setCursor(104, 9);
    u8g2.print(output);
    if (ina219_installed and (current_servo > 0 or show_current == 1)) {
      u8g2.setCursor(0, 18);
      sprintf(output,MM_LI08);                   //Servo current
      u8g2.print(output);
      sprintf(output, "%4imA", current_mA);
      u8g2.setCursor(128 - 36, 18);
      u8g2.print(output);
    }
    u8g2.setCursor(0, 64);
    u8g2.print(MM_TO01);
    if (tare > 0) {
      int y = get_length(MM_LI09);
      u8g2.setCursor(128 - y * 6, 64);
      u8g2.print(MM_LI09);
    }
    u8g2.sendBuffer();
  #endif
  #if DISPLAY_TYPE == 3 or DISPLAY_TYPE == 99
    gfx->setFont(Punk_Mono_Bold_600_375);
    if (weight != weight_alt) {
      gfx->fillRect(80, 70, 200, 55, BACKGROUND);
      gfx->setFont(Punk_Mono_Bold_600_375);
      gfx->setCursor(100, 120);
      sprintf(output, "%5ig", weight);
      gfx->print(output);
      weight_alt = weight;
    }
    if (angle != angle_is_old) {
      gfx->fillRect(97, 198, 50, 20, BACKGROUND);
      gfx->setFont(Punk_Mono_Bold_200_125);
      gfx->setCursor(5, 215);
      gfx->print(MM_LI06);
      sprintf(output, "%3i°", angle);
      int y = get_length(output);
      gfx->setCursor(160 - y * 12 - 11, 215);
      gfx->print(output);
      angle_is_old = angle;
    }
    if (pos != pos_old) {
      gfx->fillRect(97, 221, 50, 20, BACKGROUND);
      gfx->setFont(Punk_Mono_Bold_200_125);
      gfx->setCursor(5, 238);
      gfx->print(MM_LI05);
      sprintf(output, "%3i°", angle_max*pos/100);
      int y = get_length(output);
      gfx->setCursor(160 - y * 12 - 11, 238);
      gfx->print(output);
      pos_old = pos;
    }
    if (tare != tare_old) {
      gfx->fillRect(230, 175, 90, 22, BACKGROUND);
      gfx->setFont(Punk_Mono_Bold_200_125);      
      sprintf(output, "%5ig", tare);
      int y = get_length(output);
      gfx->setCursor(320 - y * 12 - 5, 192);
      gfx->print(output);
      tare_old = tare;
    }
    if (ina219_installed == 1 and current_mA != current_mA_old and (current_servo > 0 or show_current == 1)) {
      gfx->fillRect(240, 221, 80, 20, BACKGROUND);
      gfx->setFont(Punk_Mono_Bold_200_125);
      if (current_mA > current_servo and current_servo > 0) {
        gfx->setTextColor(RED);
      }
      sprintf(output, "%4imA", current_mA);
      int y = get_length(output);
      gfx->setCursor(320 - y * 12 - 5, 238);
      gfx->print(output);
      gfx->setTextColor(TEXT);
      current_mA_old = current_mA;
    }
    if (servo_active != servo_active_old) {
      gfx->fillRect(17, 79, 38, 38, BACKGROUND);
      gfx->setFont(Icons_Start_Stop);
      gfx->setCursor(10, 123);
      gfx->print("M");
      gfx->setCursor(10, 123);
      if (servo_active == 1) {
        gfx->setTextColor(GREEN);
        gfx->print("A");
      }
      else {
        gfx->setTextColor(RED);
        gfx->print("B");
      }
      servo_active_old = servo_active;
      gfx->setTextColor(TEXT);
    }
    
  #endif
  if (alarm_overcurrent) {i = 1;}
  while (i > 0) {
    inawatchdog = 0;                    //turn off the continuous INA measurement
    //Servo is closed
    if (servo.read() <= angle_min  + offset_angle and offset_angle < 3) {
      while(offset_angle < 3 and current_servo < current_mA) {
        offset_angle = offset_angle + 1;
        SERVO_WRITE(angle_min + offset_angle);
        current_mA = GetCurrent(10);
        delay(1000);
      }
    }
    //Servo is open
    //else {
    //  servo_active = 0;
    //}
    i = 0;
    inawatchdog = 1;
    alarm_overcurrent = 0;
  }
  setPreferences();         //put somewhere else. not every change must be recorded
}

void setup() {
  // enable internal pull downs for digital inputs 
  pinMode(button_start_pin, INPUT_PULLDOWN);
  pinMode(button_stop_pin, INPUT_PULLDOWN);
  pinMode(switch_mode_pin, INPUT_PULLDOWN);
  pinMode(switch_setup_pin, INPUT_PULLDOWN);
  Serial.begin(115200);
  while (!Serial) {}
  #ifdef isDebug
    Serial.println("Hanimandl Start");
  #endif
  // Try to initialize the INA219
  if (ina219.begin()) {
    ina219_installed = 1;
    #ifdef isDebug
      Serial.println("INA219 chip found");
    #endif
  }
  else {
    current_servo = 0;                              // ignore INA when none is found
    #ifdef isDebug
      Serial.println("INA219 chip not found");
    #endif
  }
  // Rotary
  pinMode(outputSW, INPUT_PULLUP);
  attachInterrupt(outputSW, isr1, FALLING);
  pinMode(outputA,INPUT);
  pinMode(outputB,INPUT);
  attachInterrupt(outputA, isr2, CHANGE);
  // Buzzer
  pinMode(buzzer_pin, OUTPUT);
  pinMode(led_pin, OUTPUT);
  // short delay to let chip power up
  delay (100);
  // read preferences frm EEPROM
  getPreferences();
  // Servo initialization and closing
  #ifdef SERVO_EXPANDED
    servo.attach(servo_pin,  750, 2500); // extended initialization, does not control every servo
    servo.setPeriodHertz(100);
  #else
    servo.attach(servo_pin, 1000, 2000); // default values. Warning, drives the zero point less far!  
  #endif
  SERVO_WRITE(angle_min);
  // Scale identification we do this before the boot screen to let it warm up for 3 seconds
  scale.begin(hx711_dt_pin, hx711_sck_pin);
  if (scale.wait_ready_timeout(1000)) {               // Scale connected? :Roli - is always zero if no HX711 is connected
    scale.power_up();
    if (scale.read() != 0) {                          // Roli - if 0 I consider that no HX711 is connected
      scale_available = 1;
      #ifdef isDebug
        Serial.println("Scale found");
      #endif
    }
  }
  // Boot screen
  #if DISPLAY_TYPE == 1 or DISPLAY_TYPE == 2 or DISPLAY_TYPE == 99
    u8g2.setBusClock(800000);   // experimental
    u8g2.begin();
    u8g2.enableUTF8Print();
  #endif
  #if DISPLAY_TYPE == 3 or DISPLAY_TYPE == 99
    tft_colors();
    tft_marker();
    gfx->begin();
    gfx->fillScreen(BACKGROUND);
    gfx->setUTF8Print(true);
  #endif
  if (showlogo) {
    #if DISPLAY_TYPE == 1 or DISPLAY_TYPE == 2 or DISPLAY_TYPE == 99
      u8g2.clearBuffer();
    #endif
    print_logo();
    delay(3000);
  }
  if (showcredits) {
    buzzer(BUZZER_SHORT);
    print_credits();   
    delay(2500);
  }
  // Scale setup, setting scaling factor
  int x_pos;
  if (scale_available ==1) {                         // Scale connected?
    if (factor == 0) {                               // Present but uncalibrated
      #if DISPLAY_TYPE == 1 or DISPLAY_TYPE == 2 or DISPLAY_TYPE == 99
        u8g2.clearBuffer();
        //u8g2.setFont(u8g2_font_courB18_tf);
        u8g2.setFont(u8g2_font_courB12_tf);
        sprintf(output,ER_LI11);
        x_pos = CenterPosX(output, 10, 128);
        u8g2.setCursor(x_pos, 30); u8g2.print(output);
        sprintf(output,ER_LI12);
        x_pos = CenterPosX(output, 10, 128);
        u8g2.setCursor(x_pos, 46); u8g2.print(output);
        u8g2.sendBuffer();
      #endif
      #if DISPLAY_TYPE == 3 or DISPLAY_TYPE == 99
        gfx->fillScreen(BACKGROUND);
        gfx->setTextColor(RED);
        gfx->setFont(Punk_Mono_Bold_320_200);
        sprintf(output,ER_LI11);
        x_pos = CenterPosX(output, 18, 320);
        gfx->setCursor(x_pos, 110);
        gfx->print(output);
        sprintf(output,ER_LI12);
        x_pos = CenterPosX(output, 18, 320);
        gfx->setCursor(x_pos, 154);
        gfx->print(output);
      #endif
      buzzer(BUZZER_ERROR);
      #ifdef isDebug
        Serial.println("Scale not calibrated");
      #endif
      delay(2000);
    }
    else {                                          // Tare and scaling setup
      scale.set_scale(factor);
      scale.set_offset(long(weight_empty));
      #ifdef isDebug
        Serial.println("Scale initialized");
      #endif
    }
  }
  else {                                            // No scale connected
    #if DISPLAY_TYPE == 1 or DISPLAY_TYPE == 2 or DISPLAY_TYPE == 99
      u8g2.clearBuffer();
      u8g2.setFont(u8g2_font_courB24_tf);
      sprintf(output,ER_LI01);
      x_pos = CenterPosX(output, 20, 128);
      u8g2.setCursor( x_pos, 24); u8g2.print(output);
      sprintf(output,ER_LI02);
      x_pos = CenterPosX(output, 20, 128);
      u8g2.setCursor( x_pos, 56);  u8g2.print(output);
      u8g2.sendBuffer();
    #endif
    #if DISPLAY_TYPE == 3 or DISPLAY_TYPE == 99
      gfx->fillScreen(BACKGROUND);
      gfx->setTextColor(RED);
      gfx->setFont(Punk_Mono_Bold_320_200);
      sprintf(output,ER_LI01);
      x_pos = CenterPosX(output, 18, 320);
      gfx->setCursor(x_pos, 110);
      gfx->print(output);
      sprintf(output,ER_LI02);
      x_pos = CenterPosX(output, 18, 320);
      gfx->setCursor(x_pos, 154);
      gfx->print(output);
    #endif
    buzzer(BUZZER_ERROR);
    #ifdef isDebug
      Serial.println("No scale!");
    #endif
    delay(2000);
  }
  // initial calibration of empty weight because of temperature variations
  // If more than 20g deviation, there is probably something on the scale.
  if (scale_available == 1) {
    weight = SCALE_GETUNITS(SCALE_READS);
    if ((weight > -20) && (weight < 20)) {
      scale.tare(10);
      buzzer(BUZZER_SUCCESS);
      #ifdef isDebug
        Serial.print("Tare overshot by: ");
        Serial.println(weight);
      #endif
    }
    else if (factor != 0) {
      int x_pos;
      #if DISPLAY_TYPE == 1 or DISPLAY_TYPE == 2 or DISPLAY_TYPE == 99
        u8g2.clearBuffer();
        u8g2.setFont(u8g2_font_courB14_tf);
        sprintf(output,ER_LI21);
        x_pos = CenterPosX(output, 11, 128);
        u8g2.setCursor( x_pos, 28); u8g2.print(output);
        sprintf(output,ER_LI22);
        x_pos = CenterPosX(output, 11, 128);
        u8g2.setCursor(x_pos, 52); u8g2.print(output);
        u8g2.sendBuffer();
      #endif
      #if DISPLAY_TYPE == 3 or DISPLAY_TYPE == 99
        gfx->fillScreen(BACKGROUND);
        gfx->setTextColor(RED);
        gfx->setFont(Punk_Mono_Bold_320_200);
        sprintf(output,ER_LI21);
        x_pos = CenterPosX(output, 18, 320);
        gfx->setCursor(x_pos, 110);
        gfx->print(output);
        sprintf(output,ER_LI22);
        x_pos = CenterPosX(output, 18, 320);
        gfx->setCursor(x_pos, 154);
        gfx->print(output);
      #endif
      #ifdef isDebug
        Serial.print("Weight on the scale: ");
        Serial.println(weight);
      #endif
      delay(5000);
      // New try, in case the weight was taken off the scale
      weight = SCALE_GETUNITS(SCALE_READS);
      if ((weight > -20) && (weight < 20)) {
        scale.tare(10);
        buzzer(BUZZER_SUCCESS);
        #ifdef isDebug
          Serial.print("Tare overshot by: ");
          Serial.println(weight);
        #endif
      }
      else {    // Warning sound played
        buzzer(BUZZER_LONG);
      }
    }
  }
  // initialization of the 4 rotary datastructures
  initRotaries(SW_ANGLE,    0,   0, 100, 5);     // Angle
  initRotaries(SW_CORRECTION, 0, -90,  20, 1);     // Correction
  initRotaries(SW_FLOW,     0,   0,  99, 1);     // Flow
  initRotaries(SW_MENU,      0,   0,   7, 1);     // Menu choices
  // Setting up rotary parameters from the preferences
  setRotariesValue(SW_ANGLE,    pos);   
  setRotariesValue(SW_CORRECTION, correction);
  setRotariesValue(SW_FLOW,     intweight);
  setRotariesValue(SW_MENU,      fquantity_index);
}

void loop() {
  //only for test
  //Serial.println("OTA START");
  //OTASetup();
  //while (WiFi.status() == WL_CONNECTED) {
  //  server.handleClient();
  //}
  //END only for test
  rotating = true;     // debounce Management
  //INA219 measurement
  if (ina219_installed and inawatchdog == 1 and (current_servo > 0 or show_current == 1) and (mode == MODE_MANUAL or mode == MODE_AUTOMATIC)) {
    ina219_measurement();
  }
  // Setup Menu 
  if ((digitalRead(switch_setup_pin)) == HIGH) {
    processSetup();
  }
  // Automatic operation
  else if ((digitalRead(switch_mode_pin)) == HIGH) {
    processAutomatic();
  }
  // Manual operation
  else if ((digitalRead(switch_mode_pin) == LOW) && (digitalRead(switch_setup_pin) == LOW)) {
    processManualmode();
  }
}

void print_credits() {
  #if DISPLAY_TYPE == 1 or DISPLAY_TYPE == 2 or DISPLAY_TYPE == 99
    u8g2.clearBuffer();
    u8g2.setFont(u8g2_font_helvB08_tf);
    u8g2.setCursor(0, 10);    u8g2.print("Idee: M. Vasterling");
    u8g2.setCursor(0, 22);    u8g2.print("Code: M. Vasterling,");
    u8g2.setCursor(0, 32);    u8g2.print("M. Wetzel, C. Gruber,");
    u8g2.setCursor(0, 42);    u8g2.print("A. Holzhammer, M. Junker,");
    u8g2.setCursor(0, 52);    u8g2.print("J. Kuder, J. Bruker,");
    u8g2.setCursor(0, 62);    u8g2.print("R. Rust, A. Pfaff");
    u8g2.sendBuffer();
  #endif
  #if DISPLAY_TYPE == 3 or DISPLAY_TYPE == 99
    int offset = 22;
    gfx->fillScreen(BACKGROUND);
    gfx->setTextColor(TEXT);
    gfx->setFont(Punk_Mono_Bold_240_150);
    gfx->setCursor(10, 1*offset);
    gfx->print("Idee:");
    gfx->setCursor(90, 1*offset);
    gfx->print("M. Vasterling");
    gfx->setCursor(10, 2*offset+10);
    gfx->print("Code:");
    gfx->setCursor(90, 2*offset+10);
    gfx->print("M. Vasterling");
    gfx->setCursor(90, 3*offset+10);
    gfx->print("M. Wetzel");
    gfx->setCursor(90, 4*offset+10);
    gfx->print("C. Gruber");
    gfx->setCursor(90, 5*offset+10);
    gfx->print("A. Holzhammer");
    gfx->setCursor(90, 6*offset+10);
    gfx->print("M. Junker");
    gfx->setCursor(90, 7*offset+10);
    gfx->print("J. Kuder");
    gfx->setCursor(90, 8*offset+10);
    gfx->print("J. Bruker");
    gfx->setCursor(90, 9*offset+10);
    gfx->print("R. Rust");             //A.P.
    gfx->setCursor(90, 10*offset+10);
    gfx->print("A. Pfaff");            //A.P.
  #endif
}

#if USER == 2
  void print_logo() {
    #if DISPLAY_TYPE == 1 or DISPLAY_TYPE == 2 or DISPLAY_TYPE == 99
      u8g2.clearBuffer();
      u8g2.drawXBM(0,0,128,64,LogoGeroldOLED);
      u8g2.sendBuffer();
    #endif
    #if DISPLAY_TYPE == 3 or DISPLAY_TYPE == 99
    gfx->drawXBitmap(0, 0, LogoGeroldTFT, 320, 240, TEXT);
  #endif
  }
#elif USER == 3
void print_logo() {
  #if DISPLAY_TYPE == 1 or DISPLAY_TYPE == 2 or DISPLAY_TYPE == 99
    u8g2.clearBuffer();
    u8g2.drawXBM(0,0,128,64,LogoRoliOLED);
    u8g2.setFont(u8g2_font_courB14_tf);
    u8g2.setCursor(30, 27);    u8g2.print("HANI");
    u8g2.setCursor(40, 43);    u8g2.print("MANDL");
    u8g2.setFont(u8g2_font_courB08_tf);
    u8g2.setCursor(95, 64);    u8g2.print(version);
    u8g2.sendBuffer();
  #endif
  #if DISPLAY_TYPE == 3 or DISPLAY_TYPE == 99
    gfx->drawXBitmap(0, 0, LogoRoliTFT, 320, 240, YELLOW);
    gfx->setTextColor(YELLOW);
    gfx->setFont(Punk_Mono_Bold_200_125);
    gfx->setCursor(232, 220);
    gfx->print(version);
  #endif
}
#else
void print_logo() {
  #if DISPLAY_TYPE == 1 or DISPLAY_TYPE == 2 or DISPLAY_TYPE == 99
    u8g2.clearBuffer();
    u8g2.drawXBM(0,0,80,64,LogoBieneOLED);
    u8g2.setFont(u8g2_font_courB14_tf);
    u8g2.setCursor(85, 27);    u8g2.print("HANI");
    u8g2.setCursor(75, 43);    u8g2.print("MANDL");
    u8g2.setFont(u8g2_font_courB08_tf);
    u8g2.setCursor(77, 64);    u8g2.print(version);
    u8g2.sendBuffer();
    //u8g2.writeBufferXBM(Serial);	
  #endif
  #if DISPLAY_TYPE == 3 or DISPLAY_TYPE == 99
    gfx->fillScreen(BACKGROUND);
    gfx->drawXBitmap(60, 10, LogoBieneTFT, 200, 160, TEXT);
    gfx->setTextColor(TEXT);
    gfx->setFont(Punk_Mono_Bold_600_375);
    gfx->setCursor(0, 180);
    gfx->print("HANI");
    gfx->setCursor(140, 235);
    gfx->print("MANDL");
    gfx->setFont(Punk_Mono_Bold_200_125);
    gfx->setCursor(255, 180);
    gfx->print(version);
  #endif
}
#endif

// We use an active buzzer, which eliminates the need for the tone library, which interacts with the servo anyway.
void buzzer(byte type) {
  if (buzzermode == 1 or ledmode == 1) {
    switch (type) {
      case BUZZER_SHORT: //short
        if (buzzermode == 1) {digitalWrite(buzzer_pin,HIGH);}
        if (ledmode == 1) {digitalWrite(led_pin,HIGH);}
        if (buzzermode == 1 or ledmode == 1) {delay(100);}
        if (buzzermode == 1) {digitalWrite(buzzer_pin,LOW);}
        if (ledmode == 1) {digitalWrite(led_pin,LOW);}
        break;
      case BUZZER_LONG: //long
        if (buzzermode == 1) {digitalWrite(buzzer_pin,HIGH);}
        if (ledmode == 1) {digitalWrite(led_pin,HIGH);}
        if (buzzermode == 1 or ledmode == 1) {delay(500);}
        if (buzzermode == 1) {digitalWrite(buzzer_pin,LOW);}
        if (ledmode == 1) {digitalWrite(led_pin,LOW);}
        break;
      case BUZZER_SUCCESS: //success
        if (buzzermode == 1) {digitalWrite(buzzer_pin,HIGH);}
        if (ledmode == 1) {digitalWrite(led_pin,HIGH);}
        if (buzzermode == 1 or ledmode == 1) {delay(100);}
        if (buzzermode == 1) {digitalWrite(buzzer_pin,LOW);}
        if (ledmode == 1) {digitalWrite(led_pin,LOW);}
        if (buzzermode == 1 or ledmode == 1) {delay(100);}
        if (buzzermode == 1) {digitalWrite(buzzer_pin,HIGH);}
        if (ledmode == 1) {digitalWrite(led_pin,HIGH);}
        if (buzzermode == 1 or ledmode == 1) {delay(100);}
        if (buzzermode == 1) {digitalWrite(buzzer_pin,LOW);}
        if (ledmode == 1) {digitalWrite(led_pin,LOW);}
        if (buzzermode == 1 or ledmode == 1) {delay(100);}
        if (buzzermode == 1) {digitalWrite(buzzer_pin,HIGH);}
        if (ledmode == 1) {digitalWrite(led_pin,HIGH);}
        if (buzzermode == 1 or ledmode == 1) {delay(100);}
        if (buzzermode == 1) {digitalWrite(buzzer_pin,LOW);}
        if (ledmode == 1) {digitalWrite(led_pin,LOW);}
        break;
      case BUZZER_ERROR: //error
        if (buzzermode == 1) {digitalWrite(buzzer_pin,HIGH);}
        if (ledmode == 1) {digitalWrite(led_pin,HIGH);}
        if (buzzermode == 1 or ledmode == 1) {delay(1500);}
        if (buzzermode == 1) {digitalWrite(buzzer_pin,LOW);}
        if (ledmode == 1) {digitalWrite(led_pin,LOW);}
        break;
    }
  }
}

// Support functions for stepwise weight adjustement
int step2weight(int step) {
  int sum = 0;
  if ( step > 210 ) { sum += (step-210)*1000; step -= (step-210); }
  if ( step > 200 ) { sum += (step-200)* 500; step -= (step-200); }  
  if ( step > 160 ) { sum += (step-160)* 100; step -= (step-160); }
  if ( step > 140 ) { sum += (step-140)*  25; step -= (step-140); }
  if ( step >  50 ) { sum += (step- 50)*   5; step -= (step- 50); }
  sum += step;  
  return sum;
}

int weight2step(int sum) {
  int step = 0;
  if ( sum > 10000 ) { step += (sum-10000)/1000; sum -= ((sum-10000)); }
  if ( sum >  5000 ) { step += (sum-5000)/500;   sum -= ((sum-5000)); }
  if ( sum >  1000 ) { step += (sum-1000)/100;   sum -= ((sum-1000)); }
  if ( sum >   500 ) { step += (sum-500)/25;     sum -= ((sum-500));  }
  if ( sum >    50 ) { step += (sum-50)/5;       sum -= (sum-50);   }
  step += sum;
  return step;
}

//Sub function for Display
int StringLenght(String a) {
  a.trim();
  int res = a.length();
  return res;
}

int CenterPosX(const char a[], float font_width, int display_width) {
  int res = 0;
  int counter = get_length(a);
  res = int(float(display_width - font_width * counter) / 2);
  return res;
}

int get_length(const char a[]) {
  int counter = 0;
  for (int i = 0; i < strlen(a); i++) {
    if ((unsigned char) a[i] >= 0x00 and (unsigned char) a[i] <= 0x7F) {
      counter++;
    }
    else if ((unsigned char) a[i] >= 0x80 and (unsigned char) a[i] <= 0x07FF) {
      i += 1;
      counter++;
    }
    else if ((unsigned char) a[i] >= 0x0800 and (unsigned char) a[i] <= 0xFFFF) {
      i += 2;
      counter++;
    }
    else if ((unsigned char) a[i] >= 0x010000 and (unsigned char) a[i] <= 0x10FFFF) {
      i += 3;
      counter++;
    }
    else {
      i += 4;      //is no UTF8 character. Should never come up here
    }
  }
  return counter;
}

//Sub function for INA219
int GetCurrent(int count) {
  int res = 0;
  for (int x = 0; x < count; x++) {
    res = res + ina219.getCurrent_mA();
  }
  if (count > 0) {
    res = res / count;
  }
  return res;
}

void ina219_measurement() {
  if (current_mA > current_servo and current_servo > 0) {
    if (last_overcurrenttime == 0) {
      last_overcurrenttime = millis();
    }
    else if (millis() - last_overcurrenttime >= overcurrenttime) {
      last_overcurrenttime = 0;
      alarm_overcurrent = 1;
    }
  }
  else {
    last_overcurrenttime = 0;
  }
  if (millis() - last_ina219_measurement >= updatetime_ina219) {
    last_ina219_measurement = millis();
    current_mA = GetCurrent(10);
  }
}

//OTA
#if OTA == 1
  void onOTAStart() {
    int x_pos;
    #ifdef isDebug
      Serial.println("OTA update started!");
    #endif
    #if DISPLAY_TYPE == 1 or DISPLAY_TYPE == 2 or DISPLAY_TYPE == 99
      u8g2.clearBuffer();
      u8g2.setFont(u8g2_font_courB08_tf);
      sprintf(output, "Update started");
      x_pos = CenterPosX(output, 6, 128);
      u8g2.setCursor(x_pos, 35);
      u8g2.print(output);
      u8g2.sendBuffer();
    #endif
    #if DISPLAY_TYPE == 3 or DISPLAY_TYPE == 99
      gfx->fillScreen(BACKGROUND);
      gfx->setTextColor(TEXT);
      gfx->setFont(Punk_Mono_Bold_320_200);
      sprintf(output, "Update started");
      x_pos = CenterPosX(output, 18, 320);
      gfx->setCursor(x_pos, 27);
      gfx->print(output);
      gfx->drawLine(0, 37, 320, 37, TEXT);
    #endif
  }

  void onOTAProgress(size_t current, size_t final) {
    if (millis() - ota_progress_millis > 1000) {
      ota_progress_millis = millis();
      #if DISPLAY_TYPE == 1 or DISPLAY_TYPE == 2 or DISPLAY_TYPE == 99
        u8g2.clearBuffer();
        u8g2.setFont(u8g2_font_courB08_tf);
        u8g2.setCursor(1, 10);
        u8g2.print("OTA Progress:");
        u8g2.setCursor(1, 25);
        sprintf(output, "Current: %u", current);
        u8g2.print(output);
        u8g2.setCursor(1, 35);
        sprintf(output, "Final:   %u", final);
        u8g2.print(output);
        u8g2.sendBuffer();
      #endif
      #if DISPLAY_TYPE == 3 or DISPLAY_TYPE == 99
        gfx->fillRect(0, 38, 320, 202, BACKGROUND);
        gfx->setTextColor(TEXT);
        gfx->setFont(Punk_Mono_Bold_320_200);
        gfx->setCursor(10, 100);
        gfx->print("OTA Progress:");
        gfx->setCursor(10, 140);
        sprintf(output, "Current: %u", current);
        gfx->print(output);
        gfx->setCursor(10, 170);
        sprintf(output, "Final:   %u", final);
        gfx->print(output);
      #endif
      #ifdef isDebug
        Serial.printf("OTA Progress Current: %u bytes, Final: %u bytes\n", current, final);
      #endif
    }
  }

  void onOTAEnd(bool success) {
    int x_pos;
    if (success) {
      #if DISPLAY_TYPE == 1 or DISPLAY_TYPE == 2 or DISPLAY_TYPE == 99
        u8g2.clearBuffer();
        u8g2.setFont(u8g2_font_courB08_tf);
        sprintf(output, "Successfully");
        x_pos = CenterPosX(output, 6, 128);
        u8g2.setCursor(x_pos, 35);
        u8g2.print(output);
        u8g2.sendBuffer();
      #endif
      #if DISPLAY_TYPE == 3 or DISPLAY_TYPE == 99
        gfx->fillScreen(BACKGROUND);
        gfx->setTextColor(GREEN);
        gfx->setFont(Punk_Mono_Bold_320_200);
        sprintf(output, "Successfully");
        x_pos = CenterPosX(output, 18, 320);
        gfx->setCursor(x_pos, 150);
        gfx->print(output);
      #endif
      ota_done = 1;
      #ifdef isDebug
        Serial.println("OTA update finished successfully!");
      #endif
    } 
    else {
      #if DISPLAY_TYPE == 1 or DISPLAY_TYPE == 2 or DISPLAY_TYPE == 99
        u8g2.clearBuffer();
        u8g2.clearBuffer();
        u8g2.setFont(u8g2_font_courB08_tf);
        sprintf(output, "Error");
        x_pos = CenterPosX(output, 6, 128);
        u8g2.setCursor(x_pos, 35);
        u8g2.print(output);
        u8g2.sendBuffer();
      #endif
      #if DISPLAY_TYPE == 3 or DISPLAY_TYPE == 99
        gfx->fillScreen(BACKGROUND);
        gfx->setTextColor(RED);
        gfx->setFont(Punk_Mono_Bold_320_200);
        sprintf(output, "Error");
        x_pos = CenterPosX(output, 18, 320);
        gfx->setCursor(x_pos, 150);
        gfx->print(output);
      #endif
      #ifdef isDebug
        Serial.println("There was an error during OTA update!");
      #endif
    }
  }

  void OTAdisconnect(void) {
    #ifdef isDebug
      Serial.println("OTAdisconnect");
    #endif
    int x_pos;
    #if DISPLAY_TYPE == 1 or DISPLAY_TYPE == 2 or DISPLAY_TYPE == 99
      u8g2.clearBuffer();
      u8g2.setFont(u8g2_font_courB14_tf);
      sprintf(output, "Stop WiFi");
      x_pos = CenterPosX(output, 11, 128);
      u8g2.setCursor(x_pos, 35);
      u8g2.print(output);
      u8g2.sendBuffer();
    #endif
    #if DISPLAY_TYPE == 3 or DISPLAY_TYPE == 99
      gfx->fillRect(0, 38, 320, 202, BACKGROUND);
      gfx->setTextColor(TEXT);
      gfx->setFont(Punk_Mono_Bold_320_200);
      sprintf(output, "Stop WiFi");
      x_pos = CenterPosX(output, 18, 320);
      gfx->setCursor(x_pos, 150);
      gfx->print(output);
    #endif
    WiFi.disconnect();
    delay(500);
    while (WiFi.status() == WL_CONNECTED and digitalRead(switch_setup_pin) == HIGH) {delay(500);}
  }

  void OTALoop(void) {
    #ifdef isDebug
      Serial.println("OTALoop");
    #endif
    while (WiFi.status() == WL_CONNECTED and digitalRead(switch_setup_pin) == HIGH) {
      if (digitalRead(button_stop_pin) == HIGH) {
        while (digitalRead(button_stop_pin) == HIGH) {delay(100);}
        OTAdisconnect();
      }
      else {
        server.handleClient();
	if (ota_done == 1) {
          delay(5000);
          ESP.restart();
        }
      }
    }
    if (digitalRead(switch_setup_pin) == LOW) {
      OTAdisconnect();
    }
  }

  void OTASetup(void) {
    int x_pos;
    #ifdef isDebug
      Serial.println("OTASetup");
    #endif
    #if DISPLAY_TYPE == 3 or DISPLAY_TYPE == 99
      gfx->fillScreen(BACKGROUND);
      gfx->setTextColor(TEXT);
      gfx->setFont(Punk_Mono_Bold_320_200);
      sprintf(output, "ElegantOTA");
      x_pos = CenterPosX(output, 18, 320);
      gfx->setCursor(x_pos, 27);
      gfx->print(output);
      gfx->drawLine(0, 37, 320, 37, TEXT);
    #endif
    WiFi.mode(WIFI_STA);
    WiFi.begin(ssid, password);
    // Wait for connection
    char x[] = "";
    int i = 0;
    unsigned long wifi_start_millis = millis();
    while (WiFi.status() != WL_CONNECTED and millis() - wifi_start_millis < 15000 and digitalRead(switch_setup_pin) == HIGH) {
      #if DISPLAY_TYPE == 1 or DISPLAY_TYPE == 2 or DISPLAY_TYPE == 99
        u8g2.clearBuffer();
        u8g2.setFont(u8g2_font_courB14_tf);
        sprintf(output, "Start WiFi");
        x_pos = CenterPosX(output, 11, 128);
        u8g2.setCursor(x_pos, 25);
        u8g2.print(output);
      #endif
      if (i < 10) {
        sprintf(x, "%s*", x);
        i++;
      }
      else {
        i = 1;
        sprintf(x, "%s", "*");
      }
      #if DISPLAY_TYPE == 1 or DISPLAY_TYPE == 2 or DISPLAY_TYPE == 99
        x_pos = CenterPosX(x, 11, 128);
        u8g2.setCursor(x_pos, 45);
        u8g2.print(x);
        u8g2.sendBuffer();
      #endif
      #if DISPLAY_TYPE == 3 or DISPLAY_TYPE == 99
        gfx->setTextColor(TEXT);
        gfx->setFont(Punk_Mono_Bold_320_200);
        sprintf(output, "Start WiFi");
        x_pos = CenterPosX(output, 18, 320);
        gfx->setCursor(x_pos, 125);
        gfx->print(output);
        gfx->fillRect(0, 150, 320, 22, BACKGROUND);
        x_pos = CenterPosX(x, 18, 320);
        gfx->setCursor(x_pos, 170);
        gfx->print(x);
      #endif
      delay(500);
    }
    if (WiFi.status() == WL_CONNECTED and digitalRead(switch_setup_pin) == HIGH) {
      #if DISPLAY_TYPE == 1 or DISPLAY_TYPE == 2 or DISPLAY_TYPE == 99
        u8g2.clearBuffer();
        u8g2.setFont(u8g2_font_courB08_tf);
        sprintf(output, "%i.%i.%i.%i", WiFi.localIP()[0], WiFi.localIP()[1], WiFi.localIP()[2], WiFi.localIP()[3]);
        x_pos = CenterPosX(output, 6, 128);
        u8g2.setCursor(x_pos, 35);
        u8g2.print(output);
        u8g2.sendBuffer();
      #endif
      #if DISPLAY_TYPE == 3 or DISPLAY_TYPE == 99
        gfx->fillRect(0, 38, 320, 202, BACKGROUND);
        gfx->setTextColor(TEXT);
        gfx->setFont(Punk_Mono_Bold_320_200);
        sprintf(output, "%i.%i.%i.%i", WiFi.localIP()[0], WiFi.localIP()[1], WiFi.localIP()[2], WiFi.localIP()[3]);
        x_pos = CenterPosX(output, 18, 320);
        gfx->setCursor(x_pos, 150);
        gfx->print(output);
      #endif
      server.on("/", []() {
        sprintf(output, "HaniMandl %s", version);
        server.send(200, "text/plain", output);
      });
      #ifdef isDebug
        Serial.print("IP adress: "); Serial.println(WiFi.localIP());
      #endif
      ElegantOTA.begin(&server);
      ElegantOTA.onStart(onOTAStart);
      ElegantOTA.onProgress(onOTAProgress);
      ElegantOTA.onEnd(onOTAEnd);
      server.begin();
    }
    if (WiFi.status() == WL_CONNECTED and digitalRead(switch_setup_pin) == HIGH) {
      OTALoop();
    }
    else {
      #if DISPLAY_TYPE == 1 or DISPLAY_TYPE == 2 or DISPLAY_TYPE == 99
        u8g2.clearBuffer();
        u8g2.setFont(u8g2_font_courB14_tf);
        sprintf(output, "Start WiFi");
        x_pos = CenterPosX(output, 11, 128);
        u8g2.setCursor(x_pos, 25);
        u8g2.print(output);
        sprintf(output, "failed");
        x_pos = CenterPosX(output, 11, 128);
        u8g2.setCursor(x_pos, 45);
        u8g2.print(output);
        u8g2.sendBuffer();
      #endif
      #if DISPLAY_TYPE == 3 or DISPLAY_TYPE == 99
        gfx->fillRect(0, 150, 320, 22, BACKGROUND);
        gfx->setTextColor(RED);
        gfx->setFont(Punk_Mono_Bold_320_200);
        sprintf(output, "failed");
        x_pos = CenterPosX(output, 18, 320);
        gfx->setCursor(x_pos, 170);
        gfx->print(output);
        gfx->setTextColor(TEXT);
      #endif
      #ifdef isDebug
        Serial.print("IP adress: "); Serial.println(WiFi.localIP());
      #endif
    }
    delay(3000);
  }
#endif
